﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using System.Runtime.InteropServices;
using AcknexWrapper.Native;

namespace AcknexWrapper
{
    public class CONTACT
    {
        private unsafe NativeCONTACT* CONTACTPointer;

        /// <summary>
        /// constructor that initializes the object
        /// </summary>
        public CONTACT(IntPtr param0)
        {
            unsafe
            {
                CONTACTPointer = (NativeCONTACT*)param0;
            }
        }

        /// <summary>
        /// v - see Gamestudio manual
        /// </summary>
        public D3DVERTEX v
        {
            get
            {
                unsafe
                {
                    return (D3DVERTEX)(IntPtr)(*CONTACTPointer).v;
                }
            }
            set
            {
                unsafe
                {
                    (*CONTACTPointer).v = (NativeD3DVERTEX*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// x - see Gamestudio manual
        /// </summary>
        private Var x_ = null;
        public Var x
        {
            get
            {
                unsafe
                {
                    if (x_ == null)
                    {
                        x_ = (Var)(IntPtr)(&((*CONTACTPointer).x));
                    }
                }
                return x_;
            }
        }

        /// <summary>
        /// y - see Gamestudio manual
        /// </summary>
        private Var y_ = null;
        public Var y
        {
            get
            {
                unsafe
                {
                    if (y_ == null)
                    {
                        y_ = (Var)(IntPtr)(&((*CONTACTPointer).y));
                    }
                }
                return y_;
            }
        }

        /// <summary>
        /// z - see Gamestudio manual
        /// </summary>
        private Var z_ = null;
        public Var z
        {
            get
            {
                unsafe
                {
                    if (z_ == null)
                    {
                        z_ = (Var)(IntPtr)(&((*CONTACTPointer).z));
                    }
                }
                return z_;
            }
        }

        /// <summary>
        /// xyz - not listed in the Gamestudio manual - allows to access x, y and z as a Vector
        /// </summary>
        private Vector xyz_ = null;
        public Vector xyz
        {
            get
            {
                unsafe
                {
                    if (xyz_ == null)
                    {
                        xyz_ = new Vector((Var)0, (Var)0, (Var)0);
                        xyz_.x = (Var)(IntPtr)(&((*CONTACTPointer).x));
                        xyz_.y = (Var)(IntPtr)(&((*CONTACTPointer).y));
                        xyz_.z = (Var)(IntPtr)(&((*CONTACTPointer).z));
                    }
                }
                return xyz_;
            }
        }

        /// <summary>
        /// nx - see Gamestudio manual
        /// </summary>
        private Var nx_ = null;
        public Var nx
        {
            get
            {
                unsafe
                {
                    if (nx_ == null)
                    {
                        nx_ = (Var)(IntPtr)(&((*CONTACTPointer).nx));
                    }
                }
                return nx_;
            }
        }

        /// <summary>
        /// ny - see Gamestudio manual
        /// </summary>
        private Var ny_ = null;
        public Var ny
        {
            get
            {
                unsafe
                {
                    if (ny_ == null)
                    {
                        ny_ = (Var)(IntPtr)(&((*CONTACTPointer).ny));
                    }
                }
                return ny_;
            }
        }

        /// <summary>
        /// nz - see Gamestudio manual
        /// </summary>
        private Var nz_ = null;
        public Var nz
        {
            get
            {
                unsafe
                {
                    if (nz_ == null)
                    {
                        nz_ = (Var)(IntPtr)(&((*CONTACTPointer).nz));
                    }
                }
                return nz_;
            }
        }

        /// <summary>
        /// nxnynz - not listed in the Gamestudio manual - allows to access x, y and z as a Vector
        /// </summary>
        private Vector nxnynz_ = null;
        public Vector nxnynz
        {
            get
            {
                unsafe
                {
                    if (nxnynz_ == null)
                    {
                        nxnynz_ = new Vector((Var)0, (Var)0, (Var)0);
                        nxnynz_.x = (Var)(IntPtr)(&((*CONTACTPointer).nx));
                        nxnynz_.y = (Var)(IntPtr)(&((*CONTACTPointer).ny));
                        nxnynz_.z = (Var)(IntPtr)(&((*CONTACTPointer).nz));
                    }
                }
                return nxnynz_;
            }
        }

        /// <summary>
        /// u1 - see Gamestudio manual
        /// </summary>
        private Var u1_ = null;
        public Var u1
        {
            get
            {
                unsafe
                {
                    if (u1_ == null)
                    {
                        u1_ = (Var)(IntPtr)(&((*CONTACTPointer).u1));
                    }
                }
                return u1_;
            }
        }

        /// <summary>
        /// v1 - see Gamestudio manual
        /// </summary>
        private Var v1_ = null;
        public Var v1
        {
            get
            {
                unsafe
                {
                    if (v1_ == null)
                    {
                        v1_ = (Var)(IntPtr)(&((*CONTACTPointer).v1));
                    }
                }
                return v1_;
            }
        }

        /// <summary>
        /// u2 - see Gamestudio manual
        /// </summary>
        private Var u2_ = null;
        public Var u2
        {
            get
            {
                unsafe
                {
                    if (u2_ == null)
                    {
                        u2_ = (Var)(IntPtr)(&((*CONTACTPointer).u2));
                    }
                }
                return u2_;
            }
        }

        /// <summary>
        /// v2 - see Gamestudio manual
        /// </summary>
        private Var v2_ = null;
        public Var v2
        {
            get
            {
                unsafe
                {
                    if (v2_ == null)
                    {
                        v2_ = (Var)(IntPtr)(&((*CONTACTPointer).v2));
                    }
                }
                return v2_;
            }
        }

        /// <summary>
        /// red - see Gamestudio manual
        /// </summary>
        private Var red_ = null;
        public Var red
        {
            get
            {
                unsafe
                {
                    if (red_ == null)
                    {
                        red_ = (Var)(IntPtr)(&((*CONTACTPointer).red));
                    }
                }
                return red_;
            }
        }

        /// <summary>
        /// green - see Gamestudio manual
        /// </summary>
        private Var green_ = null;
        public Var green
        {
            get
            {
                unsafe
                {
                    if (green_ == null)
                    {
                        green_ = (Var)(IntPtr)(&((*CONTACTPointer).green));
                    }
                }
                return green_;
            }
        }

        /// <summary>
        /// blue - see Gamestudio manual
        /// </summary>
        private Var blue_ = null;
        public Var blue
        {
            get
            {
                unsafe
                {
                    if (blue_ == null)
                    {
                        blue_ = (Var)(IntPtr)(&((*CONTACTPointer).blue));
                    }
                }
                return blue_;
            }
        }

        /// <summary>
        /// greenBlueRed - not listed in the Gamestudio manual - allows to access scale_x, scale_y and scale_z as a Vector
        /// </summary>
        private Color greenBlueRed_ = null;
        public Color greenBlueRed
        {
            get
            {
                unsafe
                {
                    if (greenBlueRed_ == null)
                    {
                        greenBlueRed_ = new Color((Var)0, (Var)0, (Var)0);
                        greenBlueRed_.red = (Var)(IntPtr)(&((*CONTACTPointer).red));
                        greenBlueRed_.green = (Var)(IntPtr)(&((*CONTACTPointer).green));
                        greenBlueRed_.blue = (Var)(IntPtr)(&((*CONTACTPointer).blue));
                    }
                }
                return greenBlueRed_;
            }
        }

        /// <summary>
        /// vertex - see Gamestudio manual
        /// </summary>
        private Var vertex_ = null;
        public Var vertex
        {
            get
            {
                unsafe
                {
                    if (vertex_ == null)
                    {
                        vertex_ = (Var)(IntPtr)(&((*CONTACTPointer).vertex));
                    }
                }
                return vertex_;
            }
        }

        /// <summary>
        /// triangle - see Gamestudio manual
        /// </summary>
        private Var triangle_ = null;
        public Var triangle
        {
            get
            {
                unsafe
                {
                    if (triangle_ == null)
                    {
                        triangle_ = (Var)(IntPtr)(&((*CONTACTPointer).triangle));
                    }
                }
                return triangle_;
            }
        }

        /// <summary>
        /// chunk - see Gamestudio manual
        /// </summary>
        private Var chunk_ = null;
        public Var chunk
        {
            get
            {
                unsafe
                {
                    if (chunk_ == null)
                    {
                        chunk_ = (Var)(IntPtr)(&((*CONTACTPointer).chunk));
                    }
                }
                return chunk_;
            }
        }

        /// <summary>
        /// hit flags
        /// </summary>
        enum HIT : int
        {
            HIT_BLOCK = 1 << 9,
            HIT_ENTITY = 1 << 10
        }

        /// <summary>
        /// hit_block - see Gamestudio manual
        /// </summary>
        public Boolean HIT_BLOCK
        {
            get
            {
                unsafe
                {
                    if (((*CONTACTPointer).flags & (int)HIT.HIT_BLOCK) == 0)
                        return false;
                    else
                        return true;
                }
            }
        }

        /// <summary>
        /// hit_entity - see Gamestudio manual
        /// </summary>
        public Boolean HIT_ENTITY
        {
            get
            {
                unsafe
                {
                    if (((*CONTACTPointer).flags & (int)HIT.HIT_ENTITY) == 0)
                        return false;
                    else
                        return true;
                }
            }
        }

        /// <summary>
        /// hit_target - see Gamestudio manual
        /// </summary>
        public Boolean HIT_TARGET
        {
            get
            {
                unsafe
                {
                    if (HIT_ENTITY | HIT_BLOCK)
                        return true;
                    else
                        return false;
                }
            }
        }

        /// <summary>
        /// entity - see Gamestudio manual
        /// </summary>
        public ENTITY entity
        {
            get
            {
                unsafe
                {
                    return (ENTITY)(IntPtr)(*CONTACTPointer).entity;
                }
            }
            set
            {
                unsafe
                {
                    (*CONTACTPointer).entity = (NativeENTITY*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// skin1 - see Gamestudio manual
        /// </summary>
        public BMAP skin1
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*CONTACTPointer).skin1;
                }
            }
            set
            {
                unsafe
                {
                    (*CONTACTPointer).skin1 = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// skin2 - see Gamestudio manual
        /// </summary>
        public BMAP skin2
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*CONTACTPointer).skin2;
                }
            }
            set
            {
                unsafe
                {
                    (*CONTACTPointer).skin2 = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// texname - see Gamestudio manual
        /// </summary>
        public String texname
        {
            get
            {
                unsafe
                {
                    return Marshal.PtrToStringAnsi((IntPtr)(*CONTACTPointer).texname);
                }
            }
            set
            {
                unsafe
                {
                    (*CONTACTPointer).texname = (char*)(void*)Marshal.StringToHGlobalAnsi(value);
                }
            }
        }

        /// <summary>
        /// subset - see Gamestudio manual
        /// </summary>
        private Var subset_ = null;
        public Var subset
        {
            get
            {
                unsafe
                {
                    if (subset_ == null)
                    {
                        subset_ = (Var)(IntPtr)(&((*CONTACTPointer).subset));
                    }
                }
                return subset_;
            }
        }

        /* -------- CONTACT -------- */
        /* CONTACT to NativeCONTACT* (address in void*) */
        public static explicit operator IntPtr(CONTACT d)
        {
            unsafe
            {
                if (d == null)
                {
                    return IntPtr.Zero;
                }
                else
                {
                    return (IntPtr)(NativeCONTACT*)d.CONTACTPointer;
                }
            }
        }
        /* NativeCONTACT* (address in void*) to CONTACT */
        public static explicit operator CONTACT(IntPtr d)
        {
            unsafe
            {
                if (d == IntPtr.Zero)
                {
                    return null;
                }
                return new CONTACT(d);
            }
        }
    }
}