﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

namespace AcknexWrapper
{
    /// <summary>
    /// this class contains ALL flags
    /// </summary>
    public class Flags
    {
        public const int FLAG1 = (1 << 0);	// flag1..flag8 for C-Script use
        public const int FLAG2 = (1 << 1);
        public const int FLAG3 = (1 << 2);
        public const int FLAG4 = (1 << 3);
        public const int FLAG5 = (1 << 4);
        public const int FLAG6 = (1 << 5);
        public const int FLAG7 = (1 << 6);
        public const int FLAG8 = (1 << 7);

        public const int INVISIBLE = (1 << 8);
        public const int PASSABLE = (1 << 9);
        public const int TRANSLUCENT = (1 << 10);	// transparent
        public const int OVERLAY = (1 << 12);	// for models and panels
        public const int SPOTLIGHT = (1 << 13);
        public const int ZNEAR = (1 << 14);
        public const int NOFILTER = (1 << 16); // point filtering
        public const int UNLIT = (1 << 17);	// no light from environment
        public const int SHADOW = (1 << 18);	// cast dynamic shadows
        public const int LIGHT = (1 << 19);	// tinted by own light color
        public const int NOFOG = (1 << 20);	// ignores fog
        public const int BRIGHT = (1 << 21);	// additive blending
        public const int DECAL = (1 << 22);	// sprite without backside
        public const int METAL = (1 << 22);	// use metal material
        public const int CAST = (1 << 23);	// don't receive shadows
        public const int POLYGON = (1 << 26);	// polygonal collision detection

        // entity nosend flags
        public const int NOSEND_TYPE = (1 << 0);
        public const int NOSEND_SCALE = (1 << 1);
        public const int NOSEND_ORIGIN = (1 << 2);
        public const int NOSEND_ANGLES = (1 << 3);
        public const int NOSEND_FRAME = (1 << 4);
        public const int NOSEND_SKIN = (1 << 5);
        public const int NOSEND_FLAGS = (1 << 6);
        public const int NOSEND_AMBIENT = (1 << 7);
        public const int NOSEND_ALPHA = (1 << 8);
        public const int NOSEND_LIGHT = (1 << 9);
        public const int NOSEND_COLOR = (1 << 10);
        public const int NOSEND_UV = (1 << 11);
        public const int NOSEND_SOUND = (1 << 12);
        public const int NOSEND_ATTACH = (1 << 13);
        public const int NOSEND_Z = (1 << 14); // don't send z positions
        public const int NOSEND = (1 << 15);	// send nothing
        public const int SEND_NOW = (1 << 20);	// send entity update immediately

        // entity/sky flags2
        public const int UNTOUCHABLE = (1 << 0);	// don't react on mouse clicks
        public const int SHOW = (1 << 14); // view/sky entity visible
        public const int SKY = (1 << 17);	// sky entity
        public const int DOME = (1 << 18);	// half dome
        public const int SCENE = (1 << 19);	// don't use the z buffer
        public const int CUBE = (1 << 20);	// six sided cube
        public const int CYLINDER = (1 << 21);	// 360 degrees cylinder

        // entity eflags
        public const int FAT = (1 << 0);	// use fat hull
        public const int NARROW = (1 << 1);	// use narrow hull
        public const int PASS_TRANSP = (1 << 7);	// render on the transparent pass (if not overlay)
        public const int PASS_BOTH = (1 << 8);	// render on the transparent and solid passes
        public const int LOD0 = (1 << 16);	// LOD step 0..3, together with LOD1
        public const int LOD1 = (1 << 17);	// 
        public const int CLIP0 = (1 << 18);	// LOD offset (also for material)
        public const int CLIP1 = (1 << 19);	// 
        public const int CLIPPED = (1 << 21);	// entity was clipped away in the last frame
        public const int ANIMATE = (1 << 24);	// entity (auto animation)
        public const int LOCAL = (1 << 25);	// local entity

        // particle effect flags when different from entity flags
        public const int STREAK = (1 << 8);
        public const int BEAM = (1 << 9);
        public const int MOVE = (1 << 11);

        //event definitions
        public const int EVENT_BLOCK = 1;	// collision with level block
        public const int EVENT_ENTITY = 2;	// collisions with entity
        public const int EVENT_STUCK = 3;	// stuck in a corner
        public const int EVENT_PUSH = 4;
        public const int EVENT_IMPACT = 5;	// hit by another entity
        public const int EVENT_FRICTION = 6;
        public const int EVENT_DETECT = 7;
        public const int EVENT_SCAN = 8;
        public const int EVENT_SHOOT = 9;
        public const int EVENT_TRIGGER = 10;
        public const int EVENT_SONAR = 11;
        public const int EVENT_FRAME = 12;

        public const int EVENT_RECEIVE = 13;	// Entity received skills
        public const int EVENT_DISCONNECT = 20;	// Entity client has disconnected

        public const int EVENT_CLICKUP = 14;	// left click over down toggle
        public const int EVENT_BUTTONUP = 15;	// mouse button released over entity
        public const int EVENT_TOUCH = 16;	// mouse pointer moves over entity
        public const int EVENT_RELEASE = 17;	// mouse pointer leaves entity
        public const int EVENT_CLICK = 18;	// left click over entity
        public const int EVENT_RIGHTCLICK = 19;	// right click over entity

        public const int EVENT_JOIN = 32;	// client joined
        public const int EVENT_LEAVE = 33;	// client left
        public const int EVENT_STRING = 34;	// received a string
        public const int EVENT_DATA = 36;	// received a data buffer (7.04+)
        public const int EVENT_VAR = 37;	// received a variable

        // Entity event flags (emask)
        public const int DYNAMIC = (1 << 0);
        public const int ENABLE_BLOCK = (1 << EVENT_BLOCK);
        public const int ENABLE_ENTITY = (1 << EVENT_ENTITY);
        public const int ENABLE_STUCK = (1 << EVENT_STUCK);
        public const int ENABLE_PUSH = (1 << EVENT_PUSH);
        public const int ENABLE_IMPACT = (1 << EVENT_IMPACT);
        public const int ENABLE_FRICTION = (1 << EVENT_FRICTION);
        public const int ENABLE_DETECT = (1 << EVENT_DETECT);
        public const int ENABLE_SCAN = (1 << EVENT_SCAN);
        public const int ENABLE_SHOOT = (1 << EVENT_SHOOT);
        public const int ENABLE_TRIGGER = (1 << EVENT_TRIGGER);
        public const int ENABLE_SONAR = (1 << EVENT_SONAR);

        public const int ENABLE_FRAME = (1 << EVENT_FRAME);
        public const int ENABLE_RECEIVE = (1 << EVENT_RECEIVE);
        public const int ENABLE_DISCONNECT = (1 << EVENT_DISCONNECT);

        public const int ENABLE_BUTTONUP = (1 << EVENT_BUTTONUP);
        public const int ENABLE_TOUCH = (1 << EVENT_TOUCH);
        public const int ENABLE_RELEASE = (1 << EVENT_RELEASE);
        public const int ENABLE_CLICK = (1 << EVENT_CLICK);
        public const int ENABLE_RIGHTCLICK = (1 << EVENT_RIGHTCLICK);

        // c_trace / c_move mode flags
        public const int IGNORE_ME = (1 << 0);
        public const int IGNORE_YOU = (1 << 1);
        public const int IGNORE_PASSABLE = (1 << 2);
        public const int IGNORE_PASSENTS = (1 << 3);
        public const int IGNORE_MODELS = (1 << 4);
        public const int IGNORE_SPRITES = (1 << 5);
        public const int IGNORE_CONTENT = (1 << 6);
        public const int IGNORE_MAPS = (1 << 7);
        public const int IGNORE_WORLD = (1 << 8);
        public const int IGNORE_PUSH = (1 << 9);
        public const int ACTIVATE_SHOOT = (1 << 10);
        public const int ACTIVATE_SONAR = (1 << 11);
        public const int ACTIVATE_TRIGGER = (1 << 12);
        public const int USE_BOX = (1 << 13);
        public const int GET_HITVERTEX = (1 << 14);
        public const int ACTIVATE_PUSH = (1 << 14);
        public const int USE_AXIS = (1 << 15);
        public const int USE_AXISR = (1 << 16);
        public const int SCAN_TEXTURE = (1 << 16);
        public const int USE_AABB = (1 << 17);
        public const int GLIDE = (1 << 18);
        public const int USE_POLYGON = (1 << 19);
        public const int IGNORE_FLAG2 = (1 << 21);

        // scan mode flags
        public const int SCAN_ENTS = (0 << 14);
        public const int SCAN_PATHS = (2 << 14);
        public const int SCAN_NODES = (3 << 14);
        public const int SCAN_POS = (4 << 14);
        public const int SCAN_LIGHTS = (5 << 14);
        public const int SCAN_LIMIT = (1 << 18);
        public const int SCAN_FLAG2 = (1 << 20);

        // save mode flags
        public const int SV_VARS = (1 << 0);
        public const int SV_INFO = (1 << 1);
        public const int SV_STRINGS = (1 << 2);
        public const int SV_POINTERS = (1 << 3);
        public const int SV_BMAPS = (1 << 4);
        public const int SV_FUNCTIONS = (1 << 5);
        public const int SV_KEYS = (1 << 6);
        public const int SV_LEVEL = (1 << 7);
        public const int SV_SKY = (1 << 8);
        public const int SV_VIEWS = (1 << 9);
        public const int SV_PANELS = (1 << 10);
        public const int SV_SYS = (1 << 12);
        public const int SV_MATERIAL = (1 << 13);
        public const int SV_PATH = (1 << 14);
        public const int SV_STRUCT = (1 << 15);
        public const int SV_PHYSICS = (1 << 17);
        public const int SV_ALL = 0x000FFFFF;

        // MATERIAL flags
        public const int PASS_SOLID = (1 << 11); // enforce solid pass
        public const int TANGENT = (1 << 16);
        public const int ENABLE_RENDER = (1 << 17);	// call event before object rendering
        public const int ENABLE_VIEW = (1 << 18);	// call event before view rendering
        public const int ENABLE_TREE = (1 << 19);	// call event before entity sorting
        public const int OVERRIDE = (1 << 22);	// override view material
        public const int AUTORELOAD = (1 << 24);	// reload effects when changed

        // BMAP flags
        public const int BMPF_ARGB = (1 << 8);	// bitmap contains alpha channel
        public const int BMPF_SAVE = (1 << 9);	// bitmap was modified
        public const int BMPF_MIPMAPS = (1 << 10); // bmap->pixels contains 3 mipmaps

        // STRING flags
        public const int STRF_SAVE = (1 << 0);  // save the string at game_save()
        public const int STRF_CONST = (1 << 2);  // don't deallocate STRING->chars

        // TEXT, VIEW & PANEL flags
        public const int WWRAP = (1 << 6);  // word wrap TTF text
        public const int ARIGHT = (1 << 7);  // align text right
        public const int CENTER_X = (1 << 8);  // center text horizontally
        public const int CENTER_Y = (1 << 9);  // center bitmap text vertically
        public const int CONDENSED = (1 << 13); // draw bitmap text condensed
        public const int VISIBLE = SHOW;    // panel / view / text visible
        public const int OUTLINE = (1 << 15);	// draw text outlined
        public const int FILTER = (1 << 20); // text / panel linear filtering

        // VIEW flags
        public const int PROCESS_TARGET = (1 << 3);	// render previous render_target only
        public const int PROCESS_SCREEN = (1 << 4);	// don't delete the screen buffer
        public const int PROCESS_STENCIL = (1 << 5);	// use view material for rendering render_stencil
        public const int ISOMETRIC = (1 << 7);  // use ortogonal projection
        public const int PORTALCLIP = (1 << 8);  // use view clipping plane
        public const int CHILD = (1 << 11); // get coordinates from parent view
        public const int NOSHADER = (1 << 12); // suppress shaders in view
        public const int NOLOD = (1 << 13); // don't change previously set LOD
        public const int AUDIBLE = (1 << 15); // view has sound focus
        public const int WIREFRAME = (1 << 16);	// view wireframe
        public const int NOCULL = (1 << 19); // view without BSP culling
        public const int NOCAST = (1 << 20); // ignore CAST entities
        public const int NOFLAG1 = (1 << 21); // ignore flag1 entities
        public const int NOSHADOW = (1 << 22); // suppress shadows
        public const int NOSKY = (1 << 23); // don't clear frame buffer and don't draw sky
        public const int NOPARTICLE = (1 << 24); // suppress particles
        public const int CULL_CW = (1 << 27); // view with reversed culling

        // connection
        public const int CONNECT_SERVER = 1;
        public const int CONNECT_CLIENT = 2;

        // send
        public const int SEND_VEC = 1;
        public const int SEND_ALL = 2;
        public const int SEND_RATE = 4;
        public const int SEND_UNRELIABLE = 8;

        // animate
        public const int ANM_CYCLE = (1 << 0);
        public const int ANM_ADD = (1 << 1);
        public const int ANM_SKIP = (1 << 2);

        // physics
        public const int PH_RIGID = 1;		// Constant for setting up physics body
        public const int PH_WAVE = 2;		// Constant for setting up water body
        public const int PH_BOX = 0;
        public const int PH_SPHERE = 1;
        public const int PH_CYLINDER = 2;
        public const int PH_POLY = 3;
        public const int PH_BALL = 1;
        public const int PH_HINGE = 2;
        public const int PH_SLIDER = 3;
        public const int PH_WHEEL = 4;

        // run_mode
        public const int RUN_SCRIPT = 1;
        public const int RUN_LEVEL = 2;
        public const int RUN_ENT = 3;
        public const int RUN_SKY = 4;
        public const int RUN_MEDIA = 5;
    }
}
