﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using System.Runtime.InteropServices;
using AcknexWrapper.Native;
using System.Windows.Forms;

namespace AcknexWrapper
{
    public class MATERIAL
    {
        private unsafe NativeMATERIAL* MATERIALPointer;

        /// <summary>
        /// constructor that initializes the object
        /// </summary>
        public MATERIAL(IntPtr param0)
        {
            unsafe
            {
                MATERIALPointer = (NativeMATERIAL*)param0;
            }
        }

        /// <summary>
        /// ambient_blue - see Gamestudio manual
        /// </summary>
        private Var ambient_blue_ = null;
        public Var ambient_blue
        {
            get
            {
                unsafe
                {
                    if (ambient_blue_ == null)
                    {
                        ambient_blue_ = (Var)(IntPtr)(&((*MATERIALPointer).ambient_blue));
                    }
                }
                return ambient_blue_;
            }
        }

        /// <summary>
        /// ambient_green - see Gamestudio manual
        /// </summary>
        private Var ambient_green_ = null;
        public Var ambient_green
        {
            get
            {
                unsafe
                {
                    if (ambient_green_ == null)
                    {
                        ambient_green_ = (Var)(IntPtr)(&((*MATERIALPointer).ambient_green));
                    }
                }
                return ambient_green_;
            }
        }

        /// <summary>
        /// ambient_red - see Gamestudio manual
        /// </summary>
        private Var ambient_red_ = null;
        public Var ambient_red
        {
            get
            {
                unsafe
                {
                    if (ambient_red_ == null)
                    {
                        ambient_red_ = (Var)(IntPtr)(&((*MATERIALPointer).ambient_red));
                    }
                }
                return ambient_red_;
            }
        }

        /// <summary>
        /// ambient_blueGreenRed - not listed in the Gamestudio manual - allows to access ambient_x, ambient_y and ambient_z as a Vector
        /// </summary>
        private Color ambient_blueGreenRed_ = null;
        public Color ambient_blueGreenRed
        {
            get
            {
                unsafe
                {
                    if (ambient_blueGreenRed_ == null)
                    {
                        ambient_blueGreenRed_ = new Color((Var)0, (Var)0, (Var)0);
                        ambient_blueGreenRed_.red = (Var)(IntPtr)(&((*MATERIALPointer).ambient_red));
                        ambient_blueGreenRed_.green = (Var)(IntPtr)(&((*MATERIALPointer).ambient_green));
                        ambient_blueGreenRed_.blue = (Var)(IntPtr)(&((*MATERIALPointer).ambient_blue));
                    }
                }
                return ambient_blueGreenRed_;
            }
        }

        /// <summary>
        /// diffuse_blue - see Gamestudio manual
        /// </summary>
        private Var diffuse_blue_ = null;
        public Var diffuse_blue
        {
            get
            {
                unsafe
                {
                    if (diffuse_blue_ == null)
                    {
                        diffuse_blue_ = (Var)(IntPtr)(&((*MATERIALPointer).diffuse_blue));
                    }
                }
                return diffuse_blue_;
            }
        }

        /// <summary>
        /// diffuse_green - see Gamestudio manual
        /// </summary>
        private Var diffuse_green_ = null;
        public Var diffuse_green
        {
            get
            {
                unsafe
                {
                    if (diffuse_green_ == null)
                    {
                        diffuse_green_ = (Var)(IntPtr)(&((*MATERIALPointer).diffuse_green));
                    }
                }
                return diffuse_green_;
            }
        }

        /// <summary>
        /// diffuse_red - see Gamestudio manual
        /// </summary>
        private Var diffuse_red_ = null;
        public Var diffuse_red
        {
            get
            {
                unsafe
                {
                    if (diffuse_red_ == null)
                    {
                        diffuse_red_ = (Var)(IntPtr)(&((*MATERIALPointer).diffuse_red));
                    }
                }
                return diffuse_red_;
            }
        }

        /// <summary>
        /// diffuse_blueGreenRed - not listed in the Gamestudio manual - allows to access diffuse_x, diffuse_y and diffuse_z as a Vector
        /// </summary>
        private Color diffuse_blueGreenRed_ = null;
        public Color diffuse_blueGreenRed
        {
            get
            {
                unsafe
                {
                    if (diffuse_blueGreenRed_ == null)
                    {
                        diffuse_blueGreenRed_ = new Color((Var)0, (Var)0, (Var)0);
                        diffuse_blueGreenRed_.red = (Var)(IntPtr)(&((*MATERIALPointer).diffuse_red));
                        diffuse_blueGreenRed_.green = (Var)(IntPtr)(&((*MATERIALPointer).diffuse_green));
                        diffuse_blueGreenRed_.blue = (Var)(IntPtr)(&((*MATERIALPointer).diffuse_blue));
                    }
                }
                return diffuse_blueGreenRed_;
            }
        }

        /// <summary>
        /// specular_blue - see Gamestudio manual
        /// </summary>
        private Var specular_blue_ = null;
        public Var specular_blue
        {
            get
            {
                unsafe
                {
                    if (specular_blue_ == null)
                    {
                        specular_blue_ = (Var)(IntPtr)(&((*MATERIALPointer).specular_blue));
                    }
                }
                return specular_blue_;
            }
        }

        /// <summary>
        /// specular_green - see Gamestudio manual
        /// </summary>
        private Var specular_green_ = null;
        public Var specular_green
        {
            get
            {
                unsafe
                {
                    if (specular_green_ == null)
                    {
                        specular_green_ = (Var)(IntPtr)(&((*MATERIALPointer).specular_green));
                    }
                }
                return specular_green_;
            }
        }

        /// <summary>
        /// specular_red - see Gamestudio manual
        /// </summary>
        private Var specular_red_ = null;
        public Var specular_red
        {
            get
            {
                unsafe
                {
                    if (specular_red_ == null)
                    {
                        specular_red_ = (Var)(IntPtr)(&((*MATERIALPointer).specular_red));
                    }
                }
                return specular_red_;
            }
        }

        /// <summary>
        /// specular_blueGreenRed - not listed in the Gamestudio manual - allows to access specular_x, specular_y and specular_z as a Vector
        /// </summary>
        private Color specular_blueGreenRed_ = null;
        public Color specular_blueGreenRed
        {
            get
            {
                unsafe
                {
                    if (specular_blueGreenRed_ == null)
                    {
                        specular_blueGreenRed_ = new Color((Var)0, (Var)0, (Var)0);
                        specular_blueGreenRed_.red = (Var)(IntPtr)(&((*MATERIALPointer).specular_red));
                        specular_blueGreenRed_.green = (Var)(IntPtr)(&((*MATERIALPointer).specular_green));
                        specular_blueGreenRed_.blue = (Var)(IntPtr)(&((*MATERIALPointer).specular_blue));
                    }
                }
                return specular_blueGreenRed_;
            }
        }

        /// <summary>
        /// emissive_blue - see Gamestudio manual
        /// </summary>
        private Var emissive_blue_ = null;
        public Var emissive_blue
        {
            get
            {
                unsafe
                {
                    if (emissive_blue_ == null)
                    {
                        emissive_blue_ = (Var)(IntPtr)(&((*MATERIALPointer).emissive_blue));
                    }
                }
                return emissive_blue_;
            }
        }

        /// <summary>
        /// emissive_green - see Gamestudio manual
        /// </summary>
        private Var emissive_green_ = null;
        public Var emissive_green
        {
            get
            {
                unsafe
                {
                    if (emissive_green_ == null)
                    {
                        emissive_green_ = (Var)(IntPtr)(&((*MATERIALPointer).emissive_green));
                    }
                }
                return emissive_green_;
            }
        }

        /// <summary>
        /// emissive_red - see Gamestudio manual
        /// </summary>
        private Var emissive_red_ = null;
        public Var emissive_red
        {
            get
            {
                unsafe
                {
                    if (emissive_red_ == null)
                    {
                        emissive_red_ = (Var)(IntPtr)(&((*MATERIALPointer).emissive_red));
                    }
                }
                return emissive_red_;
            }
        }

        /// <summary>
        /// emissive_blueGreenRed - not listed in the Gamestudio manual - allows to access emissive_x, emissive_y and emissive_z as a Vector
        /// </summary>
        private Color emissive_blueGreenRed_ = null;
        public Color emissive_blueGreenRed
        {
            get
            {
                unsafe
                {
                    if (emissive_blueGreenRed_ == null)
                    {
                        emissive_blueGreenRed_ = new Color((Var)0, (Var)0, (Var)0);
                        emissive_blueGreenRed_.red = (Var)(IntPtr)(&((*MATERIALPointer).emissive_red));
                        emissive_blueGreenRed_.green = (Var)(IntPtr)(&((*MATERIALPointer).emissive_green));
                        emissive_blueGreenRed_.blue = (Var)(IntPtr)(&((*MATERIALPointer).emissive_blue));
                    }
                }
                return emissive_blueGreenRed_;
            }
        }

        /// <summary>
        /// power - see Gamestudio manual
        /// </summary>
        private Var power_ = null;
        public Var power
        {
            get
            {
                unsafe
                {
                    if (power_ == null)
                    {
                        power_ = (Var)(IntPtr)(&((*MATERIALPointer).power));
                    }
                }
                return power_;
            }
        }

        /// <summary>
        /// alpha - see Gamestudio manual
        /// </summary>
        private Var alpha_ = null;
        public Var alpha
        {
            get
            {
                unsafe
                {
                    if (alpha_ == null)
                    {
                        alpha_ = (Var)(IntPtr)(&((*MATERIALPointer).alpha));
                    }
                }
                return alpha_;
            }
        }

        /// <summary>
        /// albedo - see Gamestudio manual
        /// </summary>
        private Var albedo_ = null;
        public Var albedo
        {
            get
            {
                unsafe
                {
                    if (albedo_ == null)
                    {
                        albedo_ = (Var)(IntPtr)(&((*MATERIALPointer).albedo));
                    }
                }
                return albedo_;
            }
        }

        /// <summary>
        /// cycle - see Gamestudio manual
        /// </summary>
        private Var cycle_ = null;
        public Var cycle
        {
            get
            {
                unsafe
                {
                    if (cycle_ == null)
                    {
                        cycle_ = (Var)(IntPtr)(&((*MATERIALPointer).cycle));
                    }
                }
                return cycle_;
            }
        }

        /// <summary>
        /// effect - see Gamestudio manual
        /// </summary>
        public String effect
        {
            get
            {
                unsafe
                {
                    return Marshal.PtrToStringAnsi((IntPtr)(*MATERIALPointer).effect);
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).effect = (char*)(void*)Marshal.StringToHGlobalAnsi(value);
                }
            }
        }

        /// <summary>
        /// event - see Gamestudio manual
        /// </summary>
        public EventVoid event_
        {
            set
            {
                unsafe
                {
                    Scheduler.EventVoidReference TempEventVoidReference = new Scheduler.EventVoidReference(value, (IntPtr)MATERIALPointer);
                    (*MATERIALPointer).event_.mPtr = (byte*)(void*)Marshal.GetFunctionPointerForDelegate((NativeEventVoid)TempEventVoidReference.NativeEventVoidFunc);
                }
            }
        }
        /// <summary>
        /// access the MATRIX of the MATERIAL
        /// </summary>
        private MATRIX matrix_ = null;
        public MATRIX matrix
        {
            get
            {
                unsafe
                {
                    if (matrix_ == null)
                    {
                        matrix_ = new MATRIX((IntPtr)(&((*MATERIALPointer).matrix_11)));;
                    }
                }
                return matrix_;
            }
        }

        /// <summary>
        /// MATERIAL flags
        /// </summary>
        public enum FLAG : int
        {
            TRANSLUCENT = 1 << 10,
            PASS_SOLID = 1 << 11,
            TANGENT = 1 << 16,
            ENABLE_RENDER = 1 << 17,
            ENABLE_VIEW = 1 << 18,
            ENABLE_TREE = 1 << 19,
            OVERRIDE = 1 << 22,
            AUTORELOAD = 1 << 24
        }
        
        /// <summary>
        /// flags - see Gamestudio manual
        /// </summary>
        public int flags
        {
            get
            {
                unsafe
                {
                    return (*MATERIALPointer).flags;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).flags = value;
                }
            }
        }

        /// <summary>
        /// tangent - see Gamestudio manual
        /// </summary>
        public Boolean TANGENT
        {
            get
            {
                unsafe
                {
                    if (((*MATERIALPointer).flags & (int)FLAG.TANGENT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags | (int)FLAG.TANGENT;
                    else
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags & ~(int)FLAG.TANGENT;

                }
            }
        }

        /// <summary>
        /// translucent - see Gamestudio manual
        /// </summary>
        public Boolean TRANSLUCENT
        {
            get
            {
                unsafe
                {
                    if (((*MATERIALPointer).flags & (int)FLAG.TRANSLUCENT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags | (int)FLAG.TRANSLUCENT;
                    else
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags & ~(int)FLAG.TRANSLUCENT;

                }
            }
        }

        /// <summary>
        /// pass_solid - see Gamestudio manual
        /// </summary>
        public Boolean PASS_SOLID
        {
            get
            {
                unsafe
                {
                    if (((*MATERIALPointer).flags & (int)FLAG.PASS_SOLID) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags | (int)FLAG.PASS_SOLID;
                    else
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags & ~(int)FLAG.PASS_SOLID;

                }
            }
        }

        /// <summary>
        /// autoreload - see Gamestudio manual
        /// </summary>
        public Boolean AUTORELOAD
        {
            get
            {
                unsafe
                {
                    if (((*MATERIALPointer).flags & (int)FLAG.AUTORELOAD) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags | (int)FLAG.AUTORELOAD;
                    else
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags & ~(int)FLAG.AUTORELOAD;

                }
            }
        }

        /// <summary>
        /// override - see Gamestudio manual
        /// </summary>
        public Boolean OVERRIDE
        {
            get
            {
                unsafe
                {
                    if (((*MATERIALPointer).flags & (int)FLAG.OVERRIDE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags | (int)FLAG.OVERRIDE;
                    else
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags & ~(int)FLAG.OVERRIDE;

                }
            }
        }

        /// <summary>
        /// enable_render
        /// </summary>
        public Boolean ENABLE_RENDER
        {
            get
            {
                unsafe
                {
                    if (((*MATERIALPointer).flags & (int)FLAG.ENABLE_RENDER) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags | (int)FLAG.ENABLE_RENDER;
                    else
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags & ~(int)FLAG.ENABLE_RENDER;

                }
            }
        }

        /// <summary>
        /// enable_tree - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_TREE
        {
            get
            {
                unsafe
                {
                    if (((*MATERIALPointer).flags & (int)FLAG.ENABLE_TREE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags | (int)FLAG.ENABLE_TREE;
                    else
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags & ~(int)FLAG.ENABLE_TREE;

                }
            }
        }

        /// <summary>
        /// enable_view - see Gamestudio manual
        /// </summary>
        public Boolean ENABLE_VIEW
        {
            get
            {
                unsafe
                {
                    if (((*MATERIALPointer).flags & (int)FLAG.ENABLE_VIEW) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags | (int)FLAG.ENABLE_VIEW;
                    else
                        (*MATERIALPointer).flags = (*MATERIALPointer).flags & ~(int)FLAG.ENABLE_VIEW;

                }
            }
        }

        /// <summary>
        /// skill1 - see Gamestudio manual
        /// </summary>
        public Var skill1
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_01;
                }
            }
            set
            {
                unsafe
                {            
                    (*MATERIALPointer).skill_01 = (NativeVar)value;                
                }
            }
        }

        /// <summary>
        /// skill2 - see Gamestudio manual
        /// </summary>
        public Var skill2
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_02;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_02 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill3 - see Gamestudio manual
        /// </summary>
        public Var skill3
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_03;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_03 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill4 - see Gamestudio manual
        /// </summary>
        public Var skill4
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_04;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_04 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill5 - see Gamestudio manual
        /// </summary>
        public Var skill5
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_05;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_05 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill6 - see Gamestudio manual
        /// </summary>
        public Var skill6
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_06;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_06 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill7 - see Gamestudio manual
        /// </summary>
        public Var skill7
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_07;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_07 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill8 - see Gamestudio manual
        /// </summary>
        public Var skill8
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_08;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_08 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill9 - see Gamestudio manual
        /// </summary>
        public Var skill9
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_09;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_09 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill10 - see Gamestudio manual
        /// </summary>
        public Var skill10
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_10;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_10 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill11 - see Gamestudio manual
        /// </summary>
        public Var skill11
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_11;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_11 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill12 - see Gamestudio manual
        /// </summary>
        public Var skill12
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_12;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_12 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill13 - see Gamestudio manual
        /// </summary>
        public Var skill13
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_13;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_13 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill14 - see Gamestudio manual
        /// </summary>
        public Var skill14
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_14;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_14 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill15 - see Gamestudio manual
        /// </summary>
        public Var skill15
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_15;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_15 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill16
        /// </summary>
        public Var skill16
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_16;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_16 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill17
        /// </summary>
        public Var skill17
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_17;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_17 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill18 - see Gamestudio manual
        /// </summary>
        public Var skill18
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_18;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_18 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill19 - see Gamestudio manual
        /// </summary>
        public Var skill19
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_19;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_19 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// skill20 - see Gamestudio manual
        /// </summary>
        public Var skill20
        {
            get
            {
                unsafe
                {
                    return (Var)(*MATERIALPointer).skill_20;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skill_20 = (NativeVar)value;
                }
            }
        }

        /// <summary>
        /// lod - see Gamestudio manual
        /// </summary>
        private Var lod_ = null;
        public Var lod
        {
            get
            {
                unsafe
                {
                    if (lod_ == null)
                    {
                        lod_ = (Var)(IntPtr)(&((*MATERIALPointer).lod));
                    }
                }
                return lod_;
            }
        }

        /// <summary>
        /// skin1 - see Gamestudio manual
        /// </summary>
        public BMAP skin1
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*MATERIALPointer).skin1;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skin1 = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// skin2 - see Gamestudio manual
        /// </summary>
        public BMAP skin2
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*MATERIALPointer).skin2;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skin2 = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// skin3 - see Gamestudio manual
        /// </summary>
        public BMAP skin3
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*MATERIALPointer).skin3;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skin3 = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// skin4 - see Gamestudio manual
        /// </summary>
        public BMAP skin4
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*MATERIALPointer).skin4;
                }
            }
            set
            {
                unsafe
                {
                    (*MATERIALPointer).skin4 = (NativeBMAP*)(IntPtr)value;
                }
            }
        }


        /* -------- MATERIAL -------- */
        /* MATERIAL to NativeMATERIAL* (address in void*) */
        public static explicit operator IntPtr(MATERIAL d)
        {
            unsafe
            {
                if (d == null)
                {
                    return IntPtr.Zero;
                }
                else
                {
                    return (IntPtr)(NativeMATERIAL*)d.MATERIALPointer;
                }
            }
        }
        /* NativeMATERIAL* (address in void*) to MATERIAL */
        public static explicit operator MATERIAL(IntPtr d)
        {
            unsafe
            {
                if (d == IntPtr.Zero)
                {
                    return null;
                }
                return new MATERIAL(d);
            }
        }
    }
}
