﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using AcknexWrapper.Native;
using System.Runtime.InteropServices;

namespace AcknexWrapper
{
    public class MATRIX
    {
        private IntPtr nativeMATRIXPointer;
        private Boolean engineExtern;

        /// <summary>
        /// represent matrix, that is referecenced to with a pointer
        /// </summary>
        /// <param name="nativeMATRIXPointer"></param>
        public MATRIX(IntPtr nativeMATRIXPointer) {
            this.nativeMATRIXPointer = nativeMATRIXPointer;
            engineExtern = false;
        }

        /// <summary>
        /// create a Matrix initialized with zeros
        /// </summary>
        public MATRIX() {
            nativeMATRIXPointer = Marshal.AllocHGlobal(Marshal.SizeOf(new NativeMATRIX()));
            this.setMatrix(
                new float[][] {
                        new float[] { 0, 0, 0, 0 },
                        new float[] { 0, 0, 0, 0 },
                        new float[] { 0, 0, 0, 0 },
                        new float[] { 0, 0, 0, 0 }
                    });
            engineExtern = true;
        }

        ~MATRIX() {
            if(engineExtern == true) {
                Marshal.FreeHGlobal(nativeMATRIXPointer);

            }
        }

        /// <summary>
        /// get a IntPtr to the Matrix
        /// </summary>
        /// <returns></returns>
        public IntPtr getMatrixPointer()
        {
            return nativeMATRIXPointer;
        }

        /// <summary>
        /// get the matrix as float[4][4]
        /// </summary>
        /// <returns>float[4][4]</returns>
        public float[][] getMatrix()
        {
            float[][] temp = new float[4][];

            for(int i = 0; i < 4; i++) {
                temp[i] = new float[4];
                for (int j = 0; j < 4; j++)
                {
                    temp[i][j] = this.getValue(i + 1, j + 1);
                }
            }
            return temp;
        }

        /// <summary>
        /// set the float[4][4] matrix
        /// </summary>
        public void setMatrix(float[][] matrix)
        {
            for (int i = 0; i < 4; i++)
            {
                for (int j = 0; j < 4; j++)
                {
                    this.setValue(i + 1, j + 1, matrix[i][j]);
                }
            }
        }

        /// <summary>
        /// get the value of a matrix element, row (1..4), column (1..4)
        /// </summary>
        /// <param name="row">1..4</param>
        /// <param name="column">1..4</param>
        /// <returns></returns>
        public float getValue(int row, int column)
        {
            if (
                (row < 1) || (row > 4) ||
                (column < 1) || (column > 4))
            {
                new Exception("Wrong index");
            }

            int index = ((row - 1) * 4) + column;
            NativeMATRIX matrix;
            unsafe {
                matrix = *((NativeMATRIX *)nativeMATRIXPointer);
            }
            switch (index)
            {
                case (1):
                    {
                        return matrix.matrix1;
                    }
                case (2):
                    {
                        return matrix.matrix2;
                    }
                case (3):
                    {
                        return matrix.matrix3;
                    }
                case (4):
                    {
                        return matrix.matrix4;
                    }
                case (5):
                    {
                        return matrix.matrix5;
                    }
                case (6):
                    {
                        return matrix.matrix6;
                    }
                case (7):
                    {
                        return matrix.matrix7;
                    }
                case (8):
                    {
                        return matrix.matrix8;
                    }
                case (9):
                    {
                        return matrix.matrix9;
                    }
                case (10):
                    {
                        return matrix.matrix10;
                    }
                case (11):
                    {
                        return matrix.matrix11;
                    }
                case (12):
                    {
                        return matrix.matrix12;
                    }
                case (13):
                    {
                        return matrix.matrix13;
                    }
                case (14):
                    {
                        return matrix.matrix14;
                    }
                case (15):
                    {
                        return matrix.matrix15;
                    }
                case (16):
                    {
                        return matrix.matrix16;
                    }
                default:
                    {
                        return 0;
                    }
            }
        }

        /// <summary>
        /// set the value of a matrix element, row (1..4), column (1..4)
        /// </summary>
        /// <param name="row">1..4</param>
        /// <param name="column">1..4</param>
        /// <returns></returns>
        public void setValue(int row, int column, float value)
        {
            if (
                (row < 1) || (row > 4) ||
                (column < 1) || (column > 4))
            {
                new Exception("Wrong index");
            }

            int index = ((row - 1) * 4) + column;
            NativeMATRIX matrix;
            unsafe
            {
                matrix = *((NativeMATRIX*)nativeMATRIXPointer);
            }
            switch (index)
            {
                case (1):
                    {
                        matrix.matrix1 = value;
                        break;
                    }
                case (2):
                    {
                        matrix.matrix2 = value;
                        break;
                    }
                case (3):
                    {
                        matrix.matrix3 = value;
                        break;
                    }
                case (4):
                    {
                        matrix.matrix4 = value;
                        break;
                    }
                case (5):
                    {
                        matrix.matrix5 = value;
                        break;
                    }
                case (6):
                    {
                        matrix.matrix6 = value;
                        break;
                    }
                case (7):
                    {
                        matrix.matrix7 = value;
                        break;
                    }
                case (8):
                    {
                        matrix.matrix8 = value;
                        break;
                    }
                case (9):
                    {
                        matrix.matrix9 = value;
                        break;
                    }
                case (10):
                    {
                        matrix.matrix10 = value;
                        break;
                    }
                case (11):
                    {
                        matrix.matrix11 = value;
                        break;
                    }
                case (12):
                    {
                        matrix.matrix12 = value;
                        break;
                    }
                case (13):
                    {
                        matrix.matrix13 = value;
                        break;
                    }
                case (14):
                    {
                        matrix.matrix14 = value;
                        break;
                    }
                case (15):
                    {
                        matrix.matrix15 = value;
                        break;
                    }
                case (16):
                    {
                        matrix.matrix16 = value;
                        break;
                    }
            }
            Marshal.StructureToPtr(matrix, nativeMATRIXPointer, false);
        }

        /// <summary>
        /// get/set matrix element in row 1, column 1
        /// </summary>
        public float matrix11
        {
            get
            {
                return getValue(1, 1);
            }
            set
            {
                setValue(1, 1, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 1, column 2
        /// </summary>
        public float matrix12
        {
            get
            {
                return getValue(1, 2);
            }
            set
            {
                setValue(1, 2, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 1, column 3
        /// </summary>
        public float matrix13
        {
            get
            {
                return getValue(1, 3);
            }
            set
            {
                setValue(1, 3, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 1, column 4
        /// </summary>
        public float matrix14
        {
            get
            {
                return getValue(1, 4);
            }
            set
            {
                setValue(1, 4, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 2, column 1
        /// </summary>
        public float matrix21
        {
            get
            {
                return getValue(2, 1);
            }
            set
            {
                setValue(1, 1, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 2, column 2
        /// </summary>
        public float matrix22
        {
            get
            {
                return getValue(2, 2);
            }
            set
            {
                setValue(2, 2, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 2, column 3
        /// </summary>
        public float matrix23
        {
            get
            {
                return getValue(2, 3);
            }
            set
            {
                setValue(2, 3, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 2, column 4
        /// </summary>
        public float matrix24
        {
            get
            {
                return getValue(2, 4);
            }
            set
            {
                setValue(2, 4, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 3, column 1
        /// </summary>
        public float matrix31
        {
            get
            {
                return getValue(3, 1);
            }
            set
            {
                setValue(3, 1, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 3, column 2
        /// </summary>
        public float matrix32
        {
            get
            {
                return getValue(3, 2);
            }
            set
            {
                setValue(3, 2, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 3, column 3
        /// </summary>
        public float matrix33
        {
            get
            {
                return getValue(3, 3);
            }
            set
            {
                setValue(3, 3, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 3, column 4
        /// </summary>
        public float matrix34
        {
            get
            {
                return getValue(3, 4);
            }
            set
            {
                setValue(3, 4, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 4, column 1
        /// </summary>
        public float matrix41
        {
            get
            {
                return getValue(4, 1);
            }
            set
            {
                setValue(4, 1, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 4, column 2
        /// </summary>
        public float matrix42
        {
            get
            {
                return getValue(4, 2);
            }
            set
            {
                setValue(4, 2, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 4, column 3
        /// </summary>
        public float matrix43
        {
            get
            {
                return getValue(4, 3);
            }
            set
            {
                setValue(4, 3, value);
            }
        }

        /// <summary>
        /// get/set matrix element in row 4, column 4
        /// </summary>
        public float matrix44
        {
            get
            {
                return getValue(4, 4);
            }
            set
            {
                setValue(4, 4, value);
            }
        }

        /* -------- MATRIX -------- */
        /* MATRIX to NativeMATRIX* (address in void*) */
        public static explicit operator IntPtr(MATRIX d)
        {
            unsafe
            {
                if (d == null)
                {
                    return IntPtr.Zero;
                }
                else
                {
                    return d.getMatrixPointer();
                }
            }
        }
        /* NativeMATRIX* (address in void*) to MATRIX */
        public static explicit operator MATRIX(IntPtr d)
        {
            unsafe
            {
                if (d == IntPtr.Zero)
                {
                    return null;
                }
                return new MATRIX(d);
            }
        }
    }
}