﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using System.Runtime.InteropServices;

namespace AcknexWrapper
{
    namespace Native
    {
        [StructLayout(LayoutKind.Sequential)]
        internal unsafe struct NativeENGINE_VARS
        {
            public NativeVar* version;
            public NativeVar* edition;
            public NativeVar* compatibility;
            public NativeVar* logo;			// logo corner, 1..4
            public NativeVar* nexus;			// current nexus consumed
            public NativeVar* event_type;
            public NativeVar* warn_level;
            public NativeVar* max_loops;		// WDL only
            public NativeVar* freeze_mode;
            public NativeVar* debug_mode;
            public NativeVar* debug_external;
            public NativeVar* mtl_mode;
            public NativeVar* last_error;
            public NativeVar* proc_mode;
            public NativeVar* run_mode;
            public NativeVar* patch;
            public NativeVar* result;
            public NativeVar* max_nexus;		// -nx command line value
            public NativeVar* diag_mode;
            public NativeVar* record_mode;

            // time & frame
            public NativeVar* time_step;		// smoothed time of the last frame 
            public NativeVar* time_factor;		// slow motion effect
            public NativeVar* time_smooth;
            public NativeVar* time_frame;		// real frametime without slow motion
            public NativeVar* total_ticks;
            public NativeVar* total_secs;
            public NativeVar* total_frames;
            public NativeVar* fps_max;			// limits fps
            public NativeVar* fps_min;
            public NativeVar* fps_lock;		// locks time to fps_max

            public NativeVar* reserved01_1;
            public NativeVar* reserved01_2;
            public NativeVar* reserved01_3;

            public NativeVar* time_actions;	// time in ms for entity actions
            public NativeVar* time_render;
            public NativeVar* time_level;
            public NativeVar* time_entities;
            public NativeVar* time_effects;
            public NativeVar* time_pfuncs;
            public NativeVar* time_portals;
            public NativeVar* time_sky;
            public NativeVar* time_client;
            public NativeVar* time_server;
            public NativeVar* time_panels;
            public NativeVar* time_update;
            public NativeVar* time_physics;
            public NativeVar* time_total;
            public NativeVar* time_idle;
            public NativeVar* time_draw;
            public NativeVar* time_scope;

            public NativeVar* sys_seconds;		// PC local time
            public NativeVar* sys_minutes;
            public NativeVar* sys_hours;
            public NativeVar* sys_day;
            public NativeVar* sys_month;
            public NativeVar* sys_year;
            public NativeVar* sys_dow;
            public NativeVar* sys_doy;
            public NativeVar* sys_memory;
            public NativeVar* sys_winversion;

            public NativeVar* reserved03_1;
            public NativeVar* reserved03_2;
            public NativeVar* reserved03_3;
            public NativeVar* reserved03_4;

            // input
            public NativeVar* enable_joystick;
            public NativeVar* num_joysticks;
            public NativeVar* joy_sense;
            public NativeVector* joy_force;
            public NativeVector* joy_raw;
            public NativeVector* joy_rot;
            public NativeVar* joy_hat;
            public NativeVector* joy2_raw;
            public NativeVector* joy2_rot;
            public NativeVar* joy2_hat;
            public NativeVar* joy_buttons;
            public NativeVar* joy2_buttons;

            public NativeVar* reserved04_1;

            public NativeVar* enable_mouse;
            public NativeVar* mouse_mode;
            public NativeVar* mouse_sense;
            public NativeVector* mouse_force;
            public NativeVector* mouse_pos;
            public NativeVector* mouse_spot;
            public NativeVar* mouse_pointer;
            public NativeVar* mouse_range;
            public NativeVar* mouse_time;
            public NativeVar* mouse_moving;
            public NativeVar* mouse_calm;
            public NativeVar* mouse_sync;
            public NativeVar* mouse_valid;
            public NativeVar* touch_state;
            public NativeVector* mickey;
            public NativeVector* mouse_cursor;	// WDL: pointer
            public NativeVector* mouse_pos3d;
            public NativeVector* mouse_dir3d;

            public NativeVar* reserved05_1;
            public NativeVar* reserved05_2;
            public NativeVar* reserved05_3;

            public NativeVar* enable_key;
            public NativeVar* key_sense;
            public NativeVector* key_force;
            public NativeVar* shift_sense;
            public NativeVar* key_shift;
            public NativeVar* key_any;
            public NativeVar* key_lastpressed;
            public NativeVar* inkey_active;
            public NativeVar* text_outline;
            public NativeVar* decal_clip;

            // video
            public NativeVar* video_aspect;
            public NativeVar* video_mode;
            public NativeVar* video_depth;
            public NativeVar* video_screen;	// 1 = fullscreen, 2 = window
            public NativeVector* screen_size;
            public NativeVector* screen_color;
            public NativeVar* window_focus;
            public NativeVar* video_gamma;
            public NativeVector* window_pos;
            public NativeVar* d3d_antialias;
            public NativeVar* mip_levels;
            public NativeVector* ambient_color;

            // renderer
            public NativeVar* d3d_caps;
            public NativeVar* d3d_optimize;
            public NativeVar* d3d_mode;
            public NativeVar* d3d_nobsp;
            public NativeVar* d3d_noents;
            public NativeVar* d3d_novis;
            public NativeVar* d3d_notex;
            public NativeVar* d3d_flags;
            public NativeVar* d3d_lightres;
            public NativeVar* d3d_monochrome;
            public NativeVar* d3d_lines;
            public NativeVar* d3d_anisotropy;
            public NativeVar* d3d_instancing;
            public NativeVar* d3d_lockable;
            public NativeVar* effect_lod;
            public NativeVar* d3d_autotransparency;
            public NativeVar* d3d_automaterial;
            public NativeVar* d3d_shaderversion;
            public NativeVar* shadow_offset;
            public NativeVar* shadow_threshold;
            public NativeVar* shadow_range;
            public NativeVar* shadow_stencil;
            public NativeVar* d3d_shadowdepth;
            public NativeVar* d3d_alphadepth;
            public NativeVar* shadow_lod;
            public NativeVar* d3d_texsmaps;
            public NativeVar* d3d_texbmaps;
            public NativeVar* d3d_texskins;
            public NativeVar* d3d_texsurfs;
            public NativeVar* d3d_textotal;
            public NativeVar* d3d_texmemory;
            public NativeVar* d3d_alpharef;
            public NativeVar* d3d_texfree;
            public NativeVar* d3d_texdepth;
            public NativeVar* d3d_texmax;
            public NativeVar* d3d_texlimit;
            public NativeVar* d3d_entsort;
            public NativeVar* d3d_near_sprites;
            public NativeVar* d3d_mipmapping;
            public NativeVar* d3d_near_models;
            public NativeVar* d3d_triplebuffer;
            public NativeVector* d3d_lodfactor;
            public NativeVector* d3d_pointlightfalloff;
            public NativeVector* d3d_spotlightfalloff;
            public NativeVector* d3d_spotlightcone;
            public NativeVar* tree_range;
            public NativeVar* clip_factor;
            public NativeVar* clip_particles;
            public NativeVar* clip_size;
            public NativeVar* floor_range;
            public NativeVar* particle_mode;
            public NativeVar* mip_shaded;
            public NativeVar* mip_flat;
            public NativeVar* mip_sprites;
            public NativeVar* shadow_mode;
            public NativeVar* trace_floor;

            // level parameters
            public NativeVar* max_particles;
            public NativeVar* max_entities;
            public NativeVar* max_lights;
            public NativeVector* sky_color;
            public NativeVar* max_bones;
            public NativeVar* sky_blend;
            public NativeVector* sky_speed;
            public NativeVector* cloud_speed;
            public NativeVar* d3d_skydepth;
            public NativeVar* sky_curve;
            public NativeVar* sky_clip;
            public NativeVar* terrain_chunk;
            public NativeVar* terrain_lod;
            public NativeVar* num_lights;
            public NativeVar* num_sounds;
            public NativeVar* sun_light;
            public NativeVector* sun_color;
            public NativeVector* sun_angle;
            public NativeVector* sun_pos;
            public NativeVar* preload_mode;
            public NativeVar* fog_color;
            public NativeVector* d3d_fogcolor1;
            public NativeVector* d3d_fogcolor2;
            public NativeVector* d3d_fogcolor3;
            public NativeVector* d3d_fogcolor4;
            public NativeVar* turb_speed;
            public NativeVar* turb_range;
            public NativeVar* tex_cycles;
            public NativeVar* detail_size;
            public NativeVar* pos_resolution;
            public NativeVar* tex_share;
            public NativeVar* map_subents;

            // statistics
            public NativeVar* num_particles;
            public NativeVar* num_entities;
            public NativeVar* num_visents;
            public NativeVar* num_vismappolys;
            public NativeVar* num_visentpolys;
            public NativeVar* num_vistriangles;
            public NativeVar* num_actions;

            // multiplayer
            public NativeVar* connection;
            public NativeVar* dplay_maxclients;
            public NativeVar* dplay_bandwidth;
            public NativeVar* dplay_unreliable;
            public NativeVar* dplay_smooth;
            public NativeVar* dplay_localfunction;
            public NativeVar* dplay_compression;
            public NativeVar* dplay_entupdate;
            public NativeVar* dplay_pingrate;
            public NativeVar* dplay_entrate;
            public NativeVar* dplay_packetsize;
            public NativeVar* dplay_timeout;
            public NativeVar* dplay_latency;
            public NativeVar* dplay_bps;
            public NativeVar* dplay_bpspeak;
            public NativeVar* dplay_bpsreliable;
            public NativeVar* dplay_bpsunreliable;
            public NativeVar* dplay_dropped;
            public NativeVar* recbuf_size;
            public NativeVar* dplay_diag;
            public NativeVar* num_clients;
            public NativeVar* dplay_encrypt;
            public NativeVar* dplay_port;
            public NativeVar* dplay_id;
            public NativeVar* dplay_status;
            public NativeVar* reserved63;

            // audio
            public NativeVar* sound_vol;
            public NativeVar* cdaudio_vol;
            public NativeVar* midi_vol;
            public NativeVar* media_handle;
            public NativeVar* media_layer;
            public NativeVar* num_mediastreams;
            public NativeVar* doppler_factor;
            public NativeVar* master_vol;
            public NativeVar* video_alpha;
            public NativeVar* reserved64;

            // physics & collision
            public NativeVector* hull_fatmin;
            public NativeVector* hull_fatmax;
            public NativeVector* hull_narrowmin;
            public NativeVector* hull_narrowmax;

            public NativeVar* max_ph_entities;
            public NativeVar* enable_polycollision;
            public NativeVar* trace_mode;
            public NativeVar* move_mode;
            public NativeVar* move_friction;
            public NativeVar* move_restrict;
            public NativeVector* normal;
            public NativeVector* bounce;
            public NativeVector* target;
            public NativeVector* vglide;
            public NativeVar* on_passable;
            public NativeVar* in_passable;
            public NativeVar* in_solid;
            public NativeVar* tex_light;
            public NativeVector* tex_color;
            public NativeVar* tex_fog;
            public NativeVar* tex_flag1;
            public NativeVar* tex_flag2;
            public NativeVar* tex_flag3;
            public NativeVar* tex_flag4;
            public NativeVar* tex_flag5;
            public NativeVar* tex_flag6;
            public NativeVar* tex_flag7;
            public NativeVar* tex_flag8;
            public NativeVar* hitvertex;
            public NativeVar* terrain_cell_size;
            public NativeVar* trace_hit;
            public NativeVar* ph_num_bodies;
            public NativeVar* ph_num_contacts;
            public NativeVar* ph_iterations;
            public NativeVar* ph_fps_max_lock;	// locks physics to fps_max
            public NativeVar* move_min_z;
            public NativeVar* disable_z_glide;
            public NativeVar* ph_max_contacts;
            public NativeVar* ph_check_distance; // min. body movement per fps_max frame before it's checked
            public NativeVar* ph_num_collisions;
            public NativeVar* move_pushback;
            public NativeVar* shadow_meshsize;
            public NativeVar* last_skill;

            public NativeVar* reserved8_1;
            public NativeVar* reserved8_2;
            public NativeVar* reserved8_3;
            public NativeVar* reserved8_4;
            public NativeVar* reserved8_5;
            public NativeVar* reserved8_6;
            public NativeVar* reserved8_7;
            public NativeVar* reserved8_8;
            public NativeVar* reserved8_9;
            public NativeVar* reserved8_10;
            public NativeVar* reserved8_11;
            public NativeVar* reserved8_12;
            public NativeVar* reserved8_13;
            public NativeVar* reserved8_14;
            public NativeVar* reserved8_15;

            // button states
            public NativeVar* key_bksp;
            public NativeVar* key_enter;
            public NativeVar* key_esc;
            public NativeVar* key_alt;
            public NativeVar* key_ctrl;
            public NativeVar* key_shiftl;
            public NativeVar* key_shiftr;
            public NativeVar* key_space;
            public NativeVar* key_tab;
            public NativeVar* key_f1;
            public NativeVar* key_f2;
            public NativeVar* key_f3;
            public NativeVar* key_f4;
            public NativeVar* key_f5;
            public NativeVar* key_f6;
            public NativeVar* key_f7;
            public NativeVar* key_f8;
            public NativeVar* key_f9;
            public NativeVar* key_f10;
            public NativeVar* key_f11;
            public NativeVar* key_f12;
            public NativeVar* key_a;
            public NativeVar* key_b;
            public NativeVar* key_c;
            public NativeVar* key_d;
            public NativeVar* key_e;
            public NativeVar* key_f;
            public NativeVar* key_g;
            public NativeVar* key_h;
            public NativeVar* key_i;
            public NativeVar* key_j;
            public NativeVar* key_k;
            public NativeVar* key_l;
            public NativeVar* key_m;
            public NativeVar* key_n;
            public NativeVar* key_o;
            public NativeVar* key_p;
            public NativeVar* key_q;
            public NativeVar* key_r;
            public NativeVar* key_s;
            public NativeVar* key_t;
            public NativeVar* key_u;
            public NativeVar* key_v;
            public NativeVar* key_w;
            public NativeVar* key_x;
            public NativeVar* key_y;	// 'Z' on US keyboards
            public NativeVar* key_z;	// 'Y' on US keyboards
            public NativeVar* key_cuu;
            public NativeVar* key_cud;
            public NativeVar* key_cur;
            public NativeVar* key_cul;
            public NativeVar* key_pgup;
            public NativeVar* key_pgdn;
            public NativeVar* key_home;
            public NativeVar* key_end;
            public NativeVar* key_ins;
            public NativeVar* key_del;
            public NativeVar* key_car;
            public NativeVar* key_cal;
            public NativeVar* key_slash;
            public NativeVar* key_equals;
            public NativeVar* key_period;
            public NativeVar* key_comma;
            public NativeVar* key_brackl;
            public NativeVar* key_brackr;
            public NativeVar* key_semic;
            public NativeVar* key_bksl;
            public NativeVar* key_apos;
            public NativeVar* key_minusc;
            public NativeVar* key_grave;
            public NativeVar* key_caps;
            public NativeVar* key_scrlk;
            public NativeVar* key_plus;
            public NativeVar* key_sz;
            public NativeVar* key_ae;
            public NativeVar* key_ue;
            public NativeVar* key_oe;
            public NativeVar* key_1;
            public NativeVar* key_2;
            public NativeVar* key_3;
            public NativeVar* key_4;
            public NativeVar* key_5;
            public NativeVar* key_6;
            public NativeVar* key_7;
            public NativeVar* key_8;
            public NativeVar* key_9;
            public NativeVar* key_0;
            public NativeVar* key_pause;

            public NativeVar* mouse_left;
            public NativeVar* mouse_right;
            public NativeVar* mouse_middle;

            public NativeVar* joy_1;
            public NativeVar* joy_2;
            public NativeVar* joy_3;
            public NativeVar* joy_4;
            public NativeVar* joy_5;
            public NativeVar* joy_6;
            public NativeVar* joy_7;
            public NativeVar* joy_8;
            public NativeVar* joy_9;
            public NativeVar* joy_10;
            public NativeVar* joy_11;
            public NativeVar* joy_12;

            public NativeVar* joy2_1;
            public NativeVar* joy2_2;
            public NativeVar* joy2_3;
            public NativeVar* joy2_4;
            public NativeVar* joy2_5;
            public NativeVar* joy2_6;
            public NativeVar* joy2_7;
            public NativeVar* joy2_8;
            public NativeVar* joy2_9;
            public NativeVar* joy2_10;
            public NativeVar* joy2_11;
            public NativeVar* joy2_12;

            public NativeVar* reserved9_1;
            public NativeVar* reserved9_2;
            public NativeVar* reserved9_3;
            public NativeVar* reserved9_4;
            public NativeVar* reserved9_5;
            public NativeVar* reserved9_6;
            public NativeVar* reserved9_7;
            public NativeVar* reserved9_8;
            public NativeVar* reserved9_9;
            public NativeVar* reserved9_10;

            // engine event functions
            public NativeEVENT* on_bksp;
            public NativeEVENT* on_enter;
            public NativeEVENT* on_esc;
            public NativeEVENT* on_alt;
            public NativeEVENT* on_ctrl;
            public NativeEVENT* on_shiftl;
            public NativeEVENT* on_shiftr;
            public NativeEVENT* on_space;
            public NativeEVENT* on_tab;
            public NativeEVENT* on_f1;
            public NativeEVENT* on_f2;
            public NativeEVENT* on_f3;
            public NativeEVENT* on_f4;
            public NativeEVENT* on_f5;
            public NativeEVENT* on_f6;
            public NativeEVENT* on_f7;
            public NativeEVENT* on_f8;
            public NativeEVENT* on_f9;
            public NativeEVENT* on_f10;
            public NativeEVENT* on_f11;
            public NativeEVENT* on_f12;
            public NativeEVENT* on_a;
            public NativeEVENT* on_b;
            public NativeEVENT* on_c;
            public NativeEVENT* on_d;
            public NativeEVENT* on_e;
            public NativeEVENT* on_f;
            public NativeEVENT* on_g;
            public NativeEVENT* on_h;
            public NativeEVENT* on_i;
            public NativeEVENT* on_j;
            public NativeEVENT* on_k;
            public NativeEVENT* on_l;
            public NativeEVENT* on_m;
            public NativeEVENT* on_n;
            public NativeEVENT* on_o;
            public NativeEVENT* on_p;
            public NativeEVENT* on_q;
            public NativeEVENT* on_r;
            public NativeEVENT* on_s;
            public NativeEVENT* on_t;
            public NativeEVENT* on_u;
            public NativeEVENT* on_v;
            public NativeEVENT* on_w;
            public NativeEVENT* on_x;
            public NativeEVENT* on_y;		// 'Z' on US keyboards
            public NativeEVENT* on_z;		// 'Y' on US keyboards
            public NativeEVENT* on_cuu;
            public NativeEVENT* on_cud;
            public NativeEVENT* on_cur;
            public NativeEVENT* on_cul;
            public NativeEVENT* on_pgup;
            public NativeEVENT* on_pgdn;
            public NativeEVENT* on_home;
            public NativeEVENT* on_end;
            public NativeEVENT* on_ins;
            public NativeEVENT* on_del;
            public NativeEVENT* on_car;
            public NativeEVENT* on_cal;
            public NativeEVENT* on_slash;
            public NativeEVENT* on_equals;
            public NativeEVENT* on_period;
            public NativeEVENT* on_comma;
            public NativeEVENT* on_brackl;
            public NativeEVENT* on_brackr;
            public NativeEVENT* on_semic;
            public NativeEVENT* on_bksl;
            public NativeEVENT* on_apos;
            public NativeEVENT* on_minusc;
            public NativeEVENT* on_grave;
            public NativeEVENT* on_caps;
            public NativeEVENT* on_scrlk;
            public NativeEVENT* on_plus;
            public NativeEVENT* on_sz;
            public NativeEVENT* on_ae;
            public NativeEVENT* on_ue;
            public NativeEVENT* on_oe;
            public NativeEVENT* on_1;
            public NativeEVENT* on_2;
            public NativeEVENT* on_3;
            public NativeEVENT* on_4;
            public NativeEVENT* on_5;
            public NativeEVENT* on_6;
            public NativeEVENT* on_7;
            public NativeEVENT* on_8;
            public NativeEVENT* on_9;
            public NativeEVENT* on_0;
            public NativeEVENT* on_pause;

            public NativeEVENT* on_mouse_left;
            public NativeEVENT* on_mouse_right;
            public NativeEVENT* on_mouse_middle;

            public NativeEVENT* on_joy1;
            public NativeEVENT* on_joy2;
            public NativeEVENT* on_joy3;
            public NativeEVENT* on_joy4;
            public NativeEVENT* on_joy5;
            public NativeEVENT* on_joy6;
            public NativeEVENT* on_joy7;
            public NativeEVENT* on_joy8;
            public NativeEVENT* on_joy9;
            public NativeEVENT* on_joy10;
            public NativeEVENT* on_joy11;
            public NativeEVENT* on_joy12;

            public NativeEVENT* on_joy2_1;
            public NativeEVENT* on_joy2_2;
            public NativeEVENT* on_joy2_3;
            public NativeEVENT* on_joy2_4;
            public NativeEVENT* on_joy2_5;
            public NativeEVENT* on_joy2_6;
            public NativeEVENT* on_joy2_7;
            public NativeEVENT* on_joy2_8;
            public NativeEVENT* on_joy2_9;
            public NativeEVENT* on_joy2_10;
            public NativeEVENT* on_joy2_11;
            public NativeEVENT* on_joy2_12;

            public NativeEVENT* on_anykey;
            public NativeEVENT* on_click;
            public NativeEVENT* on_mouse_stop;

            public NativeEVENT* on_close;
            public NativeEVENT* on_load;
            public NativeEVENT* on_level;

            public NativeEVENT* on_server;
            public NativeEVENT* on_client;

            public NativeEVENT* on_d3d_lost;
            public NativeEVENT* on_d3d_reset;

            public NativeEVENT* render_sky;
            public NativeEVENT* render_world;
            public NativeEVENT* render_entities;
            public NativeEVENT* render_solid;

            public NativeEVENT* on_exit;
            public NativeEVENT* on_frame;  // called every frame
            public NativeEVENT* on_message; // the engine window's message loop

            public NativeEVENT* on_maximize;
            public NativeEVENT* on_minimize;
            public NativeEVENT* on_resize;

            public NativeEVENT* on_reserved_1;
            public NativeEVENT* on_reserved_2;
            public NativeEVENT* on_reserved_3;
            public NativeEVENT* on_reserved_4;
            public NativeEVENT* on_reserved_5;
            public NativeEVENT* on_reserved_6;
            public NativeEVENT* on_reserved_7;
            public NativeEVENT* on_reserved_8;
            public NativeEVENT* on_reserved_9;
            public NativeEVENT* on_reserved_10;
            public NativeEVENT* on_reserved_11;
            public NativeEVENT* on_reserved_12;
            public NativeEVENT* on_reserved_13;

            // engine pointers ///////////////////////////////////////////////////
            public NativeVIEW* camera;			// predefined view
            public NativeVIEW* light_view;	    // view used for clipping dynamic lights
            public NativeVIEW* render_view;	// currently rendering view

            public NativeENTITY* watched;
            public NativeENTITY* me;
            public NativeENTITY* you;
            public NativeENTITY* sky_cube_level;
            public NativeENTITY* sky_dome_level;
            public NativeENTITY* mouse_ent;

            public NativeBMAP* mouse_map;
            public NativeBMAP* render_map_1;	// current entity textures
            public NativeBMAP* render_map_2;	// current entity textures
            public NativeBMAP* render_map_3;	// current entity textures
            public NativeBMAP* render_map_4;	// current entity textures
            public void* render_mesh;	// current entity LPD3DXMESH

            public NativeMATERIAL* mtl;			// current material
            public NativeMATERIAL* mat_flat;
            public NativeMATERIAL* mat_shaded;
            public NativeMATERIAL* mat_model;
            public NativeMATERIAL* mat_terrain;
            public NativeMATERIAL* mat_sprite;
            public NativeMATERIAL* mat_particle;
            public NativeMATERIAL* mat_metal;
            public NativeMATERIAL* mat_sky;
            public NativeMATERIAL* mat_shadow;
            public NativeMATERIAL* mat_unlit;

            public NativeSTRING* debug_str;
            public NativeSTRING* watch_str;
            public NativeSTRING* delimit_str;
            public NativeSTRING* tex_name;
            public NativeSTRING* app_name;		// name of the main script without extension
            public NativeSTRING* level_name;
            public NativeSTRING* player_name;
            public NativeSTRING* session_name;
            public NativeSTRING* server_name;
            public NativeSTRING* server_ip;
            public NativeSTRING* work_dir;
            public NativeSTRING* save_dir;
            public NativeSTRING* exe_dir;

            public NativeENTITY* player;
            public NativeSTRING* user_name;		// registered user name, 19 chars
            public NativeENTITY* passable_ent;
            public NativeSTRING* path_name;		// path name of the last opened object file
            public NativeSTRING* command_str;	// command line
            public NativeENTITY* level_ent;      // pseudo entity representing the level
            public NativePANEL* mouse_panel;    // panel under mouse
            public int epunused1_1;

            public void* render_d3dmaterial;	// current LPD3DMATERIAL9
            public void* render_d3dxeffect;	// current LPD3DXEFFECT
            public NativeMATERIAL* render_material;	// current material 
            public NativeBMAP* render_target;		// render target of the previous view stage
            public NativeBMAP* render_stencil;		// stencil map of the previous view stage
            public void* render_zbuffer;     // render target depth buffer LPDIRECT3DSURFACE9
            public NativeBMAP* video_overlay;		// engine window overlay
            public int epunused2_1;
            public int epunused2_2;
            public NativeCONTACT* hit;

            // engine matrices
            public float* matTexture;
            public float* matEffect;
            public float* matView;
            public float* matViewInv;
            public float* matProj;
            public float* matWorld;
            public float* matViewProj;

            /* dummyXX - (in engine_sdk engVar from "engine functions" to "DirectX interfaces" */
            public void* dummy1;
            public void* dummy2;
            public void* dummy3;
            public void* dummy4;
            public void* dummy5;
            public void* dummy6;
            public void* dummy7;
            public void* dummy8;
            public void* dummy9;
            public void* dummy10;
            public void* dummy11;
            public void* dummy12;
            public void* dummy13;
            public void* dummy14;
            public void* dummy15;
            public void* dummy16;
            public void* dummy17;
            public void* dummy18;
            public void* dummy19;
            public void* dummy20;
            public void* dummy21;
            public void* dummy22;

            public void* pdi;			// pointer to the DirectInput object 
            public void* pdimouse;		// pointer to the DirectInput mouse device
            public void* pdikbd;		// not used - keyboard doesn't use DirectInput
            public void* pdijoy;		// not used  -joystick doesn't use DirectInput

            public void* pdplayclient; // pointer to the DirectPlayClient interface 
            public void* pdplayserver; // pointer to the DirectPlayServer interface 

            public void* dunused1_1;	// not used 
            public void* dunused1_2;	// not used 

            public void* pds;			// pointer to the IDirectSound interface
            public void* pdsb;			// pointer to the primary IDirectSoundBuffer

            public void* pd3d;			// pointer to the Direct3D interface 
            public void* pd3ddev;		// pointer to the IDirect3DDevice
            public IntPtr hWnd;			// main app window
            public void* pd3dcaps;		// pointer to the current D3DCAPS9

            public int hInstance;	// app hinstance
            public IntPtr hWndTarget;	// target window
            public IntPtr pRectTarget;	// target rectangle
            public IntPtr pPaths;      // NULL-terminated array of path names
            public void* pVars;	    // begin of the variables
            public IntPtr hWndParent;	// parent window
            public void* dunused2_1;	// not used
            public void* dunused2_2;	// not used
            public void* dunused2_3;	// not used
            public void* pdsbs;		// pointer to the recently created secondary IDirectSoundBuffer

            public float* mat_effect1;
            public float* mat_effect2;
            public float* mat_effect3;
            public float* mat_effect4;
            public float* mat_effect5;
            public float* mat_effect6;
            public float* mat_effect7;
            public float* mat_effect8;
        }
    }
}