﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using System.Runtime.InteropServices;
using AcknexWrapper.Native;

namespace AcknexWrapper
{
    public class PANEL
    {
        private unsafe NativePANEL* PANELPointer;

        /// <summary>
        /// constructor that initializes the object
        /// </summary>
        public PANEL(IntPtr param0)
        {
            unsafe
            {
                PANELPointer = (NativePANEL*)param0;
            }
        }

        /// <summary>
        /// bmap - see Gamestudio manual
        /// </summary>
        public BMAP bmap
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*PANELPointer).bmap;
                }
            }
            set
            {
                unsafe
                {
                    (*PANELPointer).bmap = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// layer - see Gamestudio manual
        /// </summary>
        private Var layer_ = null;
        public Var layer
        {
            get
            {
                unsafe
                {
                    if (layer_ == null)
                    {
                        layer_ = (Var)(IntPtr)(&((*PANELPointer).layer));
                    }
                }
                return layer_;
            }
        }

        /// <summary>
        /// pos_x - see Gamestudio manual
        /// </summary>
        private Var pos_x_ = null;
        public Var pos_x
        {
            get
            {
                unsafe
                {
                    if (pos_x_ == null)
                    {
                        pos_x_ = (Var)(IntPtr)(&((*PANELPointer).pos_x));
                    }
                }
                return pos_x_;
            }
        }

        /// <summary>
        /// pos_y - see Gamestudio manual
        /// </summary>
        private Var pos_y_ = null;
        public Var pos_y
        {
            get
            {
                unsafe
                {
                    if (pos_y_ == null)
                    {
                        pos_y_ = (Var)(IntPtr)(&((*PANELPointer).pos_y));
                    }
                }
                return pos_y_;
            }
        }

        /// <summary>
        /// size_x - see Gamestudio manual
        /// </summary>
        private Var size_x_ = null;
        public Var size_x
        {
            get
            {
                unsafe
                {
                    if (size_x_ == null)
                    {
                        size_x_ = (Var)(IntPtr)(&((*PANELPointer).size_x));
                    }
                }
                return size_x_;
            }
        }

        /// <summary>
        /// size_y - see Gamestudio manual
        /// </summary>
        private Var size_y_ = null;
        public Var size_y
        {
            get
            {
                unsafe
                {
                    if (size_y_ == null)
                    {
                        size_y_ = (Var)(IntPtr)(&((*PANELPointer).size_y));
                    }
                }
                return size_y_;
            }
        }

        /// <summary>
        /// scale_x - see Gamestudio manual
        /// </summary>
        private Var scale_x_ = null;
        public Var scale_x
        {
            get
            {
                unsafe
                {
                    if (scale_x_ == null)
                    {
                        scale_x_ = (Var)(IntPtr)(&((*PANELPointer).scale_x));
                    }
                }
                return scale_x_;
            }
        }

        /// <summary>
        /// scale_y - see Gamestudio manual
        /// </summary>
        private Var scale_y_ = null;
        public Var scale_y
        {
            get
            {
                unsafe
                {
                    if (scale_y_ == null)
                    {
                        scale_y_ = (Var)(IntPtr)(&((*PANELPointer).scale_y));
                    }
                }
                return scale_y_;
            }
        }

        /// <summary>
        /// center_x - see Gamestudio manual
        /// </summary>
        private Var center_x_ = null;
        public Var center_x
        {
            get
            {
                unsafe
                {
                    if (center_x_ == null)
                    {
                        center_x_ = (Var)(IntPtr)(&((*PANELPointer).center_x));
                    }
                }
                return center_x_;
            }
        }

        /// <summary>
        /// center_y - see Gamestudio manual
        /// </summary>
        private Var center_y_ = null;
        public Var center_y
        {
            get
            {
                unsafe
                {
                    if (center_y_ == null)
                    {
                        center_y_ = (Var)(IntPtr)(&((*PANELPointer).center_y));
                    }
                }
                return center_y_;
            }
        }

        /// <summary>
        /// angle - see Gamestudio manual
        /// </summary>
        private Var angle_ = null;
        public Var angle
        {
            get
            {
                unsafe
                {
                    if (angle_ == null)
                    {
                        angle_ = (Var)(IntPtr)(&((*PANELPointer).angle));
                    }
                }
                return angle_;
            }
        }

        /// <summary>
        /// alpha - see Gamestudio manual
        /// </summary>
        private Var alpha_ = null;
        public Var alpha
        {
            get
            {
                unsafe
                {
                    if (alpha_ == null)
                    {
                        alpha_ = (Var)(IntPtr)(&((*PANELPointer).alpha));
                    }
                }
                return alpha_;
            }
        }

        /// <summary>
        /// PANEL flags
        /// </summary>
        public enum FLAG : int
        {
            UNTOUCHABLE = (1 << 0),
            SHOW = 1 << 14,
            OVERLAY = 1 << 12,
            TRANSLUCENT = 1 << 10,
            FILTER = 1 << 20,
            LIGHT = 1 << 19,
            CENTER_X = 1 << 8,
            CENTER_Y = 1 << 9,
            SHADOW = 1 << 18,
            OUTLINE = 1 << 15
        }

        /// <summary>
        /// flags - see Gamestudio manual
        /// </summary>
        public int flags
        {
            get
            {
                unsafe
                {
                    return (*PANELPointer).flags;
                }
            }
            set
            {
                unsafe
                {
                    (*PANELPointer).flags = value;
                }
            }
        }

        /// <summary>
        /// show - see Gamestudio manual
        /// </summary>
        public Boolean SHOW
        {
            get
            {
                unsafe
                {
                    if (((*PANELPointer).flags & (int)FLAG.SHOW) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PANELPointer).flags = (*PANELPointer).flags | (int)FLAG.SHOW;
                    else
                        (*PANELPointer).flags = (*PANELPointer).flags & ~(int)FLAG.SHOW;

                }
            }
        }

        /// <summary>
        /// overlay - see Gamestudio manual
        /// </summary>
        public Boolean OVERLAY
        {
            get
            {
                unsafe
                {
                    if (((*PANELPointer).flags & (int)FLAG.OVERLAY) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PANELPointer).flags = (*PANELPointer).flags | (int)FLAG.OVERLAY;
                    else
                        (*PANELPointer).flags = (*PANELPointer).flags & ~(int)FLAG.OVERLAY;

                }
            }
        }

        /// <summary>
        /// translucent
        /// </summary>
        public Boolean TRANSLUCENT
        {
            get
            {
                unsafe
                {
                    if (((*PANELPointer).flags & (int)FLAG.TRANSLUCENT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PANELPointer).flags = (*PANELPointer).flags | (int)FLAG.TRANSLUCENT;
                    else
                        (*PANELPointer).flags = (*PANELPointer).flags & ~(int)FLAG.TRANSLUCENT;

                }
            }
        }

        /// <summary>
        /// filter - see Gamestudio manual
        /// </summary>
        public Boolean FILTER
        {
            get
            {
                unsafe
                {
                    if (((*PANELPointer).flags & (int)FLAG.FILTER) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PANELPointer).flags = (*PANELPointer).flags | (int)FLAG.FILTER;
                    else
                        (*PANELPointer).flags = (*PANELPointer).flags & ~(int)FLAG.FILTER;

                }
            }
        }

        /// <summary>
        /// light - see Gamestudio manual
        /// </summary>
        public Boolean LIGHT
        {
            get
            {
                unsafe
                {
                    if (((*PANELPointer).flags & (int)FLAG.LIGHT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PANELPointer).flags = (*PANELPointer).flags | (int)FLAG.LIGHT;
                    else
                        (*PANELPointer).flags = (*PANELPointer).flags & ~(int)FLAG.LIGHT;

                }
            }
        }

        /// <summary>
        /// center_x - see Gamestudio manual
        /// </summary>
        public Boolean CENTER_X
        {
            get
            {
                unsafe
                {
                    if (((*PANELPointer).flags & (int)FLAG.CENTER_X) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PANELPointer).flags = (*PANELPointer).flags | (int)FLAG.CENTER_X;
                    else
                        (*PANELPointer).flags = (*PANELPointer).flags & ~(int)FLAG.CENTER_X;

                }
            }
        }

        /// <summary>
        /// center_y - see Gamestudio manual
        /// </summary>
        public Boolean CENTER_Y
        {
            get
            {
                unsafe
                {
                    if (((*PANELPointer).flags & (int)FLAG.CENTER_Y) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PANELPointer).flags = (*PANELPointer).flags | (int)FLAG.CENTER_Y;
                    else
                        (*PANELPointer).flags = (*PANELPointer).flags & ~(int)FLAG.CENTER_Y;

                }
            }
        }

        /// <summary>
        /// shadow - see Gamestudio manual
        /// </summary>
        public Boolean SHADOW
        {
            get
            {
                unsafe
                {
                    if (((*PANELPointer).flags & (int)FLAG.SHADOW) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PANELPointer).flags = (*PANELPointer).flags | (int)FLAG.SHADOW;
                    else
                        (*PANELPointer).flags = (*PANELPointer).flags & ~(int)FLAG.SHADOW;

                }
            }
        }

        /// <summary>
        /// outline - see Gamestudio manual
        /// </summary>
        public Boolean OUTLINE
        {
            get
            {
                unsafe
                {
                    if (((*PANELPointer).flags & (int)FLAG.OUTLINE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PANELPointer).flags = (*PANELPointer).flags | (int)FLAG.OUTLINE;
                    else
                        (*PANELPointer).flags = (*PANELPointer).flags & ~(int)FLAG.OUTLINE;

                }
            }
        }


        /// <summary>
        /// untouchable - see Gamestudio manual
        /// </summary>
        public Boolean UNTOUCHABLE
        {
            get
            {
                unsafe
                {
                    if (((*PANELPointer).flags & (int)FLAG.UNTOUCHABLE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PANELPointer).flags = (*PANELPointer).flags | (int)FLAG.UNTOUCHABLE;
                    else
                        (*PANELPointer).flags = (*PANELPointer).flags & ~(int)FLAG.UNTOUCHABLE;
                }
            }
        }

        /// <summary>
        /// red - see Gamestudio manual
        /// </summary>
        private Var red_ = null;
        public Var red
        {
            get
            {
                unsafe
                {
                    if (red_ == null)
                    {
                        red_ = (Var)(IntPtr)(&((*PANELPointer).red));
                    }
                }
                return red_;
            }
        }

        /// <summary>
        /// green - see Gamestudio manual
        /// </summary>
        private Var green_ = null;
        public Var green
        {
            get
            {
                unsafe
                {
                    if (green_ == null)
                    {
                        green_ = (Var)(IntPtr)(&((*PANELPointer).green));
                    }
                }
                return green_;
            }
        }

        /// <summary>
        /// blue - see Gamestudio manual
        /// </summary>
        private Var blue_ = null;
        public Var blue
        {
            get
            {
                unsafe
                {
                    if (blue_ == null)
                    {
                        blue_ = (Var)(IntPtr)(&((*PANELPointer).blue));
                    }
                }
                return blue_;
            }
        }

        /// <summary>
        /// greenBlueRed - not listed in the Gamestudio manual - allows to access scale_x, scale_y and scale_z as a Vector
        /// </summary>
        private Color greenBlueRed_ = null;
        public Color greenBlueRed
        {
            get
            {
                unsafe
                {
                    if (greenBlueRed_ == null)
                    {
                        greenBlueRed_ = new Color((Var)0, (Var)0, (Var)0);
                        greenBlueRed_.red = (Var)(IntPtr)(&((*PANELPointer).red));
                        greenBlueRed_.green = (Var)(IntPtr)(&((*PANELPointer).green));
                        greenBlueRed_.blue = (Var)(IntPtr)(&((*PANELPointer).blue));
                    }
                }
                return greenBlueRed_;
            }
        }

        /// <summary>
        /// mouse_map - see Gamestudio manual
        /// </summary>
        public BMAP mouse_map
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*PANELPointer).mouse_map;
                }
            }
            set
            {
                unsafe
                {
                    (*PANELPointer).mouse_map = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// target_map - see Gamestudio manual
        /// </summary>
        public BMAP target_map
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*PANELPointer).target_map;
                }
            }
            set
            {
                unsafe
                {
                    (*PANELPointer).target_map = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// event - see Gamestudio manual
        /// </summary>
        public EventIntPtr event_
        {
            set
            {
                unsafe
                {
                    Scheduler.EventIntPtrReference TempEventIntPtrReference = new Scheduler.EventIntPtrReference(value, (IntPtr)PANELPointer);
                    (*PANELPointer).event_.mPtr = (byte*)(void*)Marshal.GetFunctionPointerForDelegate((NativeEventIntPtr)TempEventIntPtrReference.NativeEventIntPtrFunc);
                }
            }
        }

        /// <summary>
        /// skill_x - see Gamestudio manual
        /// </summary>
        private Var skill_x_ = null;
        public Var skill_x
        {
            get
            {
                unsafe
                {
                    if (skill_x_ == null)
                    {
                        skill_x_ = (Var)(IntPtr)(&((*PANELPointer).skill_x));
                    }
                }
                return skill_x_;
            }
        }

        /// <summary>
        /// skill_y - see Gamestudio manual
        /// </summary>
        private Var skill_y_ = null;
        public Var skill_y
        {
            get
            {
                unsafe
                {
                    if (skill_y_ == null)
                    {
                        skill_y_ = (Var)(IntPtr)(&((*PANELPointer).skill_y));
                    }
                }
                return skill_y_;
            }
        }

        /* -------- PANEL -------- */
        /* PANEL to NativePANEL* (address in void*) */
        public static explicit operator IntPtr(PANEL d)
        {
            unsafe
            {
                if (d == null)
                {
                    return IntPtr.Zero;
                }
                else
                {
                    return (IntPtr)(NativePANEL*)d.PANELPointer;
                }
            }
        }
        /* NativePANEL* (address in void*) to PANEL */
        public static explicit operator PANEL(IntPtr d)
        {
            unsafe
            {
                if (d == IntPtr.Zero)
                {
                    return null;
                }
                return new PANEL(d);
            }
        }
    }
}