﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using System.Runtime.InteropServices;
using AcknexWrapper.Native;

namespace AcknexWrapper
{
    public class PARTICLE
    {
        private unsafe NativePARTICLE* PARTICLEPointer;

        /// <summary>
        /// constructor that initializes the object
        /// </summary>
        public PARTICLE(IntPtr param0)
        {
            unsafe
            {
                PARTICLEPointer = (NativePARTICLE*)param0;
            }
        }

        /// <summary>
        /// x - see Gamestudio manual
        /// </summary>
        private Var x_ = null;
        public Var x
        {
            get
            {
                unsafe
                {
                    if (x_ == null)
                    {
                        x_ = (Var)(IntPtr)(&((*PARTICLEPointer).x));
                    }
                }
                return x_;
            }
        }

        /// <summary>
        /// y - see Gamestudio manual
        /// </summary>
        private Var y_ = null;
        public Var y
        {
            get
            {
                unsafe
                {
                    if (y_ == null)
                    {
                        y_ = (Var)(IntPtr)(&((*PARTICLEPointer).y));
                    }
                }
                return y_;
            }
        }

        /// <summary>
        /// z - see Gamestudio manual
        /// </summary>
        private Var z_ = null;
        public Var z
        {
            get
            {
                unsafe
                {
                    if (z_ == null)
                    {
                        z_ = (Var)(IntPtr)(&((*PARTICLEPointer).z));
                    }
                }
                return z_;
            }
        }

        /// <summary>
        /// xyz - not listed in the Gamestudio manual - allows to access x, y and z as a Vector
        /// </summary>
        private Vector xyz_ = null;
        public Vector xyz
        {
            get
            {
                unsafe
                {
                    if (xyz_ == null)
                    {
                        xyz_ = new Vector((Var)0, (Var)0, (Var)0);
                        xyz_.x = (Var)(IntPtr)(&((*PARTICLEPointer).x));
                        xyz_.y = (Var)(IntPtr)(&((*PARTICLEPointer).y));
                        xyz_.z = (Var)(IntPtr)(&((*PARTICLEPointer).z));
                    }
                }
                return xyz_;
            }
        }

        /// <summary>
        /// red - see Gamestudio manual
        /// </summary>
        private Var red_ = null;
        public Var red
        {
            get
            {
                unsafe
                {
                    if (red_ == null)
                    {
                        red_ = (Var)(IntPtr)(&((*PARTICLEPointer).red));
                    }
                }
                return red_;
            }
        }

        /// <summary>
        /// green - see Gamestudio manual
        /// </summary>
        private Var green_ = null;
        public Var green
        {
            get
            {
                unsafe
                {
                    if (green_ == null)
                    {
                        green_ = (Var)(IntPtr)(&((*PARTICLEPointer).green));
                    }
                }
                return green_;
            }
        }

        /// <summary>
        /// blue - see Gamestudio manual
        /// </summary>
        private Var blue_ = null;
        public Var blue
        {
            get
            {
                unsafe
                {
                    if (blue_ == null)
                    {
                        blue_ = (Var)(IntPtr)(&((*PARTICLEPointer).blue));
                    }
                }
                return blue_;
            }
        }

        /// <summary>
        /// greenBlueRed - not listed in the Gamestudio manual - allows to access scale_x, scale_y and scale_z as a Vector
        /// </summary>
        private Color greenBlueRed_ = null;
        public Color greenBlueRed
        {
            get
            {
                unsafe
                {
                    if (greenBlueRed_ == null)
                    {
                        greenBlueRed_ = new Color((Var)0, (Var)0, (Var)0);
                        greenBlueRed_.red = (Var)(IntPtr)(&((*PARTICLEPointer).red));
                        greenBlueRed_.green = (Var)(IntPtr)(&((*PARTICLEPointer).green));
                        greenBlueRed_.blue = (Var)(IntPtr)(&((*PARTICLEPointer).blue));
                    }
                }
                return greenBlueRed_;
            }
        }

        /// <summary>
        /// alpha - see Gamestudio manual
        /// </summary>
        private Var alpha_ = null;
        public Var alpha
        {
            get
            {
                unsafe
                {
                    if (alpha_ == null)
                    {
                        alpha_ = (Var)(IntPtr)(&((*PARTICLEPointer).alpha));
                    }
                }
                return alpha_;
            }
        }

        /// <summary>
        /// bmap - see Gamestudio manual
        /// </summary>
        public BMAP bmap
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*PARTICLEPointer).bmap;
                }
            }
            set
            {
                unsafe
                {
                    (*PARTICLEPointer).bmap = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// material - see Gamestudio manual
        /// </summary>
        public MATERIAL material
        {
            get
            {
                unsafe
                {
                    return (MATERIAL)(IntPtr)(*PARTICLEPointer).material;
                }
            }
            set
            {
                unsafe
                {
                    (*PARTICLEPointer).material = (NativeMATERIAL*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// event - see Gamestudio manual
        /// </summary>
        public EventPARTICLE event_
        {
            set
            {
                unsafe
                {
                    (*PARTICLEPointer).event_.mPtr = (byte*)(void*)Marshal.GetFunctionPointerForDelegate((NativeEventPARTICLE)(new Scheduler.EventParticleReference(value)).NativeEventParticleFunc);
                }
            }
        }

        /// <summary>
        /// lifespan - see Gamestudio manual
        /// </summary>
        private Var lifespan_ = null;
        public Var lifespan
        {
            get
            {
                unsafe
                {
                    if (lifespan_ == null)
                    {
                        lifespan_ = (Var)(IntPtr)(&((*PARTICLEPointer).lifespan));
                    }
                }
                return lifespan_;
            }
        }

        /// <summary>
        /// size - see Gamestudio manual
        /// </summary>
        private Var size_ = null;
        public Var size
        {
            get
            {
                unsafe
                {
                    if (size_ == null)
                    {
                        size_ = (Var)(IntPtr)(&((*PARTICLEPointer).size));
                    }
                }
                return size_;
            }
        }

        /// <summary>
        /// gravity - see Gamestudio manual
        /// </summary>
        private Var gravity_ = null;
        public Var gravity
        {
            get
            {
                unsafe
                {
                    if (gravity_ == null)
                    {
                        gravity_ = (Var)(IntPtr)(&((*PARTICLEPointer).gravity));
                    }
                }
                return gravity_;
            }
        }

        /// <summary>
        /// vel_x - see Gamestudio manual
        /// </summary>
        private Var vel_x_ = null;
        public Var vel_x
        {
            get
            {
                unsafe
                {
                    if (vel_x_ == null)
                    {
                        vel_x_ = (Var)(IntPtr)(&((*PARTICLEPointer).vel_x));
                    }
                }
                return vel_x_;
            }
        }

        /// <summary>
        /// vel_y - see Gamestudio manual
        /// </summary>
        private Var vel_y_ = null;
        public Var vel_y
        {
            get
            {
                unsafe
                {
                    if (vel_y_ == null)
                    {
                        vel_y_ = (Var)(IntPtr)(&((*PARTICLEPointer).vel_y));
                    }
                }
                return vel_y_;
            }
        }

        /// <summary>
        /// vel_x - see Gamestudio manual
        /// </summary>
        private Var vel_z_ = null;
        public Var vel_z
        {
            get
            {
                unsafe
                {
                    if (vel_z_ == null)
                    {
                        vel_z_ = (Var)(IntPtr)(&((*PARTICLEPointer).vel_z));
                    }
                }
                return vel_z_;
            }
        }

        /// <summary>
        /// vel_xyz - not listed in the Gamestudio manual - allows to access x, y and z as a Vector
        /// </summary>
        private Vector vel_xyz_ = null;
        public Vector vel_xyz
        {
            get
            {
                unsafe
                {
                    if (vel_xyz_ == null)
                    {
                        vel_xyz_ = new Vector((Var)0, (Var)0, (Var)0);
                        vel_xyz_.x = (Var)(IntPtr)(&((*PARTICLEPointer).vel_x));
                        vel_xyz_.y = (Var)(IntPtr)(&((*PARTICLEPointer).vel_y));
                        vel_xyz_.z = (Var)(IntPtr)(&((*PARTICLEPointer).vel_z));
                    }
                }
                return vel_xyz_;
            }
        }

        /// <summary>
        /// skill_a - see Gamestudio manual
        /// </summary>
        private Var skill_a_ = null;
        public Var skill_a
        {
            get
            {
                unsafe
                {
                    if (skill_a_ == null)
                    {
                        skill_a_ = (Var)(IntPtr)(&((*PARTICLEPointer).skill_0));
                    }
                }
                return skill_a_;
            }
        }

        /// <summary>
        /// skill_b - see Gamestudio manual
        /// </summary>
        private Var skill_b_ = null;
        public Var skill_b
        {
            get
            {
                unsafe
                {
                    if (skill_b_ == null)
                    {
                        skill_b_ = (Var)(IntPtr)(&((*PARTICLEPointer).skill_1));
                    }
                }
                return skill_b_;
            }
        }

        /// <summary>
        /// skill_c - see Gamestudio manual
        /// </summary>
        private Var skill_c_ = null;
        public Var skill_c
        {
            get
            {
                unsafe
                {
                    if (skill_c_ == null)
                    {
                        skill_c_ = (Var)(IntPtr)(&((*PARTICLEPointer).skill_2));
                    }
                }
                return skill_c_;
            }
        }

        /// <summary>
        /// skill_d - see Gamestudio manual
        /// </summary>
        private Var skill_d_ = null;
        public Var skill_d
        {
            get
            {
                unsafe
                {
                    if (skill_d_ == null)
                    {
                        skill_d_ = (Var)(IntPtr)(&((*PARTICLEPointer).skill_3));
                    }
                }
                return skill_d_;
            }
        }

        /// <summary>
        /// skill_x - see Gamestudio manual
        /// </summary>
        private Var skill_x_ = null;
        public Var skill_x
        {
            get
            {
                unsafe
                {
                    if (skill_x_ == null)
                    {
                        skill_x_ = (Var)(IntPtr)(&((*PARTICLEPointer).skill_x));
                    }
                }
                return skill_x_;
            }
        }

        /// <summary>
        /// skill_y - see Gamestudio manual
        /// </summary>
        private Var skill_y_ = null;
        public Var skill_y
        {
            get
            {
                unsafe
                {
                    if (skill_y_ == null)
                    {
                        skill_y_ = (Var)(IntPtr)(&((*PARTICLEPointer).skill_y));
                    }
                }
                return skill_y_;
            }
        }

        /// <summary>
        /// skill_z - see Gamestudio manual
        /// </summary>
        private Var skill_z_ = null;
        public Var skill_z
        {
            get
            {
                unsafe
                {
                    if (skill_z_ == null)
                    {
                        skill_z_ = (Var)(IntPtr)(&((*PARTICLEPointer).skill_z));
                    }
                }
                return skill_z_;
            }
        }

        /// <summary>
        /// skill_xyz - not listed in the Gamestudio manual - allows to access skill_x, skill_y and skill_z as a Vector
        /// </summary>
        private Vector skill_xyz_ = null;
        public Vector skill_xyz
        {
            get
            {
                unsafe
                {
                    if (skill_xyz_ == null)
                    {
                        skill_xyz_ = new Vector((Var)0, (Var)0, (Var)0);
                        skill_xyz_.x = (Var)(IntPtr)(&((*PARTICLEPointer).skill_x));
                        skill_xyz_.y = (Var)(IntPtr)(&((*PARTICLEPointer).skill_y));
                        skill_xyz_.z = (Var)(IntPtr)(&((*PARTICLEPointer).skill_z));
                    }
                }
                return skill_xyz_;
            }
        }


        public enum FLAG : int
        {
            FLAG1 = 1 << 0,
            FLAG2 = 1 << 1,
            FLAG3 = 1 << 2,
            FLAG4 = 1 << 3,
            FLAG5 = 1 << 4,
            FLAG6 = 1 << 5,
            FLAG7 = 1 << 6,
            FLAG8 = 1 << 7,
            STREAK = 1 << 8,
            BEAM = 1 << 9,
            TRANSLUCENT = 1 << 10,
            MOVE = 1 << 11,
            OVERLAY = 1 << 12,
            NOFILTER = 1 << 16,
            BRIGHT = 1 << 21
        }

        /// <summary>
        /// flags - see Gamestudio manual
        /// </summary>
        public int flags
        {
            get
            {
                unsafe
                {
                    return (*PARTICLEPointer).flags;
                }
            }
            set
            {
                unsafe
                {
                    (*PARTICLEPointer).flags = value;
                }
            }
        }

        /// <summary>
        /// flag1 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG1
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.FLAG1) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.FLAG1;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.FLAG1;

                }
            }
        }

        /// <summary>
        /// flag2 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG2
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.FLAG2) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.FLAG2;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.FLAG2;

                }
            }
        }

        /// <summary>
        /// flag3 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG3
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.FLAG3) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.FLAG3;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.FLAG3;

                }
            }
        }

        /// <summary>
        /// flag4 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG4
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.FLAG4) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.FLAG4;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.FLAG4;

                }
            }
        }

        /// <summary>
        /// flag5 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG5
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.FLAG5) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.FLAG5;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.FLAG5;

                }
            }
        }

        /// <summary>
        /// flag6 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG6
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.FLAG6) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.FLAG6;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.FLAG6;

                }
            }
        }

        /// <summary>
        /// flag7 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG7
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.FLAG7) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.FLAG7;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.FLAG7;

                }
            }
        }

        /// <summary>
        /// flag8 - see Gamestudio manual
        /// </summary>
        public Boolean FLAG8
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.FLAG8) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.FLAG8;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.FLAG8;

                }
            }
        }

        /// <summary>
        /// translucent - see Gamestudio manual
        /// </summary>
        public Boolean TRANSLUCENT
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.TRANSLUCENT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.TRANSLUCENT;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.TRANSLUCENT;

                }
            }
        }

        /// <summary>
        /// bright - see Gamestudio manual
        /// </summary>
        public Boolean BRIGHT
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.BRIGHT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.BRIGHT;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.BRIGHT;

                }
            }
        }

        /// <summary>
        /// overlay - see Gamestudio manual
        /// </summary>
        public Boolean OVERLAY
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.OVERLAY) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.OVERLAY;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.OVERLAY;

                }
            }
        }

        /// <summary>
        /// nofilter - see Gamestudio manual
        /// </summary>
        public Boolean NOFILTER
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.NOFILTER) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.NOFILTER;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.NOFILTER;

                }
            }
        }

        /// <summary>
        /// beam - see Gamestudio manual
        /// </summary>
        public Boolean BEAM
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.BEAM) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.BEAM;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.BEAM;

                }
            }
        }

        /// <summary>
        /// streak - see Gamestudio manual
        /// </summary>
        public Boolean STREAK
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.STREAK) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.STREAK;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.STREAK;

                }
            }
        }

        /// <summary>
        /// move - see Gamestudio manual
        /// </summary>
        public Boolean MOVE
        {
            get
            {
                unsafe
                {
                    if (((*PARTICLEPointer).flags & (int)FLAG.MOVE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags | (int)FLAG.MOVE;
                    else
                        (*PARTICLEPointer).flags = (*PARTICLEPointer).flags & ~(int)FLAG.MOVE;

                }
            }
        }




        /* -------- PARTICLE -------- */
        /* PARTICLE to NativePARTICLE* (address in void*) */
        public static explicit operator IntPtr(PARTICLE d)
        {
            unsafe
            {
                if (d == null)
                {
                    return IntPtr.Zero;
                }
                else
                {
                    return (IntPtr)(NativePARTICLE*)d.PARTICLEPointer;
                }
            }
        }
        /* NativePARTICLE* (address in void*) to PARTICLE */
        public static explicit operator PARTICLE(IntPtr d)
        {
            unsafe
            {
                if (d == IntPtr.Zero)
                {
                    return null;
                }
                return new PARTICLE(d);
            }
        }
    }
}