﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using System.Runtime.InteropServices;
using AcknexWrapper.Native;

namespace AcknexWrapper
{
    public class VIEW
    {
        private unsafe NativeVIEW* VIEWPointer;

        /// <summary>
        /// constructor that initializes the object
        /// </summary>
        public VIEW(IntPtr param0)
        {
            unsafe
            {
                VIEWPointer = (NativeVIEW*)param0;
            }
        }

        /// <summary>
        /// layer - see Gamestudio manual
        /// </summary>
        private Var layer_ = null;
        public Var layer
        {
            get
            {
                unsafe
                {
                    if (layer_ == null)
                    {
                        layer_ = (Var)(IntPtr)(&((*VIEWPointer).layer));
                    }
                }
                return layer_;
            }
        }

        /// <summary>
        /// pos_x - see Gamestudio manual
        /// </summary>
        private Var pos_x_ = null;
        public Var pos_x
        {
            get
            {
                unsafe
                {
                    if (pos_x_ == null)
                    {
                        pos_x_ = (Var)(IntPtr)(&((*VIEWPointer).pos_x));
                    }
                }
                return pos_x_;
            }
        }

        /// <summary>
        /// pos_y - see Gamestudio manual
        /// </summary>
        private Var pos_y_ = null;
        public Var pos_y
        {
            get
            {
                unsafe
                {
                    if (pos_y_ == null)
                    {
                        pos_y_ = (Var)(IntPtr)(&((*VIEWPointer).pos_y));
                    }
                }
                return pos_y_;
            }
        }

        /// <summary>
        /// size_x - see Gamestudio manual
        /// </summary>
        private Var size_x_ = null;
        public Var size_x
        {
            get
            {
                unsafe
                {
                    if (size_x_ == null)
                    {
                        size_x_ = (Var)(IntPtr)(&((*VIEWPointer).size_x));
                    }
                }
                return size_x_;
            }
        }

        /// <summary>
        /// size_y - see Gamestudio manual
        /// </summary>
        private Var size_y_ = null;
        public Var size_y
        {
            get
            {
                unsafe
                {
                    if (size_y_ == null)
                    {
                        size_y_ = (Var)(IntPtr)(&((*VIEWPointer).size_y));
                    }
                }
                return size_y_;
            }
        }

        /// <summary>
        /// x - see Gamestudio manual
        /// </summary>
        private Var x_ = null;
        public Var x
        {
            get
            {
                unsafe
                {
                    if (x_ == null)
                    {
                        x_ = (Var)(IntPtr)(&((*VIEWPointer).x));
                    }
                }
                return x_;
            }
        }

        /// <summary>
        /// y - see Gamestudio manual
        /// </summary>
        private Var y_ = null;
        public Var y
        {
            get
            {
                unsafe
                {
                    if (y_ == null)
                    {
                        y_ = (Var)(IntPtr)(&((*VIEWPointer).y));
                    }
                }
                return y_;
            }
        }

        /// <summary>
        /// z - see Gamestudio manual
        /// </summary>
        private Var z_ = null;
        public Var z
        {
            get
            {
                unsafe
                {
                    if (z_ == null)
                    {
                        z_ = (Var)(IntPtr)(&((*VIEWPointer).z));
                    }
                }
                return z_;
            }
        }

        /// <summary>
        /// xyz - not listed in the Gamestudio manual - allows to access x, y and z as a Vector
        /// </summary>
        private Vector xyz_ = null;
        public Vector xyz
        {
            get
            {
                unsafe
                {
                    if (xyz_ == null)
                    {
                        xyz_ = new Vector((Var)0, (Var)0, (Var)0);
                        xyz_.x = (Var)(IntPtr)(&((*VIEWPointer).x));
                        xyz_.y = (Var)(IntPtr)(&((*VIEWPointer).y));
                        xyz_.z = (Var)(IntPtr)(&((*VIEWPointer).z));
                    }
                }
                return xyz_;
            }
        }

        /// <summary>
        /// pan - see Gamestudio manual
        /// </summary>
        private Var pan_ = null;
        public Var pan
        {
            get
            {
                unsafe
                {
                    if (pan_ == null)
                    {
                        pan_ = (Var)(IntPtr)(&((*VIEWPointer).pan));
                    }
                }
                return pan_;
            }
        }

        /// <summary>
        /// tilt - see Gamestudio manual
        /// </summary>
        private Var tilt_ = null;
        public Var tilt
        {
            get
            {
                unsafe
                {
                    if (tilt_ == null)
                    {
                        tilt_ = (Var)(IntPtr)(&((*VIEWPointer).tilt));
                    }
                }
                return tilt_;
            }
        }

        /// <summary>
        /// roll - see Gamestudio manual
        /// </summary>
        private Var roll_ = null;
        public Var roll
        {
            get
            {
                unsafe
                {
                    if (roll_ == null)
                    {
                        roll_ = (Var)(IntPtr)(&((*VIEWPointer).roll));
                    }
                }
                return roll_;
            }
        }

        /// <summary>
        /// panTiltRoll - not listed in the Gamestudio manual - allows to access pan, tilt and roll as a Vector
        /// </summary>
        private Angle panTiltRoll_ = null;
        public Angle panTiltRoll
        {
            get
            {
                unsafe
                {
                    if (panTiltRoll_ == null)
                    {
                        panTiltRoll_ = new Angle((Var)0, (Var)0, (Var)0);
                        panTiltRoll_.pan = (Var)(IntPtr)(&((*VIEWPointer).pan));
                        panTiltRoll_.tilt = (Var)(IntPtr)(&((*VIEWPointer).tilt));
                        panTiltRoll_.roll = (Var)(IntPtr)(&((*VIEWPointer).roll));
                    }
                }
                return panTiltRoll_;
            }
        }

        /// <summary>
        /// alpha - see Gamestudio manual
        /// </summary>
        private Var alpha_ = null;
        public Var alpha
        {
            get
            {
                unsafe
                {
                    if (alpha_ == null)
                    {
                        alpha_ = (Var)(IntPtr)(&((*VIEWPointer).alpha));
                    }
                }
                return alpha_;
            }
        }

        /// <summary>
        /// ambient - see Gamestudio manual
        /// </summary>
        private Var ambient_ = null;
        public Var ambient
        {
            get
            {
                unsafe
                {
                    if (ambient_ == null)
                    {
                        ambient_ = (Var)(IntPtr)(&((*VIEWPointer).ambient));
                    }
                }
                return ambient_;
            }
        }

        /// <summary>
        /// arc - see Gamestudio manual
        /// </summary>
        private Var arc_ = null;
        public Var arc
        {
            get
            {
                unsafe
                {
                    if (arc_ == null)
                    {
                        arc_ = (Var)(IntPtr)(&((*VIEWPointer).arc));
                    }
                }
                return arc_;
            }
        }

        /// <summary>
        /// aspect - see Gamestudio manual
        /// </summary>
        private Var aspect_ = null;
        public Var aspect
        {
            get
            {
                unsafe
                {
                    if (aspect_ == null)
                    {
                        aspect_ = (Var)(IntPtr)(&((*VIEWPointer).aspect));
                    }
                }
                return aspect_;
            }
        }

        /// <summary>
        /// lod - see Gamestudio manual
        /// </summary>
        private Var lod_ = null;
        public Var lod
        {
            get
            {
                unsafe
                {
                    if (lod_ == null)
                    {
                        lod_ = (Var)(IntPtr)(&((*VIEWPointer).lod));
                    }
                }
                return lod_;
            }
        }

        /// <summary>
        /// left - see Gamestudio manual
        /// </summary>
        private Var left_ = null;
        public Var left
        {
            get
            {
                unsafe
                {
                    if (left_ == null)
                    {
                        left_ = (Var)(IntPtr)(&((*VIEWPointer).left));
                    }
                }
                return left_;
            }
        }

        /// <summary>
        /// right - see Gamestudio manual
        /// </summary>
        private Var right_ = null;
        public Var right
        {
            get
            {
                unsafe
                {
                    if (right_ == null)
                    {
                        right_ = (Var)(IntPtr)(&((*VIEWPointer).right));
                    }
                }
                return right_;
            }
        }

        /// <summary>
        /// top - see Gamestudio manual
        /// </summary>
        private Var top_ = null;
        public Var top
        {
            get
            {
                unsafe
                {
                    if (top_ == null)
                    {
                        top_ = (Var)(IntPtr)(&((*VIEWPointer).top));
                    }
                }
                return top_;
            }
        }

        /// <summary>
        /// bottom - see Gamestudio manual
        /// </summary>
        private Var bottom_ = null;
        public Var bottom
        {
            get
            {
                unsafe
                {
                    if (bottom_ == null)
                    {
                        bottom_ = (Var)(IntPtr)(&((*VIEWPointer).bottom));
                    }
                }
                return bottom_;
            }
        }

        /// <summary>
        /// stage - see Gamestudio manual
        /// </summary>
        public VIEW stage
        {
            get
            {
                unsafe
                {
                    return (VIEW)(IntPtr)(*VIEWPointer).stage;
                }
            }
            set
            {
                unsafe
                {
                    (*VIEWPointer).stage = (NativeVIEW*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// bmap - see Gamestudio manual
        /// </summary>
        public BMAP bmap
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*VIEWPointer).bmap;
                }
            }
            set
            {
                unsafe
                {
                    (*VIEWPointer).bmap = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// material - see Gamestudio manual
        /// </summary>
        public MATERIAL material
        {
            get
            {
                unsafe
                {
                    return (MATERIAL)(IntPtr)(*VIEWPointer).material;
                }
            }
            set
            {
                unsafe
                {
                    (*VIEWPointer).material = (NativeMATERIAL*)(IntPtr)value;
                }
            }
        }


        /// <summary>
        /// VIEW flags (flags)
        /// </summary>
        enum FLAG : int
        {
            UNTOUCHABLE = 1 << 0,
            PROCESS_TARGET = 1 << 3,
            PROCESS_SCREEN = 1 << 4,
            ISOMETRIC = 1 << 7,
            PORTALCLIP = 1 << 8,
            TRANSLUCENT = 1 << 10,
            CHILD = 1 << 11,
            NOSHADER = 1 << 12,
            NOLOD = 1 << 13,
            SHOW = 1 << 14,
            AUDIBLE = 1 << 15,
            SHADOW = 1 << 18,
            NOCULL = 1 << 19,
            NOFLAG1 = 1 << 21,
            NOSHADOW = 1 << 22,
            NOSKY = 1 << 23,
            NOPARTICLE = 1 << 24,
            CULL_CW = 1 << 27
        }

        /// <summary>
        /// flags - see Gamestudio manual
        /// </summary>
        public int flags
        {
            get
            {
                unsafe
                {
                    return (*VIEWPointer).flags;
                }
            }
            set
            {
                unsafe
                {
                    (*VIEWPointer).flags = value;
                }
            }
        }

        /// <summary>
        /// show - see Gamestudio manual
        /// </summary>
        public Boolean SHOW
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.SHOW) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.SHOW;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.SHOW;

                }
            }
        }

        /// <summary>
        /// audible - see Gamestudio manual
        /// </summary>
        public Boolean AUDIBLE
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.AUDIBLE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.AUDIBLE;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.AUDIBLE;

                }
            }
        }

        /// <summary>
        /// isometric - see Gamestudio manual
        /// </summary>
        public Boolean ISOMETRIC
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.ISOMETRIC) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.ISOMETRIC;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.ISOMETRIC;

                }
            }
        }

        /// <summary>
        /// untouchable - see Gamestudio manual
        /// </summary>
        public Boolean UNTOUCHABLE
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.UNTOUCHABLE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.UNTOUCHABLE;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.UNTOUCHABLE;

                }
            }
        }

        /// <summary>
        /// process_screen - see Gamestudio manual
        /// </summary>
        public Boolean PROCESS_SCREEN
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.PROCESS_SCREEN) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.PROCESS_SCREEN;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.PROCESS_SCREEN;

                }
            }
        }

        /// <summary>
        /// process_target - see Gamestudio manual
        /// </summary>
        public Boolean PROCESS_TARGET
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.PROCESS_TARGET) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.PROCESS_TARGET;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.PROCESS_TARGET;

                }
            }
        }

        /// <summary>
        /// child - see Gamestudio manual
        /// </summary>
        public Boolean CHILD
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.CHILD) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.CHILD;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.CHILD;

                }
            }
        }

        /// <summary>
        /// translucent - see Gamestudio manual
        /// </summary>
        public Boolean TRANSLUCENT
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.TRANSLUCENT) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.TRANSLUCENT;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.TRANSLUCENT;

                }
            }
        }

        /// <summary>
        /// portalclip - see Gamestudio manual
        /// </summary>
        public Boolean PORTALCLIP
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.PORTALCLIP) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.PORTALCLIP;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.PORTALCLIP;

                }
            }
        }

        /// <summary>
        /// cull_cw - see Gamestudio manual
        /// </summary>
        public Boolean CULL_CW
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.CULL_CW) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.CULL_CW;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.CULL_CW;

                }
            }
        }

        /// <summary>
        /// nocull - see Gamestudio manual
        /// </summary>
        public Boolean NOCULL
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.NOCULL) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.NOCULL;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.NOCULL;

                }
            }
        }

        /// <summary>
        /// noshadow - see Gamestudio manual
        /// </summary>
        public Boolean NOSHADOW
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.NOSHADOW) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.NOSHADOW;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.NOSHADOW;

                }
            }
        }

        /// <summary>
        /// shadow - see Gamestudio manual
        /// </summary>
        public Boolean SHADOW
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.SHADOW) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.SHADOW;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.SHADOW;

                }
            }
        }

        /// <summary>
        /// NOSKY - see Gamestudio manual
        /// </summary>
        public Boolean NOSKY
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.NOSKY) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.NOSKY;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.NOSKY;

                }
            }
        }

        /// <summary>
        /// noparticle - see Gamestudio manual
        /// </summary>
        public Boolean NOPARTICLE
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.NOPARTICLE) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.NOPARTICLE;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.NOPARTICLE;

                }
            }
        }

        /// <summary>
        /// noshader - see Gamestudio manual
        /// </summary>
        public Boolean NOSHADER
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.NOSHADER) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.NOSHADER;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.NOSHADER;

                }
            }
        }

        /// <summary>
        /// noflag1 - see Gamestudio manual
        /// </summary>
        public Boolean NOFLAG1
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.NOFLAG1) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.NOFLAG1;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.NOFLAG1;

                }
            }
        }

        /// <summary>
        /// nolod - see Gamestudio manual
        /// </summary>
        public Boolean NOLOD
        {
            get
            {
                unsafe
                {
                    if (((*VIEWPointer).flags & (int)FLAG.NOLOD) == 0)
                        return false;
                    else
                        return true;
                }
            }
            set
            {
                unsafe
                {
                    if (value == true)
                        (*VIEWPointer).flags = (*VIEWPointer).flags | (int)FLAG.NOLOD;
                    else
                        (*VIEWPointer).flags = (*VIEWPointer).flags & ~(int)FLAG.NOLOD;

                }
            }
        }

        /// <summary>
        /// fog_start - see Gamestudio manual
        /// </summary>
        private Var fog_start_ = null;
        public Var fog_start
        {
            get
            {
                unsafe
                {
                    if (fog_start_ == null)
                    {
                        fog_start_ = (Var)(IntPtr)(&((*VIEWPointer).fog_start));
                    }
                }
                return fog_start_;
            }
        }

        /// <summary>
        /// fog_end - see Gamestudio manual
        /// </summary>
        private Var fog_end_ = null;
        public Var fog_end
        {
            get
            {
                unsafe
                {
                    if (fog_end_ == null)
                    {
                        fog_end_ = (Var)(IntPtr)(&((*VIEWPointer).fog_end));
                    }
                }
                return fog_end_;
            }
        }

        /// <summary>
        /// clip_near - see Gamestudio manual
        /// </summary>
        private Var clip_near_ = null;
        public Var clip_near
        {
            get
            {
                unsafe
                {
                    if (clip_near_ == null)
                    {
                        clip_near_ = (Var)(IntPtr)(&((*VIEWPointer).clip_near));
                    }
                }
                return clip_near_;
            }
        }

        /// <summary>
        /// clip_far - see Gamestudio manual
        /// </summary>
        private Var clip_far_ = null;
        public Var clip_far
        {
            get
            {
                unsafe
                {
                    if (clip_far_ == null)
                    {
                        clip_far_ = (Var)(IntPtr)(&((*VIEWPointer).clip_far));
                    }
                }
                return clip_far_;
            }
        }

        /// <summary>
        /// depth - see Gamestudio manual
        /// </summary>
        private Var depth_ = null;
        public Var depth
        {
            get
            {
                unsafe
                {
                    if (depth_ == null)
                    {
                        depth_ = (Var)(IntPtr)(&((*VIEWPointer).depth));
                    }
                }
                return depth_;
            }
        }


        /// <summary>
        /// genius - see Gamestudio manual
        /// </summary>
        public ENTITY genius
        {
            get
            {
                unsafe
                {
                    return (ENTITY)(IntPtr)(*VIEWPointer).genius;
                }
            }
            set
            {
                unsafe
                {
                    (*VIEWPointer).genius = (NativeENTITY*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// portal_x - see Gamestudio manual
        /// </summary>
        private Var portal_x_ = null;
        public Var portal_x
        {
            get
            {
                unsafe
                {
                    if (portal_x_ == null)
                    {
                        portal_x_ = (Var)(IntPtr)(&((*VIEWPointer).portal_x));
                    }
                }
                return portal_x_;
            }
        }

        /// <summary>
        /// portal_y - see Gamestudio manual
        /// </summary>
        private Var portal_y_ = null;
        public Var portal_y
        {
            get
            {
                unsafe
                {
                    if (portal_y_ == null)
                    {
                        portal_y_ = (Var)(IntPtr)(&((*VIEWPointer).portal_y));
                    }
                }
                return portal_y_;
            }
        }

        /// <summary>
        /// portal_z - see Gamestudio manual
        /// </summary>
        private Var portal_z_ = null;
        public Var portal_z
        {
            get
            {
                unsafe
                {
                    if (portal_z_ == null)
                    {
                        portal_z_ = (Var)(IntPtr)(&((*VIEWPointer).portal_z));
                    }
                }
                return portal_z_;
            }
        }

        /// <summary>
        /// portal_xyz - not listed in the Gamestudio manual - allows to access portal_x, portal_y and portal_z as a Vector
        /// </summary>
        private Vector portal_xyz_ = null;
        public Vector portal_xyz
        {
            get
            {
                unsafe
                {
                    if (portal_xyz_ == null)
                    {
                        portal_xyz_ = new Vector((Var)0, (Var)0, (Var)0);
                        portal_xyz_.x = (Var)(IntPtr)(&((*VIEWPointer).portal_x));
                        portal_xyz_.y = (Var)(IntPtr)(&((*VIEWPointer).portal_y));
                        portal_xyz_.z = (Var)(IntPtr)(&((*VIEWPointer).portal_z));
                    }
                }
                return portal_xyz_;
            }
        }

        /// <summary>
        /// pnormal_x - see Gamestudio manual
        /// </summary>
        private Var pnormal_x_ = null;
        public Var pnormal_x
        {
            get
            {
                unsafe
                {
                    if (pnormal_x_ == null)
                    {
                        pnormal_x_ = (Var)(IntPtr)(&((*VIEWPointer).pnormal_x));
                    }
                }
                return pnormal_x_;
            }
        }

        /// <summary>
        /// pnormal_y - see Gamestudio manual
        /// </summary>
        private Var pnormal_y_ = null;
        public Var pnormal_y
        {
            get
            {
                unsafe
                {
                    if (pnormal_y_ == null)
                    {
                        pnormal_y_ = (Var)(IntPtr)(&((*VIEWPointer).pnormal_y));
                    }
                }
                return pnormal_y_;
            }
        }

        /// <summary>
        /// pnormal_z - see Gamestudio manual
        /// </summary>
        private Var pnormal_z_ = null;
        public Var pnormal_z
        {
            get
            {
                unsafe
                {
                    if (pnormal_z_ == null)
                    {
                        pnormal_z_ = (Var)(IntPtr)(&((*VIEWPointer).pnormal_z));
                    }
                }
                return pnormal_z_;
            }
        }

        /// <summary>
        /// pnormal_xyz - not listed in the Gamestudio manual - allows to access pnormal_x, pnormal_y and pnormal_z as a Vector
        /// </summary>
        private Vector pnormal_xyz_ = null;
        public Vector pnormal_xyz
        {
            get
            {
                unsafe
                {
                    if (pnormal_xyz_ == null)
                    {
                        pnormal_xyz_ = new Vector((Var)0, (Var)0, (Var)0);
                        pnormal_xyz_.x = (Var)(IntPtr)(&((*VIEWPointer).pnormal_x));
                        pnormal_xyz_.y = (Var)(IntPtr)(&((*VIEWPointer).pnormal_y));
                        pnormal_xyz_.z = (Var)(IntPtr)(&((*VIEWPointer).pnormal_z));
                    }
                }
                return pnormal_xyz_;
            }
        }

        /// <summary>
        /// skill_x - see Gamestudio manual
        /// </summary>
        private Var skill_x_ = null;
        public Var skill_x
        {
            get
            {
                unsafe
                {
                    if (skill_x_ == null)
                    {
                        skill_x_ = (Var)(IntPtr)(&((*VIEWPointer).skill_x));
                    }
                }
                return skill_x_;
            }
        }

        /// <summary>
        /// skill_y - see Gamestudio manual
        /// </summary>
        private Var skill_y_ = null;
        public Var skill_y
        {
            get
            {
                unsafe
                {
                    if (skill_y_ == null)
                    {
                        skill_y_ = (Var)(IntPtr)(&((*VIEWPointer).skill_y));
                    }
                }
                return skill_y_;
            }
        }

        /// <summary>
        /// target1 - see Gamestudio manual
        /// </summary>
        public BMAP target1
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*VIEWPointer).target1;
                }
            }
            set
            {
                unsafe
                {
                    (*VIEWPointer).target1 = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// target2 - see Gamestudio manual
        /// </summary>
        public BMAP target2
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*VIEWPointer).target2;
                }
            }
            set
            {
                unsafe
                {
                    (*VIEWPointer).target2 = (NativeBMAP*)(IntPtr)value;
                }
            }
        }

        /// <summary>
        /// target3 - see Gamestudio manual
        /// </summary>
        public BMAP target3
        {
            get
            {
                unsafe
                {
                    return (BMAP)(IntPtr)(*VIEWPointer).target3;
                }
            }
            set
            {
                unsafe
                {
                    (*VIEWPointer).target3 = (NativeBMAP*)(IntPtr)value;
                }
            }
        }


        /* -------- VIEW -------- */
        /* VIEW to NativeVIEW* (address in void*) */
        public static explicit operator IntPtr(VIEW d)
        {
            unsafe
            {
                if (d == null)
                {
                    return IntPtr.Zero;
                }
                else
                {
                    return (IntPtr)(NativeVIEW*)d.VIEWPointer;
                }
            }
        }
        /* NativeVIEW* (address in void*) to VIEW */
        public static explicit operator VIEW(IntPtr d)
        {
            unsafe
            {
                if (d == IntPtr.Zero)
                {
                    return null;
                }
                return new VIEW(d);
            }
        }

    }
}