﻿/*
 * project : AcknexWrapper 1.0.2 for V7.82.3
 * created by : Matthias Auer
 * contact : mail@matthias-auer.net
 * homepage : http://acknexwrapper.matthias-auer.net/
 * date : 05.04.10
 */

using System;
using System.Runtime.InteropServices;
using AcknexWrapper.Native;

namespace AcknexWrapper
{
    /// <summary>
    /// standard Variable for A7
    /// </summary>
    public class Var
    {
        private NativeVar HiddenVar = new NativeVar();
        private IntPtr VarIntPtr = IntPtr.Zero;

        private int mValue
        {
            get
            {
                if (VarIntPtr != IntPtr.Zero)
                {
                    unsafe
                    {
                        HiddenVar = *((NativeVar*)VarIntPtr);
                    }
                }
                return HiddenVar.mValue;
            }
            set
            {
                HiddenVar.mValue = value;
                if (VarIntPtr != IntPtr.Zero)  //update structure in unmanaged memory if the var was already used as IntPtr
                {
                    Marshal.StructureToPtr(HiddenVar, VarIntPtr, false);
                }
            }
        }

        /// <summary>
        /// returns if this object uses a pointer - else it only stores a value
        /// </summary>
        public Boolean ISPointer()
        {
            if (VarIntPtr == IntPtr.Zero)
                return false;
            else
                return true;
        }

        /// <summary>
        /// set the value of the Var object (this method should not be used, as it uses another format - use explicit conversation instead)
        /// </summary>
        public void setRealInt(int mValue)
        {
            this.mValue = mValue;
        }

        /// <summary>
        /// set the pointer of the current object - use carefully
        /// </summary>
        public void setPointer(IntPtr VarPtr)
        {
            VarIntPtr = VarPtr;
        }

        /// <summary>
        /// set the pointer of the current object - use carefully
        /// </summary>
        public IntPtr getPointer()
        {
            return VarIntPtr;
        }

        /// <summary>
        /// set the value of the current object to the value of the object given as parameter (only value no pointer or object is copied)
        /// </summary>
        public void getValueFrom(Var tempVar)
        {
            mValue = tempVar.getRealInt();
        }

        /// <summary> ,
        /// get the value of the Var object (this method should not be used, as it uses another format - use explicit conversation instead)
        /// </summary>
        public int getRealInt()
        {
            return mValue;
        }

        /// <summary>
        /// creates a Var with the given Value
        /// </summary>
        public Var(float value)
        {
            mValue = (int)(value * (1 << 10));
        }

        /// <summary>
        /// creates a Var with the given Value
        /// </summary>
        public Var(int value)
        {
            mValue = value << 10;
        }

        /// <summary>
        /// standard constructor - a value isn't assigned to it yet
        /// </summary>
        public Var()
        {
        }

        /// <summary>
        /// Value of Var casted as Integer
        /// </summary>
        public int IntValue
        {
            get
            {
                return mValue >> 10;
            }
            set
            {
                mValue = value << 10;
            }
        }

        /// <summary>
        /// Value of Var casted as Float
        /// </summary>
        public float FloatValue
        {
            get
            {
                return ((float)mValue) / (1 << 10);
            }
            set
            {
                mValue = (int)(value * (1 << 10));
            }
        }

        public static explicit operator IntPtr(Var d)   //return a pointer to the structure stored in unmanaged memory
        {
            if (d.VarIntPtr == IntPtr.Zero)
            {
                d.VarIntPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(NativeVar)));
                Marshal.StructureToPtr(d.HiddenVar, d.VarIntPtr, false);
            }
            return d.VarIntPtr;
        }

        public static explicit operator Var(IntPtr d)
        {
            Var tempVar = new Var(0);
            tempVar.VarIntPtr = d;
            return tempVar;
        }

        /* ------------------- implicit ------------------- */

        /* -------- int -------- */
        /* Var to int */
        public static implicit operator int(Var d)
        {
            if (d == null)
            {
                return 0;
            }
            else
            {
                return d.mValue >> 10;
            }
        }
        /* int to Var */
        public static implicit operator Var(int d)
        {
            Var tempVar = new Var(0);
            tempVar.mValue = (d << 10);
            return tempVar;
        }

        /* -------- float -------- */
        /* Var to float */
        public static implicit operator float(Var d)
        {
            if (d == null)
            {
                return 0;
            }
            else
            {
                return ((float)d.mValue) / (1 << 10);
            }
        }
        /* float to Var */
        public static implicit operator Var(float d)
        {
            Var tempVar = new Var(0);
            tempVar.mValue = (int)(d * (1 << 10));
            return tempVar;
        }
    }
}