﻿using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using AcknexWrapper;
using System.Drawing.Design;
using System.Windows.Forms.Design;

namespace FXED.Classes
{
    // This class contains common functions used by various objects.

    public class Functions
    {
        /// <summary>
        /// Returns a new unique name for a specified component type.
        /// </summary>
        /// <param name="type">The component type.</param>
        /// <returns></returns>
        public static string GetNewName(string type)
        {
            switch (type)
            {
                case "Pass":
                    for (int i = 1; i < Classes.Variables.copy_check_count; i++)
                    {
                        if (Classes.Variables.my_effect.Passes.Find(delegate(Classes.Objects.Pass pass) { return pass.Name == type + i; }) == null)
                        {
                            return type + i;
                        }
                    }
                    break;

                case "Variable":
                    for (int i = 1; i < Classes.Variables.copy_check_count; i++)
                    {
                        if (Classes.Variables.my_effect.Variables.Find(delegate(Classes.Objects.ParentComponent var) { return var.Name == type + i; }) == null)
                        {
                            return type + i;
                        }
                    }
                    break;

                case "Array":
                    for (int i = 1; i < Classes.Variables.copy_check_count; i++)
                    {
                        if (Classes.Variables.my_effect.Arrays.Find(delegate(Classes.Objects.ParentComponent arr) { return arr.Name == type + i; }) == null)
                        {
                            return type + i;
                        }
                    }
                    break;

                case "Vector":
                    for (int i = 1; i < Classes.Variables.copy_check_count; i++)
                    {
                        if (Classes.Variables.my_effect.Vectors.Find(delegate(Classes.Objects.ParentComponent vec) { return vec.Name == type + i; }) == null)
                        {
                            return type + i;
                        }
                    }
                    break;

                case "Color":
                    for (int i = 1; i < Classes.Variables.copy_check_count; i++)
                    {
                        if (Classes.Variables.my_effect.Colors.Find(delegate(Classes.Objects.ParentComponent clr) { return clr.Name == type + i; }) == null)
                        {
                            return type + i;
                        }
                    }
                    break;

                case "Matrix":
                    for (int i = 1; i < Classes.Variables.copy_check_count; i++)
                    {
                        if (Classes.Variables.my_effect.Matrices.Find(delegate(Classes.Objects.ParentComponent mat) { return mat.Name == type + i; }) == null)
                        {
                            return type + i;
                        }
                    }
                    break;

                case "Sampler":
                    for (int i = 1; i < Classes.Variables.copy_check_count; i++)
                    {
                        if (Classes.Variables.my_effect.Samplers.Find(delegate(Classes.Objects.ParentComponent sam) { return sam.Name == type + i; }) == null)
                        {
                            return type + i;
                        }
                    }
                    break;

                case "Function":
                    for (int i = 1; i < Classes.Variables.copy_check_count; i++)
                    {
                        if (Classes.Variables.my_effect.Functions.Find(delegate(Classes.Objects.ParentComponent fun) { return fun.Name == type + i; }) == null)
                        {
                            return type + i;
                        }
                    }
                    break;

                case "Define":
                    for (int i = 1; i < Classes.Variables.copy_check_count; i++)
                    {
                        if (Classes.Variables.my_effect.Defines.Find(delegate(Classes.Objects.Define def) { return def.Name == type + i; }) == null)
                        {
                            return type + i;
                        }
                    }
                    break;

                case "Light":
                    for (int i = 1; i < Classes.Variables.copy_check_count; i++)
                    {
                        if (Classes.Variables.my_effect.Settings.Lights.Find(delegate(Classes.Objects.Light lit) { return lit.Name == type + i; }) == null)
                        {
                            return type + i;
                        }
                    }
                    break;

                case "Struct":
                    for (int i = 1; i < Classes.Variables.copy_check_count; i++)
                    {
                        if (Classes.Variables.my_effect.Structs.Find(delegate(Classes.Objects.Struct str) { return str.Name == type + i; }) == null)
                        {
                            return type + i;
                        }
                    }
                    break;
            }

            return null;
        }

        public static string GetShaderModel(string type, string model)
        {
            string key = (type == "Vertex") ? "vs" : "ps";

            switch (model)
            {
                case "1.0": return key + "_1_0";
                case "1.1": return key + "_1_1";
                case "1.3": return key + "_1_3";
                case "1.4": return key + "_1_4";
                case "2.0": return key + "_2_0";
                case "2.0a": return key + "_2_a";
                case "2.0b": return key + "_2_b";
                case "3.0": return key + "_3_0";
            }
            return string.Empty;
        }

        /// <summary>
        /// Checks if a name is in a suitable format.
        /// </summary>
        /// <param name="name">The name to check.</param>
        /// <returns></returns>
        public static bool CheckNameIsAppropriate(string name, string type)
        {
            // check name isn't blank or null
            if (name == null || name == string.Empty) { return false; }

            // check name doesn't start with a number
            if (char.IsNumber(name[0]))
            {
                MessageBox.Show("A "+type+" name can not begin with a number.\r\nPlease rename the current "+type+".", "Invalid "+type+" name", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return false;
            }

            // check name doesn't contain any spaces
            if (name.Contains(" "))
            {
                MessageBox.Show("A " + type + " name can not contain any special characters or spaces.\r\nPlease rename the current "+type+".", "Invalid "+type+" name", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return false;
            }

            // check name doesn't contain any special characters
            foreach (char c in name)
            {
                if (c == '_') { continue; }
                if (char.IsPunctuation(c) || char.IsSymbol(c))
                {
                    MessageBox.Show("A "+type+" name can not contain any special characters or spaces.\r\nPlease rename the current "+type+".", "Invalid "+type+" name", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    return false;
                }
            }

            // check name isn't a reserved word

            if (Classes.Variables.reserved_words.Contains(name))
            {
                MessageBox.Show("A "+type+" name can not be a reserved word.\r\nPlease rename the current "+type+".", "Invalid "+type+" name", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return false;
            }

            // check name isn't an intrinsic function

            if (Classes.Variables.intrinsic_functions.Contains(name))
            {
                MessageBox.Show("A "+type+" name can not be the same as a built-in function.\r\nPlease rename the current "+type+".", "Invalid "+type+" name", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return false;
            }

            // check name isn't a predefined "+type+"

            if (Classes.Variables.predefined_components.Contains(name))
            {
                MessageBox.Show("A "+type+" name can not be the same as a predefined component.\r\nPlease rename the current "+type+".", "Invalid "+type+" name", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return false;
            }


            return true;
        }

        /// <summary>
        /// Checks if there is an object with the same name.
        /// </summary>
        /// <param name="name">The name to check.</param>
        /// <returns></returns>
        public static bool CheckNameIsUnique(string name, string type)
        {
            if (!CheckNameIsAppropriate(name,type)) { return false; }

            bool exists = false;

            while (true)
            {
                // check passes

                if (Variables.my_effect.Passes.Find(delegate(Classes.Objects.Pass pass) { return pass.Name == name; }) != null)
                {
                    exists = true; break;
                }

                // check variables

                if (Variables.my_effect.Variables.Find(delegate(Classes.Objects.ParentComponent var) { return var.Name == name; }) != null)
                {
                    exists = true; break;
                }

                // check arrays

                if (Variables.my_effect.Arrays.Find(delegate(Classes.Objects.ParentComponent arr) { return arr.Name == name; }) != null)
                {
                    exists = true; break;
                }

                // check vectors

                if (Variables.my_effect.Vectors.Find(delegate(Classes.Objects.ParentComponent vec) { return vec.Name == name; }) != null)
                {
                    exists = true; break;
                }

                // check colors

                if (Variables.my_effect.Colors.Find(delegate(Classes.Objects.ParentComponent clr) { return clr.Name == name; }) != null)
                {
                    exists = true; break;
                }

                // check matrices

                if (Variables.my_effect.Matrices.Find(delegate(Classes.Objects.ParentComponent mat) { return mat.Name == name; }) != null)
                {
                    exists = true; break;
                }

                // check textures

                if (Variables.my_effect.Textures.Find(delegate(Classes.Objects.ParentComponent tex) { return tex.Name == name; }) != null)
                {
                    exists = true; break;
                }

                // check samplers

                if (Variables.my_effect.Samplers.Find(delegate(Classes.Objects.ParentComponent sam) { return sam.Name == name; }) != null)
                {
                    exists = true; break;
                }

                // check functions

                if (Variables.my_effect.Functions.Find(delegate(Classes.Objects.ParentComponent fun) { return fun.Name == name; }) != null)
                {
                    exists = true; break;
                }

                if (Variables.my_effect.Structs.Find(delegate(Classes.Objects.Struct str) { return str.Name == name; }) != null)
                {
                    exists = true; break;
                }
                
                break;
            }

            if (exists)
            {
                MessageBox.Show("An object with this name already exists.\r\nPlease rename the current " + type + ".", "Duplicate " + type + " name", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return false;
            }
            return true;
        }

        /// <summary>
        /// Checks if the specified cast is valid
        /// </summary>
        /// <param name="type">The type trying to cast to.</param>
        /// <param name="value">The value to cast.</param>
        /// <returns></returns>
        public static bool CheckCastIsValid(string type, string value)
        {
            bool error = false;
            int int_i;
            float float_i;
            double double_i;
            string bool_i = value;

            switch (type)
            {
                case "int": if (!int.TryParse(value, out int_i)) { error = true; } break;
                case "half": if (!float.TryParse(value, out float_i) || (Math.Abs(Convert.ToSingle(value)) > 610000)) { error = true; } break;
                case "float": if (!float.TryParse(value, out float_i)) { error = true; } break;
                case "double": if (!double.TryParse(value, out double_i)) { error = true; } break;
                case "bool": if (bool_i == "true" || bool_i == "false") { error = false; } else { error = true; } break;
            }

            if (error) {return false; }
            return true;
        }

        /// <summary>
        /// Retrieves the default value of a data type
        /// </summary>
        /// <param name="type">The selected type.</param>
        /// <returns></returns>
        public static string GetDefaultValue(string type)
        {
            switch (type)
            {
                case "int": return "0";
                case "int2": return "int2(0,0)";
                case "int3": return "int3(0,0,0)";
                case "int4": return "int4(0,0,0,0)";

                case "bool": return "false";
                case "bool2": return "bool2(false,false)";
                case "bool3": return "bool3(false,false,false)";
                case "bool4": return "bool4(false,false,false,false)";

                case "half": return "0.0";
                case "half2": return "half2(0.0f,0.0f)";
                case "half3": return "half3(0.0f,0.0f,0.0f)";
                case "half4": return "half4(0.0f,0.0f,0.0f,0.0f)";

                case "float": return "0.0";
                case "float2": return "float2(0.0f,0.0f)";
                case "float3": return "float3(0.0f,0.0f,0.0f)";
                case "float4": return "float4(0.0f,0.0f,0.0f,0.0f)";

                case "double": return "0.0";
                case "double2": return "double2(0.0,0.0)";
                case "double3": return "double3(0.0,0.0,0.0)";
                case "double4": return "double4(0.0,0.0,0.0,0.0)";
            }

            return string.Empty;
        }

        /// <summary>
        /// Sets the application in a state to be saved.
        /// </summary>
        public static void ProjectChanged()
        {
            Classes.Variables.project_changed = true;
            Classes.Variables.main_form.Text = "Effect Editor - " + Classes.Variables.current_name+"*";
        }

        /// <summary>
        /// Recompiles the current effect.
        /// </summary>
        public static void Recompile()
        {
            if (Properties.Settings.Default.AutoRecompile)
            {
                if (Classes.Variables.preview != null)
                {
                    Scheduler.AddEventVoidThreadSafe(Classes.Variables.preview.ApplyShader);
                }
            }
        }

        /// <summary>
        /// Updates the syntax highlighting in vertex/pixel shaders / functions.
        /// </summary>
        public static void UpdateHighlighting()
        {
            foreach (Form frm in Classes.Variables.main_form.MdiChildren)
            {
                if (frm is Forms.frm_shader)
                {
                    ((Forms.frm_shader)frm).UpdateObjects();
                    continue;
                }

                if (frm is Forms.frm_function)
                {
                    ((Forms.frm_function)frm).UpdateObjects();
                }
            }
        }

        /// <summary>
        /// Performs a 'deep-clone' of the selected instance.
        /// </summary>
        public static object Clone(object target)
        {
            try
            {
                BinaryFormatter bf = new BinaryFormatter();     //helper to serialize
                MemoryStream memStream = new MemoryStream();
                bf.Serialize(memStream, target);
                memStream.Flush();
                memStream.Position = 0;
                return bf.Deserialize(memStream);  //this returns the copy of the instance.
            }
            catch (Exception)
            {

            }
            return null;
        }

        /// <summary>
        /// Shows a dialog to allow the user to save changes to the current shader.
        /// </summary>
        /// <returns></returns>
        public static bool save_return = false;
        public static bool IsProjectSaved()
        {
            if (!Classes.Variables.project_changed) { return true; }
          
            else
            {
                switch (MessageBox.Show("Save changes to the current effect?", "Confirm effect save", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question))
                {
                    case DialogResult.Yes: Classes.Variables.main_form.SaveProject(null,EventArgs.Empty); if (save_return) { return true; } else { return false; }
                    case DialogResult.No: return true;
                    case DialogResult.Cancel: return false;
                }
            }

            return false;
        }  

        /// <summary>
        /// Updates the form title for the current project name.
        /// </summary>
        public static void UpdateCurrentName()
        {
            Classes.Variables.main_form.Text =  "Effect Editor - "+Classes.Variables.current_name;
        }

        /// <summary>
        /// A generic clamp function.
        /// </summary>
        public static T Clamp<T>(T value, T min, T max) where T : System.IComparable<T>        
        {
            if (value.CompareTo(min) < 0) { return min; } 
            if (value.CompareTo(max) > 0) { return max; }        
            return value;
        }

        /// <summary>
        /// Updates other components that are dependent upon deleted/renamed components.
        /// </summary>
        public static void UpdateReferences(string type, string old_value, string new_value)
        {
            switch (type)
            {
                case "Texture":
                    foreach (Classes.Objects.Sampler sampler in Classes.Variables.my_effect.Samplers)
                    {
                        if (sampler.Texture == old_value) { sampler.Texture = new_value; }
                    }
                    break;

                case "Pass":
                    foreach (Classes.Objects.Pass pass in Classes.Variables.my_effect.Passes)
                    {
                        if (pass.VertexLink == old_value) { pass.VertexLink = new_value; }
                        if (pass.PixelLink == old_value) { pass.PixelLink = new_value; }
                    }
                    break;

                case "Struct":

                    // custom functions

                    foreach (Classes.Objects.Function func in Classes.Variables.my_effect.Functions)
                    {
                        for (int i = 0; i < func.ReturnType.Count; i++)
                        {
                            if (func.ReturnType[i] == old_value) { func.ReturnType[i] = new_value; }
                        }
                    }

                    foreach (Form frm in Classes.Variables.main_form.MdiChildren)
                    {
                        if (frm is Forms.frm_function)
                        {
                            ((Forms.frm_function)frm).ReloadReturnTypes();
                        }
                    }

                    // shader functions

                    foreach (Classes.Objects.Pass pass in Classes.Variables.my_effect.Passes)
                    {
                        if (pass.VertexShader != null)
                        {
                            for (int i = 0; i < pass.VertexShader.Inputs.Count; i++)
                            {
                                if (pass.VertexShader.Inputs[i].DataType == old_value) { pass.VertexShader.Inputs[i].DataType = new_value; }
                            }

                            for (int i = 0; i < pass.VertexShader.Outputs.Count; i++)
                            {
                                if (pass.VertexShader.Outputs[i].DataType == old_value) { pass.VertexShader.Outputs[i].DataType = new_value; }
                            }
                        }

                        if (pass.PixelShader != null)
                        {
                            for (int i = 0; i < pass.PixelShader.Inputs.Count; i++)
                            {
                                if (pass.PixelShader.Inputs[i].DataType == old_value) { pass.PixelShader.Inputs[i].DataType = new_value; }
                            }

                            for (int i = 0; i < pass.PixelShader.Outputs.Count; i++)
                            {
                                if (pass.PixelShader.Outputs[i].DataType == old_value) { pass.PixelShader.Outputs[i].DataType = new_value; }
                            }
                        }
                    }
                    break;
            }
        }

        /// <summary>
        /// Sets the 'Browsable' attribute of a property, at runtime.
        /// </summary>
        /// <param name="instance">the instance containing the property to change.</param>
        /// <param name="property">the name of the property to change.</param>
        /// <param name="value">the new value to set.</param>
        public static void SetBrowsable(object instance, string property, bool value)
        {
            PropertyDescriptor descriptor = TypeDescriptor.GetProperties(instance.GetType())[property];

            BrowsableAttribute attribute = (BrowsableAttribute)descriptor.Attributes[typeof(BrowsableAttribute)];

            FieldInfo field_info = attribute.GetType().GetField("browsable", BindingFlags.NonPublic | BindingFlags.Instance);

            field_info.SetValue(attribute, value);
        }

        /// <summary>
        /// Sets the 'DisplayName' attribute of a property, at runtime.
        /// </summary>
        /// <param name="instance">the instance containing the property to change.</param>
        /// <param name="property">the name of the property to change.</param>
        /// <param name="value">the new value to set.</param>
        public static void SetDisplayName(object instance, string property, string value)
        {
            PropertyDescriptor descriptor = TypeDescriptor.GetProperties(instance.GetType())[property];

            DisplayNameAttribute attribute = (DisplayNameAttribute)descriptor.Attributes[typeof(DisplayNameAttribute)];

            FieldInfo field_info = attribute.GetType().GetField("_displayName", BindingFlags.NonPublic | BindingFlags.Instance);

            field_info.SetValue(attribute, value);
        }

        /// <summary>
        /// Sets the 'Description' attribute of a property, at runtime.
        /// </summary>
        /// <param name="instance">the instance containing the property to change.</param>
        /// <param name="property">the name of the property to change.</param>
        /// <param name="value">the new value to set.</param>
        public static void SetDescription(object instance, string property, string value)
        {
            PropertyDescriptor descriptor = TypeDescriptor.GetProperties(instance.GetType())[property];

            DescriptionAttribute attribute = (DescriptionAttribute)descriptor.Attributes[typeof(DescriptionAttribute)];

            FieldInfo field_info = attribute.GetType().GetField("description", BindingFlags.NonPublic | BindingFlags.Instance);

            field_info.SetValue(attribute, value);
        }

    }
}
