﻿using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Drawing;
using System.Drawing.Design;
using System.Reflection;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.Design;

namespace FXED.Classes.Helper
{
    public class ListConverter : StringConverter
    {
        public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
        {
            return true;
        }

        public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
        {
            //true = limited list, false = free-entry
            return true;
        }
    }

    public class ColorModes : ListConverter
    {
        public override System.ComponentModel.TypeConverter.StandardValuesCollection
            GetStandardValues(ITypeDescriptorContext context)
        {
            return new StandardValuesCollection(new string[] { "ARGB", "RGBA", "AYUV", "RGB", "YUV" });
        }
    }

    public class VertexModels : ListConverter
    {
        public override System.ComponentModel.TypeConverter.StandardValuesCollection
            GetStandardValues(ITypeDescriptorContext context)
        {
            return new StandardValuesCollection(new string[] { "1.0","1.1","2.0a","3.0" });
        }
    }

    public class PixelModels : ListConverter
    {
        public override System.ComponentModel.TypeConverter.StandardValuesCollection
            GetStandardValues(ITypeDescriptorContext context)
        {
            return new StandardValuesCollection(new string[] { "1.0", "1.1", "1.3", "1.4", "2.0", "2.0a", "2.0b", "3.0" });
        }
    }

    public class ParameterIndices : ListConverter
    {
        public override System.ComponentModel.TypeConverter.StandardValuesCollection
            GetStandardValues(ITypeDescriptorContext context)
        {
            return new StandardValuesCollection(new string[]{ "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15" });
        }
    }

    public class StructSemantics : ListConverter
    {
        public override System.ComponentModel.TypeConverter.StandardValuesCollection
            GetStandardValues(ITypeDescriptorContext context)
        {
            return new StandardValuesCollection(new string[]
            { 
                "BINORMAL","BLENDINDICES","BLENDWEIGHT","COLOR","DEPTH","FOG","NORMAL","POSITION","POSITIONT","PSIZE",
                "TANGENT","TESSFACTOR","TEXCOORD","VFACE","VPOS"
            }
            );
        }
    }

    public class ParameterSemantics : ListConverter
    {
        private string[] GetValues(Objects.Parameter param)
        {
            switch (Classes.Variables.current_shader.Type)
            {
                case "Vertex":
                    if (param.Mode == "in")
                    {
                        return new string[]
                        {
                          "BINORMAL","BLENDINDICES","BLENDWEIGHT","COLOR","NORMAL","POSITION","POSITIONT","PSIZE",
                          "TANGENT","TESSFACTOR","TEXCOORD"
                        };
                    }
                    else
                    {
                        return new string[]
                        {
                          "COLOR","FOG","POSITION","PSIZE","TESSFACTOR","TEXCOORD"
                        };
                    }

                case "Pixel":
                    if (param.Mode == "in")
                    {
                        return new string[]
                        {
                          "COLOR","TEXCOORD","VFACE","VPOS"
                        };
                    }
                    else
                    {
                        return new string[]
                        {
                          "COLOR","DEPTH"
                        };
                    }
            }
            return null;
        }

        public override System.ComponentModel.TypeConverter.StandardValuesCollection
            GetStandardValues(ITypeDescriptorContext context)
        {
            return new StandardValuesCollection(GetValues((Objects.Parameter)context.Instance));
        }
    }

    public class VectorElements : ListConverter
    {
        public override System.ComponentModel.TypeConverter.StandardValuesCollection
            GetStandardValues(ITypeDescriptorContext context)
        {
            return new StandardValuesCollection(new string[] { "2","3","4" });
        }
    }

    public class MatrixModifiers : ListConverter
    {
        public override System.ComponentModel.TypeConverter.StandardValuesCollection
            GetStandardValues(ITypeDescriptorContext context)
        {
            return new StandardValuesCollection(new string[] { "None","row_major","column_major" });
        }
    }

    public class CurrentTextures : ListConverter
    {
        public override System.ComponentModel.TypeConverter.StandardValuesCollection
            GetStandardValues(ITypeDescriptorContext context)
        {
            List<string> return_list = new List<string> { "(none)" };

            foreach (Classes.Objects.Texture tex in Classes.Variables.my_effect.Textures)
            {
                return_list.Add(tex.Name);
            }

            return new StandardValuesCollection(return_list);
        }
    }

    public class CurrentPasses : ListConverter
    {
        public override System.ComponentModel.TypeConverter.StandardValuesCollection
            GetStandardValues(ITypeDescriptorContext context)
        {
            List<string> return_list = new List<string> { "(none)" };

            foreach (Classes.Objects.Pass pass in Classes.Variables.my_effect.Passes)
            {
                if (context.PropertyDescriptor.Name == "VertexLink" && pass.VertexLink != "(none)") { continue; }
                if (context.PropertyDescriptor.Name == "PixelLink" && pass.PixelLink != "(none)") { continue; }

                if (((Classes.Objects.Pass)context.Instance).Name != pass.Name)
                {
                    return_list.Add(pass.Name);
                }
            }

            return new StandardValuesCollection(return_list);
        }
    }

    public class RenderStates : ListConverter
    {
        public static List<string> true_false_values = new List<string> { "False", "True" };
        public static List<string> zenable_values = new List<string> { "False", "True", "UseW" };
        public static List<string> fillmode_values = new List<string> { "Point", "WireFrame", "Solid" };
        public static List<string> shademode_values = new List<string> { "Flat", "Gouraud", "Phong" };
        public static List<string> blend_values = new List<string> 
        { 
            "Zero", "One", "SrcColor", "InvSrcColor", "SrcAlpha", "InvSrcAlpha", "DestColor", "InvDestColor",
            "DestAlpha","InvDestAlpha", "SrcAlphaSat","BothSrcAlpha","BothInvSrcAlpha","BlendFactor","InvBlendFactor","SrcColor2",
            "InvSrcColor2"
        };
        public static List<string> cullmode_values = new List<string> { "None", "CW", "CCW" };
        public static List<string> func_values = new List<string> { "Never", "Less", "Equal", "LessEqual", "Greater", "NotEqual", "GreaterEqual", "Always" };
        public static List<string> fogmode_values = new List<string> { "None", "Exp", "Exp2", "Linear" };
        public static List<string> stencil_operation_values = new List<string> { "Keep", "Zero", "Replace", "IncrSat", "DecrSat", "Invert", "Incr", "Decr" };
        public static List<string> material_source_values = new List<string> { "Material","Color1","Color2" };
        public static List<string> vertex_blend_values = new List<string> { "Disable", "1Weights", "2Weights", "3Weights", "Tweening", "0Weights" };
        public static List<string> patch_edge_values = new List<string> { "Discrete","Continuous" };
        public static List<string> blendop_values = new List<string> { "Add", "Subtract","RevSubtract","Min","Max" };
        public static List<string> degree_values = new List<string> { "Linear","Quadratic","Cubic","Quintic" };

        public override System.ComponentModel.TypeConverter.StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
        {
            switch (context.PropertyDescriptor.Name)
            {
                case "ZEnable": return new StandardValuesCollection(zenable_values);
                case "FillMode": return new StandardValuesCollection(fillmode_values);
                case "ShadeMode": return new StandardValuesCollection(shademode_values);
                case "ZWriteEnable": return new StandardValuesCollection(true_false_values);
                case "AlphaTestEnable": return new StandardValuesCollection(true_false_values);
                case "LastPixel": return new StandardValuesCollection(true_false_values);
                case "SrcBlend": return new StandardValuesCollection(blend_values);
                case "DestBlend": return new StandardValuesCollection(blend_values);
                case "CullMode": return new StandardValuesCollection(cullmode_values);
                case "ZFunc": return new StandardValuesCollection(func_values);
                case "AlphaFunc": return new StandardValuesCollection(func_values);
                case "DitherEnable": return new StandardValuesCollection(true_false_values);
                case "AlphaBlendEnable": return new StandardValuesCollection(true_false_values);
                case "FogEnable": return new StandardValuesCollection(true_false_values);
                case "SpecularEnable": return new StandardValuesCollection(true_false_values);
                case "FogTableMode": return new StandardValuesCollection(fogmode_values);
                case "RangeFogEnable": return new StandardValuesCollection(true_false_values);
                case "StencilEnable": return new StandardValuesCollection(true_false_values);
                case "StencilFail": return new StandardValuesCollection(stencil_operation_values);
                case "StencilZFail": return new StandardValuesCollection(stencil_operation_values);
                case "StencilPass": return new StandardValuesCollection(stencil_operation_values);
                case "StencilFunc": return new StandardValuesCollection(func_values);
                case "Clipping": return new StandardValuesCollection(true_false_values);
                case "Lighting": return new StandardValuesCollection(true_false_values);
                case "FogVertexMode": return new StandardValuesCollection(fogmode_values);
                case "ColorVertex": return new StandardValuesCollection(true_false_values);
                case "LocalViewer": return new StandardValuesCollection(true_false_values);
                case "NormalizeNormals": return new StandardValuesCollection(true_false_values);
                case "DiffuseMaterialSource": return new StandardValuesCollection(material_source_values);
                case "SpecularMaterialSource": return new StandardValuesCollection(material_source_values);
                case "AmbientMaterialSource": return new StandardValuesCollection(material_source_values);
                case "EmissiveMaterialSource": return new StandardValuesCollection(material_source_values);
                case "VertexBlend": return new StandardValuesCollection(vertex_blend_values);
                case "PointSpriteEnable": return new StandardValuesCollection(true_false_values);
                case "PointScaleEnable": return new StandardValuesCollection(true_false_values);
                case "MultiSampleAntiAlias": return new StandardValuesCollection(true_false_values);
                case "PatchEdgeStyle": return new StandardValuesCollection(patch_edge_values);
                case "BlendOP": return new StandardValuesCollection(blendop_values);
                case "PositionDegree": return new StandardValuesCollection(degree_values);
                case "NormalDegree": return new StandardValuesCollection(degree_values);
                case "ScissorTestEnable": return new StandardValuesCollection(true_false_values);
                case "AntiAliasedLineEnable": return new StandardValuesCollection(true_false_values);
                case "EnableAdaptiveTessellation": return new StandardValuesCollection(true_false_values);
                case "TwoSidedStencilMode": return new StandardValuesCollection(true_false_values);
                case "CCW_StencilFail": return new StandardValuesCollection(stencil_operation_values);
                case "CCW_StencilZFail": return new StandardValuesCollection(stencil_operation_values);
                case "CCW_StencilPass": return new StandardValuesCollection(stencil_operation_values);
                case "CCW_StencilFunc": return new StandardValuesCollection(func_values);
                case "SeperateAlphaBlendEnable": return new StandardValuesCollection(true_false_values);
                case "SrcBlendAlpha": return new StandardValuesCollection(blend_values);
                case "DestBlendAlpha": return new StandardValuesCollection(blend_values);
                case "BlendOPAlpha": return new StandardValuesCollection(blendop_values);
            }
            return null;
        }
    }

    public class SamplerStates : ListConverter
    {
        public static List<string> address_values = new List<string> { "Wrap", "Mirror", "Clamp", "Border", "MirrorOnce" };
        public static List<string> filter_values = new List<string> { "None", "Point", "Linear", "Anisotropic", "PyramidalQuad", "GaussianQuad", "ConvolutionMono" };

        public override System.ComponentModel.TypeConverter.StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
        {
            switch (context.PropertyDescriptor.Name)
            {
                case "AddressU": return new StandardValuesCollection(address_values);
                case "AddressV": return new StandardValuesCollection(address_values);
                case "AddressW": return new StandardValuesCollection(address_values);
                case "MagFilter": return new StandardValuesCollection(filter_values);
                case "MinFilter": return new StandardValuesCollection(filter_values);
                case "MipFilter": return new StandardValuesCollection(filter_values);
            }
            return null;
        }
    }

    public class DataTypes : ListConverter
    {
        public static List<string> GetValues()
        {
            List<string> values = new List<string>
            {
                "bool", "bool2", "bool3", "bool4", "int", "int2", "int3", "int4", "half", "half2", "half3", "half4", "float", "float2", "float3", "float4", "double", "double2", "double3", "double4"
            };

            foreach (Classes.Objects.Struct str in Classes.Variables.my_effect.Structs)
            {
                values.Add(str.Name);
            }
            return values;
        }

        public override System.ComponentModel.TypeConverter.StandardValuesCollection
            GetStandardValues(ITypeDescriptorContext context)
        {
            List<string> values = GetValues();
            if (context.Instance is Classes.Objects.StructMember)
            {
                values.Remove(Classes.Variables.current_struct.Name);
            }
            values.Sort();
            return new StandardValuesCollection(values);
        }
    }

    public class CurrentTechniqueNames : ListConverter
    {
        public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
        {
            return false;
        }

        public override System.ComponentModel.TypeConverter.StandardValuesCollection
            GetStandardValues(ITypeDescriptorContext context)
        {
            List<string> return_list = new List<string> { "(default)" };

            foreach (Classes.Objects.Pass pass in Classes.Variables.my_effect.Passes)
            {
                if (!return_list.Contains(pass.TechniqueName)) { return_list.Add(pass.TechniqueName); }
            }

            return new StandardValuesCollection(return_list);
        }
    }

    [Serializable]
    public class MatrixEditor : UITypeEditor
    {
        public override System.Drawing.Design.UITypeEditorEditStyle GetEditStyle(System.ComponentModel.ITypeDescriptorContext context)
        {
            return UITypeEditorEditStyle.DropDown;
        }

        public override object EditValue(System.ComponentModel.ITypeDescriptorContext context, System.IServiceProvider provider, object value)
        {
            IWindowsFormsEditorService edSvc = (IWindowsFormsEditorService)provider.GetService(typeof(IWindowsFormsEditorService));
            if (edSvc != null)
            {
                Classes.Objects.Matrix matrix = (Classes.Objects.Matrix)context.Instance;

                CustomControls.MatrixEditor editor = new CustomControls.MatrixEditor(matrix);

                editor.Width = 277;
                editor.Height = 120;

                edSvc.DropDownControl(editor);

                matrix.GridValues[0] = editor.txt_11.Text; matrix.GridValues[1] = editor.txt_12.Text; matrix.GridValues[2] = editor.txt_13.Text; matrix.GridValues[3] = editor.txt_14.Text;
                matrix.GridValues[4] = editor.txt_21.Text; matrix.GridValues[5] = editor.txt_22.Text; matrix.GridValues[6] = editor.txt_23.Text; matrix.GridValues[7] = editor.txt_24.Text;
                matrix.GridValues[8] = editor.txt_31.Text; matrix.GridValues[9] = editor.txt_32.Text; matrix.GridValues[10] = editor.txt_33.Text; matrix.GridValues[11] = editor.txt_34.Text;
                matrix.GridValues[12] = editor.txt_41.Text; matrix.GridValues[13] = editor.txt_42.Text; matrix.GridValues[14] = editor.txt_43.Text; matrix.GridValues[15] = editor.txt_44.Text;

                Classes.Functions.ProjectChanged();
                Classes.Functions.Recompile();

                return string.Empty;
            }
            return value;
        }
    }

    [Serializable]
    public class SkillValueEditor : UITypeEditor
    {
        public override System.Drawing.Design.UITypeEditorEditStyle GetEditStyle(System.ComponentModel.ITypeDescriptorContext context)
        {
            return UITypeEditorEditStyle.DropDown;
        }

        public override object EditValue(System.ComponentModel.ITypeDescriptorContext context, System.IServiceProvider provider, object value)
        {
            IWindowsFormsEditorService edSvc = (IWindowsFormsEditorService)provider.GetService(typeof(IWindowsFormsEditorService));
            if (edSvc != null)
            {
                Classes.Objects.Skill skill = (Classes.Objects.Skill)context.Instance;

                TrackBar trk_skills = new TrackBar();
                trk_skills.Tag = skill;
                trk_skills.TickStyle = TickStyle.None;
                trk_skills.Minimum = (int)(skill.Min * 100);
                trk_skills.Maximum = (int)(skill.Max * 100);
                trk_skills.Value = (int)(skill.Value * 100);
                trk_skills.ValueChanged += new EventHandler(trk_skills_ValueChanged);
                trk_skills.Width = 200;
                trk_skills.Height = 120;

                edSvc.DropDownControl(trk_skills);

                Classes.Functions.ProjectChanged();

                return ((float)trk_skills.Value) / 100;
            }
            return value;
        }

        void trk_skills_ValueChanged(object sender, EventArgs e)
        {
            TrackBar trk = (TrackBar)sender;

            Classes.Variables.preview.SetSkill(((Classes.Objects.Skill)trk.Tag).SkillNumber, ((float)trk.Value) / 100);
        }
    }

    [Serializable]
    public class ParentCollectionEditor : CollectionEditor
    {
        private PropertyGrid propertyGrid = null;
        private string text = "Value Editor";

        public ParentCollectionEditor(Type type) : base(type) { }
        public ParentCollectionEditor(Type type, string text) : base(type) { this.text = text; }

        protected override CollectionForm CreateCollectionForm()
        {
            //Get a reference top new collection form
            CollectionForm form = base.CreateCollectionForm();
            form.Shown += new EventHandler(FormShown);
            form.Closing += new CancelEventHandler(FormClosing);

            //Center the form 
            form.StartPosition = FormStartPosition.CenterParent;
            form.Text = text;
            form.Size = new Size(600, 450);

            TableLayoutPanel tlpLayout = form.Controls[0] as TableLayoutPanel;

            Button btnOk = (Button)tlpLayout.Controls[6].Controls[0]; //OK button
            btnOk.Click += new EventHandler(OKClicked);

            //Get the forms type
            Type formType = form.GetType();

            //Get a reference to the private fieldtype propertyBrowser
            //This is the propertgrid inside the collectionform
            FieldInfo fieldInfo = formType.GetField("propertyBrowser", BindingFlags.NonPublic | BindingFlags.Instance);

            if (fieldInfo != null)
            {
                //get a reference to the propertygrid instance located on the form
                propertyGrid = (PropertyGrid)fieldInfo.GetValue(form);

                if (propertyGrid != null)
                {
                    propertyGrid.SelectedObjectsChanged += new EventHandler(ItemChanged);

                    //Make the tool bar visible
                    propertyGrid.ToolbarVisible = true;

                    //Make the help/description visible
                    propertyGrid.HelpVisible = true;

                    //Get the property grid's type.
                    //Note that this is a vsPropertyGrid located in System.Windows.Forms.Design
                    Type propertyGridType = propertyGrid.GetType();

                    //Get a reference to the non-public property ToolStripRenderer
                    PropertyInfo propertyInfo = propertyGridType.GetProperty("ToolStripRenderer", BindingFlags.NonPublic | BindingFlags.Instance);

                    if (propertyInfo != null)
                    {
                        //Assign a ToolStripProfessionalRenderer with our custom color table
                        propertyInfo.SetValue(propertyGrid, new ToolStripProfessionalRenderer(new ProfessionalColorTable()), null);
                    }
                }
            }

            //Return the form
            return form;
        }

        public virtual void FormShown(object sender, EventArgs e)
        {

        }

        public virtual void FormClosing(object sender, CancelEventArgs e)
        {

        }

        public virtual void OKClicked(object sender, EventArgs e)
        {

        }

        public virtual void ItemChanged(object sender, EventArgs e)
        {

        }

        public PropertyGrid PropertyGrid
        {
            get { return propertyGrid; }
            set { propertyGrid = value; }
        }

        public string Text
        {
            get { return text; }
            set { text = value; }
        }
    }

    [Serializable]
    public class StructMemberCollectionEditor : ParentCollectionEditor
    {
        public StructMemberCollectionEditor(Type type): base(type)
        {
            base.Text = "Struct Member Editor";
        }

        protected override CollectionEditor.CollectionForm CreateCollectionForm()
        {
            Classes.Variables.current_struct = (Classes.Objects.Struct)base.Context.Instance;
            return base.CreateCollectionForm();
        }

        public override void ItemChanged(object sender, EventArgs e)
        {
            try
            {
                Classes.Objects.StructMember selected = (Classes.Objects.StructMember)((PropertyGrid)sender).SelectedObject;

                if (Classes.Variables.my_effect.Structs.Find(delegate(Classes.Objects.Struct str) { return str.Name == selected.DataType; }) != null)
                {
                    Classes.Functions.SetBrowsable(selected, "Semantic", false);
                    Classes.Functions.SetBrowsable(selected, "Index", false);
                }
                else
                {
                    Classes.Functions.SetBrowsable(selected, "Semantic", true);
                    Classes.Functions.SetBrowsable(selected, "Index", true);
                }
            }
            catch (Exception) { return; }
        }
    }

    [Serializable]
    public class ArrayValueCollectionEditor : ParentCollectionEditor
    {
        public ArrayValueCollectionEditor(Type type) : base(type) {}

        public override void FormShown(object sender, EventArgs e)
        {
            Classes.Objects.Array array = (Classes.Objects.Array)base.Context.Instance;

            if (array.Values.Count != 0)
            {
                if (array.Mode == "Variable")
                {
                    Functions.SetBrowsable(array.Values[0], "Value", true);
                    Functions.SetBrowsable(array.Values[0], "X", false);
                    Functions.SetBrowsable(array.Values[0], "Y", false);
                    Functions.SetBrowsable(array.Values[0], "Z", false);
                    Functions.SetBrowsable(array.Values[0], "W", false);
                }
                else
                {
                    Functions.SetBrowsable(array.Values[0], "Value", false);
                    switch (array.Elements)
                    {
                        case "2":
                            Functions.SetBrowsable(array.Values[0], "X", true);
                            Functions.SetBrowsable(array.Values[0], "Y", true);
                            Functions.SetBrowsable(array.Values[0], "Z", false);
                            Functions.SetBrowsable(array.Values[0], "W", false);
                            break;
                        case "3":
                            Functions.SetBrowsable(array.Values[0], "X", true);
                            Functions.SetBrowsable(array.Values[0], "Y", true);
                            Functions.SetBrowsable(array.Values[0], "Z", true);
                            Functions.SetBrowsable(array.Values[0], "W", false);
                            break;
                        case "4":
                            Functions.SetBrowsable(array.Values[0], "X", true);
                            Functions.SetBrowsable(array.Values[0], "Y", true);
                            Functions.SetBrowsable(array.Values[0], "Z", true);
                            Functions.SetBrowsable(array.Values[0], "W", true);
                            break;
                    }
                }
            }
            base.PropertyGrid.Refresh();
        }

        protected override object CreateInstance(Type itemType)
        {
            Classes.Objects.Array array = (Classes.Objects.Array)Classes.Variables.main_form.pgr_main.SelectedObject;
            return new Classes.Objects.ArrayItem(array.DataType.ToString(), array.Mode, array.Elements);        
        }
    }

    [Serializable]
    public class FunctionParameterCollection : ParentCollectionEditor
    {
        IList my_list;
        private CollectionEditor.CollectionForm form = null;
        private bool can_close = true;

        public FunctionParameterCollection(IList list): base(list.GetType())
        {
            my_list = list;
            base.Text = "Function Parameter Editor";
        }

        protected override CollectionEditor.CollectionForm CreateCollectionForm()
        {
            form = base.CreateCollectionForm();
            return form;
        }

        protected override void CancelChanges()
        {
            Variables.edit_ok = false;
            can_close = true;
            form.Close();
        }

        protected override object CreateInstance(Type itemType)
        {
            Classes.Objects.FunctionParameter new_parameter = new FXED.Classes.Objects.FunctionParameter();

            for (int i = 1; i < Classes.Variables.copy_check_count; i++)
            {
                if (((List<Classes.Objects.FunctionParameter>)my_list).Find(delegate(Classes.Objects.FunctionParameter param) { return param.Name == "Parameter" + i; }) == null)
                {
                    new_parameter.Name = "Parameter" + i; break;
                }
            }
            return new_parameter;
        }

        public override void OKClicked(object sender, EventArgs e)
        {
            // check no duplicate parameter signatures

            string param_list = Variables.current_function.GenerateParameterSignature((List<Classes.Objects.FunctionParameter>)my_list);
            can_close = true;
            Variables.edit_ok = true;

            if (Variables.current_function.Signatures.Contains(param_list))
            {
                if (Variables.current_function.CurrentOverload != Variables.current_function.Signatures.IndexOf(param_list))
                {
                    MessageBox.Show("An overload with this parameter set already exists.\r\nPlease change the current parameter set.", "Duplicate overload", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    can_close = false;

                    MethodInfo methodInfo = form.GetType().GetMethod("AddItems", BindingFlags.NonPublic | BindingFlags.Instance);
                    methodInfo.Invoke(form, new object[] { my_list });
                }
            }
        }

        public override void FormClosing(object sender, CancelEventArgs e)
        {
            e.Cancel = !can_close;
        }
    }

    [Serializable]
    public class ShaderParameterCollection: ParentCollectionEditor
    {
        private string mode = string.Empty;
        private string pass_name = string.Empty;

        public ShaderParameterCollection(Type type, string mode, string pass) : base(type) 
        {
            if (mode == "in")
            {
                base.Text = "Input(s) Editor";
            }
            else
            {
                base.Text = "Output(s) Editor";
            }
            this.mode = mode;
            this.pass_name = pass;
        }

        protected override object CreateInstance(Type itemType)
        {
            Classes.Objects.Parameter new_param = new FXED.Classes.Objects.Parameter();
            new_param.Mode = this.mode;
            return new_param;
        }

        public override void ItemChanged(object sender, EventArgs e)
        {
            try
            {
                Classes.Objects.Parameter selected = (Classes.Objects.Parameter)((PropertyGrid)sender).SelectedObject;

                if (Classes.Variables.my_effect.Structs.Find(delegate(Classes.Objects.Struct str) { return str.Name == selected.DataType; }) != null)
                {
                    Classes.Functions.SetBrowsable(selected, "Semantic", false);
                    Classes.Functions.SetBrowsable(selected, "Index", false);
                    Classes.Functions.SetBrowsable(selected, "Uniform", false);
                }
                else
                {
                    Classes.Functions.SetBrowsable(selected, "Uniform", true);

                    if (selected.Uniform)
                    {
                        Classes.Functions.SetBrowsable(selected, "Semantic", false);
                        Classes.Functions.SetBrowsable(selected, "Index", false);
                    }
                    else
                    {
                        Classes.Functions.SetBrowsable(selected, "Semantic", true);
                        Classes.Functions.SetBrowsable(selected, "Index", true);
                    }
                }
            }
            catch (Exception) { return; }
        }

        public override void OKClicked(object sender, EventArgs e)
        {
            Classes.Objects.Pass pass = Classes.Variables.my_effect.Passes.Find(delegate(Classes.Objects.Pass p) { return p.Name == this.pass_name; });
            pass.UpdateVertexUniforms(pass);
            pass.UpdatePixelUniforms(pass);
        }

    }

    [Serializable]
    public class RuntimeCollectionEditor : IWindowsFormsEditorService, IServiceProvider, ITypeDescriptorContext
    {
        private Form mOwner;

        public IList EditCollection(IList collection, CollectionEditor editor, Form owner)
        {
            mOwner = owner;
            IList retval = editor.EditValue(this, this, collection) as IList;
            mOwner = null;
            return retval;
        }

        void IWindowsFormsEditorService.CloseDropDown() { }
        void IWindowsFormsEditorService.DropDownControl(Control control) { }
        DialogResult IWindowsFormsEditorService.ShowDialog(Form dialog)
        {
            return dialog.ShowDialog(mOwner);
        }

        object IServiceProvider.GetService(Type serviceType)
        {
            if (serviceType == typeof(IWindowsFormsEditorService)) return this;
            return null;
        }

        IContainer ITypeDescriptorContext.Container { get { return null; } }
        object ITypeDescriptorContext.Instance { get { return null; } }
        void ITypeDescriptorContext.OnComponentChanged() { }
        bool ITypeDescriptorContext.OnComponentChanging() { return true; }
        PropertyDescriptor ITypeDescriptorContext.PropertyDescriptor { get { return null; } }
    }

    public class SkillExpandableConverter : ExpandableObjectConverter
    {
        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
        {
            Classes.Objects.Skill skill = (Classes.Objects.Skill)value;
            return skill.Value.ToString();
        }

        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            if (destinationType == typeof(Classes.Objects.Skill)) return true;
            return base.CanConvertTo(context, destinationType);
        }


        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(string)) { return true; }
            return base.CanConvertFrom(context, sourceType);
        }

        public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
        {
            try
            {
                Classes.Objects.Skill skill = (Classes.Objects.Skill)Classes.Variables.skills.SkillsGrid.SelectedGridItem.Value;
                float f = 0.0f;
                if(Single.TryParse(value.ToString(),out f))
                {
                skill.Value = Convert.ToSingle(value);
                }
                return skill;
            }
            catch (Exception) { }

            return base.ConvertFrom(context, culture, value);
        }


    }
}
