﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace FXED.Classes
{
    // This class contains public values that may be changed at runtime.

    public abstract class Variables
    {
        /// <summary>
        /// A reference to the main form.
        /// </summary>
        public static Forms.frm_main main_form = null;

        /// <summary>
        /// A reference to the engine preview form.
        /// </summary>
        public static Forms.frm_preview preview = null;

        /// <summary>
        /// A reference to the skins form.
        /// </summary>
        public static Forms.frm_skins skins = null;

        /// <summary>
        /// A reference to the skills form.
        /// </summary>
        public static Forms.frm_skills skills = null;

        /// <summary>
        /// The effect being currently edited.
        /// </summary>
        public static Classes.Objects.Effect my_effect = new FXED.Classes.Objects.Effect();

        /// <summary>
        /// A reference to the current shader being edited.
        /// </summary>
        public static Classes.Objects.Shader current_shader = null;

        /// <summary>
        /// A reference to the current function being edited.
        /// </summary>
        public static Classes.Objects.Function current_function = null;

        /// <summary>
        /// A reference to the current struct being edited.
        /// </summary>
        public static Classes.Objects.Struct current_struct = null;

        /// <summary>
        /// A temporary location to swap parameters.
        /// </summary>
        public static List<Classes.Objects.FunctionParameter> temp_params = null;
        public static bool edit_ok = false;

        /// <summary>
        /// The type of shader being developed (entity/pp).
        /// </summary>
        public static string shader_type = "entity";

        /// <summary>
        /// The maximum times to check for a duplicate name.
        /// </summary>
        public static int copy_check_count = 5000000;

        /// <summary>
        /// A list of objects (variables, vectors, etc) being used for highlighting purposes.
        /// </summary>
        public static List<string> objects = new List<string>();

        /// <summary>
        /// The name of the current effect.
        /// </summary>
        public static string current_name = "New Effect.lcfx";

        /// <summary>
        /// The filename of the current effect if saved.
        /// </summary>
        public static string current_file = string.Empty;

        /// <summary>
        /// The state of the project.
        /// </summary>
        public static bool project_changed = false;

        /// <summary>
        /// A list of reserved words / keywords used in shaders.
        /// </summary>
        public static string[] reserved_words = new string[]
        {
            "if","else","switch","case","break","continue","for","while","do","true","return",
            "false","discard","const","static","in","out","texture","sampler","sample1D","sampler2D","sampler3D","samplerCUBE",
            "technique","pass","null","VertexShader","PixelShader","void","vector","matrix","compile","asm","decl","extern",
            "inline","inout","shared","string","struct","volatile","typedef","uniform",

            "bool","bool2","bool3","bool4","bool1x1","bool1x2","bool1x3","bool1x4","bool2x1","bool2x2",
            "bool2x3","bool2x4","bool3x1","bool3x2","bool3x3","bool3x4","bool4x1","bool4x2",
            "bool4x3","bool4x4",
            
            "int","int2","int3","int4","int1x1","int1x2","int1x3","int1x4","int2x1","int2x2",
            "int2x3","int2x4","int3x1","int3x2","int3x3","int3x4","int4x1","int4x2",
            "int4x3","int4x4",

            "half","half2","half3","half4","half1x1","half1x2","half1x3","half1x4","half2x1","half2x2",
            "half2x3","half2x4","half3x1","half3x2","half3x3","half3x4","half4x1","half4x2",
            "half4x3","half4x4",

            "float","float2","float3","float4","float1x1","float1x2","float1x3","float1x4","float2x1","float2x2",
            "float2x3","float2x4","float3x1","float3x2","float3x3","float3x4","float4x1","float4x2",
            "float4x3","float4x4",

            "double","double2","double3","double4","double1x1","double1x2","double1x3","double1x4","double2x1","double2x2",
            "double2x3","double2x4","double3x1","double3x2","double3x3","double3x4","double4x1","double4x2",
            "double4x3","double4x4"
        };

        /// <summary>
        /// A list of HLSL built-in functions.
        /// </summary>
        public static string[] intrinsic_functions = new string[]
        {
            "abs","acos","all","any","asin","atan","atan2",
            "ceil","clamp","clip","cos","cosh","cross",
            "ddx","ddy","degrees","determinant","distance","dot","D3DCOLORtoUBYTE4",
            "exp","exp2","faceforward","floor","fmod","frac",
            "frexp","fwidth","isinf","isnan","ldexp","length",
            "lerp","lit","log","log10","log2","max",
            "min","modf","mul","noise","normalize","pow",
            "radians","reflect","refract","round","rsqrt","saturate",
            "sign","sin","sincos","sinh","smoothstep","sqrt",
            "step","tan","tanh","tex1D","tex1Dbias","tex1Dgrad",
            "tex1Dlod","tex1Dproj","tex2D","tex2Dbias","tex2Dgrad","tex2Dlod",
            "tex2Dproj","tex3D","tex3Dbias","tex3Dgrad","tex3Dlod","tex3Dproj",
            "texCUBE","texCUBEbias","texCUBEgrad","texCUBElod","texCUBEproj","transpose",
            "trunc",

            // not intrinsic HLSL, but whatever!

            "CreateTangents","CreateViewTangents","DoAmbient","DoColor","DoDefault","DoFog",
            "DoLight","DoLightFactor","DoLightFactorN","DoLightmap","DoNormal","DoPhong",
            "DoPointLight","DoPos","DoReflect","DoShine","DoSpecular","DoSunLight","DoTangent",
            "DoTransform","DoView"
        };

        /// <summary>
        /// A list of predefined components passed by the engine.
        /// </summary>
        public static string[] predefined_components = new string[]
        {
            "fAlpha","fAmbient","fAlbedo","fPower","fLayer","iRepeat",
            "vecAmbient","vecDiffuse","vecSpecular","vecEmissive","vecCycle","vecTime",
            "vecFog","vecFogColor","vecColor","vecLight","vecLightPos","vecLightViewPos",
            "vecLightColor","vecLightDir","vecSunPos","vecSunDir","vecSunColor",
            "vecViewPos","vecViewDir","vecViewPort","vecPortal",
            "matView","matViewInv","matProj","matViewProj","matWorld","matWorldInv","matWorldView",
            "matWorldViewProj","matTexture","matTangent","matMtl","matEffect","matEffect1",
            "matEffect2","matEffect3","matEffect4","matEffect5","matEffect6","matEffect7",
            "matEffect8","entSkin1","entSkin2","entSkin3","entSkin4","mtlSkin1","mtlSkin2",
            "mtlSkin2","mtlSkin4","TargetMap","StencilMap","LightMap"
        };

        /// <summary>
        /// A list of shader semantics.
        /// </summary>
        public static string[] predefined_semantics = new string[]
        {
            "BINORMAL","BINORMAL0","BINORMAL1","BINORMAL2","BINORMAL3","BINORMAL4","BINORMAL5","BINORMAL6","BINORMAL7","BINORMAL8","BINORMAL9","BINORMAL10","BINORMAL11","BINORMAL12","BINORMAL13","BINORMAL14","BINORMAL15",
            "BLENDINDICES","BLENDINDICES0","BLENDINDICES1","BLENDINDICES2","BLENDINDICES3","BLENDINDICES4","BLENDINDICES5","BLENDINDICES6","BLENDINDICES7","BLENDINDICES8","BLENDINDICES9","BLENDINDICES10","BLENDINDICES11","BLENDINDICES12","BLENDINDICES13","BLENDINDICES14","BLENDINDICES15",
            "BLENDWEIGHT","BLENDWEIGHT0","BLENDWEIGHT1","BLENDWEIGHT2","BLENDWEIGHT3","BLENDWEIGHT4","BLENDWEIGHT5","BLENDWEIGHT6","BLENDWEIGHT7","BLENDWEIGHT8","BLENDWEIGHT9","BLENDWEIGHT10","BLENDWEIGHT11","BLENDWEIGHT12","BLENDWEIGHT13","BLENDWEIGHT14","BLENDWEIGHT15",
            "COLOR", "COLOR0","COLOR1","COLOR2","COLOR3","COLOR4","COLOR5","COLOR6","COLOR7","COLOR8","COLOR9","COLOR10","COLOR11","COLOR12","COLOR13","COLOR14","COLOR15",
            "DEPTH", "DEPTH0","DEPTH1","DEPTH2","DEPTH3","DEPTH4","DEPTH5","DEPTH6","DEPTH7","DEPTH8","DEPTH9","DEPTH10","DEPTH11","DEPTH12","DEPTH13","DEPTH14","DEPTH15",
            "FOG", "FOG0","FOG1","FOG2","FOG3","FOG4","FOG5","FOG6","FOG7","FOG8","FOG9","FOG10","FOG11","FOG12","FOG13","FOG14","FOG15",
            "NORMAL", "NORMAL0","NORMAL1","NORMAL2","NORMAL3","NORMAL4","NORMAL5","NORMAL6","NORMAL7","NORMAL8","NORMAL9","NORMAL10","NORMAL11","NORMAL12","NORMAL13","NORMAL14","NORMAL15",
            "POSITION", "POSITION0","POSITION1","POSITION2","POSITION3","POSITION4","POSITION5","POSITION6","POSITION7","POSITION8","POSITION9","POSITION10","POSITION11","POSITION12","POSITION13","POSITION14","POSITION15",
            "POSITIONT", "POSITIONT0","POSITIONT1","POSITIONT2","POSITIONT3","POSITIONT4","POSITIONT5","POSITIONT6","POSITIONT7","POSITIONT8","POSITIONT9","POSITIONT10","POSITIONT11","POSITIONT12","POSITIONT13","POSITIONT14","POSITIONT15",
            "PSIZE", "PSIZE0","PSIZE1","PSIZE2","PSIZE3","PSIZE4","PSIZE5","PSIZE6","PSIZE7","PSIZE8","PSIZE9","PSIZE10","PSIZE11","PSIZE12","PSIZE13","PSIZE14","PSIZE15",
            "TANGENT", "TANGENT0","TANGENT1","TANGENT2","TANGENT3","TANGENT4","TANGENT5","TANGENT6","TANGENT7","TANGENT8","TANGENT9","TANGENT10","TANGENT11","TANGENT12","TANGENT13","TANGENT14","TANGENT15",
            "TESSFACTOR","TESSFACTOR0","TESSFACTOR1","TESSFACTOR2","TESSFACTOR3","TESSFACTOR4","TESSFACTOR5","TESSFACTOR6","TESSFACTOR7","TESSFACTOR8","TESSFACTOR9","TESSFACTOR10","TESSFACTOR11","TESSFACTOR12","TESSFACTOR13","TESSFACTOR14","TESSFACTOR15",
            "TEXCOORD", "TEXCOORD0","TEXCOORD1","TEXCOORD2","TEXCOORD3","TEXCOORD4","TEXCOORD5","TEXCOORD6","TEXCOORD7","TEXCOORD8","TEXCOORD9","TEXCOORD10","TEXCOORD11","TEXCOORD12","TEXCOORD13","TEXCOORD14","TEXCOORD15",
            "VFACE", "VFACE0","VFACE1","VFACE2","VFACE3","VFACE4","VFACE5","VFACE6","VFACE7","VFACE8","VFACE9","VFACE10","VFACE11","VFACE12","VFACE13","VFACE14","VFACE15",
            "VPOS", "VPOS0","VPOS1","VPOS2","VPOS3","VPOS4","VPOS5","VPOS6","VPOS7","VPOS8","VPOS9","VPOS10","VPOS11","VPOS12","VPOS13","VPOS14","VPOS15"
        };
    }
}
