﻿using System;
using System.Collections;
using System.ComponentModel;
using System.IO;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

namespace FXED.CustomControls
{
    public class DynamicPropertyGrid : PropertyGrid
    {
        private System.ComponentModel.Container components = null;

        public DynamicPropertyGrid() { }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        protected override PropertyTab CreatePropertyTab(Type tabType)
        {
            return new Tab();
        }
    }

    public class Tab : PropertyTab
    {
        public Tab()
        {
        }

        // get the properties of the selected component
        public override System.ComponentModel.PropertyDescriptorCollection GetProperties(object component, System.Attribute[] attributes)
        {
            PropertyDescriptorCollection properties;
            if (attributes != null)
                properties = TypeDescriptor.GetProperties(component, attributes);
            else
                properties = TypeDescriptor.GetProperties(component);

            //Component must implement the dynamic interface.
            try
            {
                Classes.PropertySupport.IDynamicClass bclass = (Classes.PropertySupport.IDynamicClass)component;

                //The new array of properties, based on 'PublicProperties'.
                PropertyDescriptor[] arrProp = new PropertyDescriptor[bclass.PublicProperties.Count];

                for (int i = 0; i < bclass.PublicProperties.Count; i++)
                {
                    //Find the properties in the array with matching names
                    PropertyDescriptor prop = properties.Find(bclass.PublicProperties[i].Name, true);
                    //Build new properties
                    arrProp[i] = TypeDescriptor.CreateProperty(prop.ComponentType, prop, new CategoryAttribute(bclass.PublicProperties[i].Category));
                }
                return new PropertyDescriptorCollection(arrProp);
            }
            catch (Exception) { return null; }
        }

        public override System.ComponentModel.PropertyDescriptorCollection GetProperties(object component)
        {
            return this.GetProperties(component, null);
        }

        // PropertyTab Name
        public override string TabName
        {
            get
            {
                return "Properties";
            }
        }

        //Image of the property tab (return a blank 16x16 Bitmap)
        public override System.Drawing.Bitmap Bitmap
        {
            get
            {
                return new Bitmap(16, 16); ;
            }
        }
    }
}
