﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace FXED.Forms
{
    public partial class frm_fx_editor : Form
    {
        public frm_fx_editor(string filename)
        {
            InitializeComponent();

            FileStream open_stream = new FileStream(filename, FileMode.Open, FileAccess.Read);
            StreamReader reader = new StreamReader(open_stream);

            Classes.Variables.my_effect.FxCode = reader.ReadToEnd();
            reader.Close();
            open_stream.Close();

            stx_editor.PerformHighlighting();
            stx_editor.TextChanged += new EventHandler(UpdateCode);
            stx_editor.Text = Classes.Variables.my_effect.FxCode;
        }

        #region Text Operations

        private void CutText(object sender, EventArgs e)
        {
            stx_editor.Cut();
        }

        private void CopyText(object sender, EventArgs e)
        {
            stx_editor.Copy();
        }

        private void PasteText(object sender, EventArgs e)
        {
            stx_editor.Paste();
        }

        private void SelectAllText(object sender, EventArgs e)
        {
            stx_editor.SelectAll();
        }

        private void DeleteText(object sender, EventArgs e)
        {
            stx_editor.SelectedText = string.Empty;
        }

        private void UndoText(object sender, EventArgs e)
        {
            stx_editor.Undo();
        }

        private void RedoText(object sender, EventArgs e)
        {
            stx_editor.Redo();
        }

        private void InsertFunction(object sender, EventArgs e)
        {
            ToolStripMenuItem item = sender as ToolStripMenuItem;
            if (stx_editor.SelectedText != string.Empty)
            {
                stx_editor.SelectedText = item.Tag.ToString();
            }
            else
            {
                stx_editor.Text = stx_editor.Text.Insert(stx_editor.SelectionStart, item.Tag.ToString());
            }
        }

        private void ShortcutPressed(object sender, KeyEventArgs e)
        {
            if (e.Control)
            {
                switch (e.KeyCode)
                {
                    case Keys.Y: stx_editor.Redo(); break;
                    case Keys.Z: stx_editor.Undo(); break;
                }
            }
        }

        private void CommentText(object sender, EventArgs e)
        {
            PrependToLine("//");
        }

        private void IndentText(object sender, EventArgs e)
        {
            PrependToLine("\t");
        }

        private void PrependToLine(string value)
        {
            int start = stx_editor.SelectionStart;
            int end = stx_editor.SelectionStart + stx_editor.SelectionLength - 1;
            bool zeroWidth = (stx_editor.SelectionLength == 0);
            int pos = FindStartOfFirstLine();
            bool selectionStartsAtBOL = start == pos;
            while (pos <= end || zeroWidth)
            {
                stx_editor.SelectionStart = pos;
                stx_editor.SelectionLength = 0;
                stx_editor.SelectedText = value;
                end += value.Length;
                while (pos <= end && stx_editor.Text[pos++] != '\n') { }
                if (zeroWidth) { break; }
            }
            if (!selectionStartsAtBOL || zeroWidth)
            {
                start += value.Length;
            }

            stx_editor.Select(start, end - start + 1);
        }

        private int FindStartOfFirstLine()
        {
            int pos = stx_editor.SelectionStart - 1;

            while (pos > 0)
            {
                if (stx_editor.Text[pos] == '\n') { break; }
                pos--;
            }

            if (pos != 0 || stx_editor.Text[pos] == '\n') { pos++; }

            return pos;
        }

        private void RemovePrefixFromLineRange(bool isComment, params string[] prefixList)
        {
            int rememberStart = stx_editor.SelectionStart;
            int endPos = stx_editor.SelectionStart + System.Math.Max(0, stx_editor.SelectionLength - 1);
            bool zeroWidth = (stx_editor.SelectionLength == 0);
            int pos = FindStartOfFirstLine();
            bool selectionStartsAtBOL = rememberStart == pos;
            string foundPrefix = "";
            bool cursorStartsBeforePrefix = false;

            while ((pos < stx_editor.TextLength) && (pos <= endPos))
            {
                if (isComment)
                {
                    while (pos < stx_editor.TextLength && stx_editor.Text[pos] != '\n' && Char.IsWhiteSpace(stx_editor.Text[pos]))
                    { pos++; }
                }

                cursorStartsBeforePrefix |= (rememberStart < pos);

                foreach (var prefix in prefixList)
                {
                    if (DeletePrefix(prefix, pos, ref endPos)) { foundPrefix = prefix; break; }
                }
                while (pos <= endPos && pos < stx_editor.TextLength && stx_editor.Text[pos++] != '\n') { }

            }

            if (!cursorStartsBeforePrefix)
            { rememberStart = System.Math.Max(0, rememberStart - foundPrefix.Length); }

            stx_editor.SelectionStart = rememberStart;
            if (!zeroWidth)
            { stx_editor.Select(rememberStart, endPos - rememberStart + 1); }
        }

        private bool DeletePrefix(string prefix, int pos, ref int endPos)
        {
            stx_editor.Select(pos, prefix.Length);
            if (stx_editor.SelectedText.Equals(prefix))
            {
                stx_editor.SelectedText = "";
                endPos -= prefix.Length;
                return true;
            }
            return false;
        }

        private void OutdentText(object sender, EventArgs e)
        {
            RemovePrefixFromLineRange(false, new string[] { "\t", "    " });
        }

        private void UncommentText(object sender, EventArgs e)
        {
            RemovePrefixFromLineRange(true, new string[] { "//", "//\t", "//" });
        }

        private void UpdateCode(object sender, EventArgs e)
        {
            stx_editor.PerformHighlighting();
            Classes.Variables.my_effect.FxCode = stx_editor.Text;
            Classes.Functions.ProjectChanged();
        }

        #endregion
        
    }
}
