﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace FXED.Forms
{
    public partial class frm_shader : Form
    {
        Classes.Objects.Shader my_shader = null;
        public Classes.Objects.Shader MyShader
        {
            get { return my_shader; }
        }

        public frm_shader(Classes.Objects.Shader shader)
        {
            this.my_shader = shader;

            InitializeComponent();
        }

        private void frm_shader_Load(object sender, EventArgs e)
        {
            this.Location = my_shader.Location;
            this.Size = my_shader.Size;
            this.Text = my_shader.Type + " Shader (" + this.Tag.ToString() + ")";
            cmb_models.Text = my_shader.Model;
            this.Closing += new CancelEventHandler(frm_shader_Closing);
            this.LocationChanged += new EventHandler(FormMoved);
            this.ResizeEnd += new EventHandler(FormResized);
            this.cmb_models.SelectedIndexChanged+=new EventHandler(SetShaderModel);

            if (my_shader.Type == "Vertex")
            {
                cmb_models.Items.AddRange(new string[] { "1.1", "2.0", "2.0a", "3.0" });
            }
            else
            {
                cmb_models.Items.AddRange(new string[] { "1.0", "1.1", "1.3", "1.4", "2.0", "2.0a", "2.0b", "3.0" });
            }

            UpdateObjects();
            UpdateParameters();

            rtb_code.Font = Properties.Settings.Default.EditorFont;
            rtb_code.Text = my_shader.Code;
            rtb_code.PerformHighlighting();
            rtb_code.TextChanged += new EventHandler(UpdateCode);       
        } 

        public CustomControls.SyntaxEditor Editor
        {
            get { return rtb_code; }
        }

        public void UpdateObjects()
        {
            rtb_code.ObjectDescriptors.Clear();
            foreach (string obj in Classes.Variables.objects)
            {
                rtb_code.AddObject(obj);
            }
            rtb_code.PerformHighlighting();
        }

        public void UpdateParameters()
        {
            rtb_code.ParameterDescriptors.Clear();

            foreach (Classes.Objects.Parameter param in my_shader.Inputs)
            {
                rtb_code.AddParameter(param.Name);
            }

            foreach (Classes.Objects.Parameter param in my_shader.Outputs)
            {
                rtb_code.AddParameter(param.Name);
            }

            rtb_code.PerformHighlighting();
        }

        #region Text Operations

        private void CutText(object sender, EventArgs e)
        {
            rtb_code.Cut();
        }

        private void CopyText(object sender, EventArgs e)
        {
            rtb_code.Copy();
        }

        private void PasteText(object sender, EventArgs e)
        {
            rtb_code.Paste();
        }

        private void SelectAllText(object sender, EventArgs e)
        {
            rtb_code.SelectAll();
        }

        private void DeleteText(object sender, EventArgs e)
        {
            rtb_code.SelectedText = string.Empty;
        }

        private void UndoText(object sender, EventArgs e)
        {
            rtb_code.Undo();
        }

        private void RedoText(object sender, EventArgs e)
        {
            rtb_code.Redo();
        }

        private void UpdateCode(object sender, EventArgs e)
        {
            rtb_code.PerformHighlighting();
            my_shader.Code = rtb_code.Text;
        }

        private void InsertFunction(object sender, EventArgs e)
        {
            ToolStripMenuItem item = sender as ToolStripMenuItem;
            if (rtb_code.SelectedText != string.Empty)
            {
                rtb_code.SelectedText = item.Tag.ToString();
            }
            else
            {
                rtb_code.Text = rtb_code.Text.Insert(rtb_code.SelectionStart, item.Tag.ToString());
            }
        }

        private void ShortcutPressed(object sender, KeyEventArgs e)
        {
            if (e.Control)
            {
                switch (e.KeyCode)
                {
                    case Keys.Y: rtb_code.Redo(); break;
                    case Keys.Z: rtb_code.Undo(); break;
                }
            }
        }

        private void CommentText(object sender, EventArgs e)
        {
            PrependToLine("//");
        }

        private void IndentText(object sender, EventArgs e)
        {
            PrependToLine("\t");
        }

        private void PrependToLine(string value)
        {
            int start = rtb_code.SelectionStart;
            int end = rtb_code.SelectionStart + rtb_code.SelectionLength - 1;
            bool zeroWidth = (rtb_code.SelectionLength == 0);
            int pos = FindStartOfFirstLine();
            bool selectionStartsAtBOL = start == pos;
            while (pos <= end || zeroWidth)
            {
                rtb_code.SelectionStart = pos;
                rtb_code.SelectionLength = 0;
                rtb_code.SelectedText = value;
                end += value.Length;
                while (pos <= end && rtb_code.Text[pos++] != '\n') { }
                if (zeroWidth) { break; }
            }
            if (!selectionStartsAtBOL || zeroWidth)
            {
                start += value.Length;
            }

            rtb_code.Select(start, end - start + 1);
        }

        private int FindStartOfFirstLine()
        {
            int pos = rtb_code.SelectionStart - 1;

            while (pos > 0)
            {
                if (rtb_code.Text[pos] == '\n') { break; }
                pos--;
            }

            if (pos != 0 || rtb_code.Text[pos] == '\n') { pos++; }

            return pos;
        }

        private void RemovePrefixFromLineRange(bool isComment, params string[] prefixList)
        {
            int rememberStart = rtb_code.SelectionStart;
            int endPos = rtb_code.SelectionStart + System.Math.Max(0, rtb_code.SelectionLength - 1);
            bool zeroWidth = (rtb_code.SelectionLength == 0);
            int pos = FindStartOfFirstLine();
            bool selectionStartsAtBOL = rememberStart == pos;
            string foundPrefix = "";
            bool cursorStartsBeforePrefix = false;

            while ((pos < rtb_code.TextLength) && (pos <= endPos))
            {
                if (isComment)
                {
                    while (pos < rtb_code.TextLength && rtb_code.Text[pos] != '\n' && Char.IsWhiteSpace(rtb_code.Text[pos]))
                    { pos++; }
                }

                cursorStartsBeforePrefix |= (rememberStart < pos);

                foreach (var prefix in prefixList)
                {
                    if (DeletePrefix(prefix, pos, ref endPos)) { foundPrefix = prefix; break; }
                }
                while (pos <= endPos && pos < rtb_code.TextLength && rtb_code.Text[pos++] != '\n') { }

            }

            if (!cursorStartsBeforePrefix)
            { rememberStart = System.Math.Max(0, rememberStart - foundPrefix.Length); }

            rtb_code.SelectionStart = rememberStart;
            if (!zeroWidth)
            { rtb_code.Select(rememberStart, endPos - rememberStart + 1); }
        }

        private bool DeletePrefix(string prefix, int pos, ref int endPos)
        {
            rtb_code.Select(pos, prefix.Length);
            if (rtb_code.SelectedText.Equals(prefix))
            {
                rtb_code.SelectedText = "";
                endPos -= prefix.Length;
                return true;
            }
            return false;
        }

        private void OutdentText(object sender, EventArgs e)
        {
            RemovePrefixFromLineRange(false, new string[] { "\t", "    " });
        }

        private void UncommentText(object sender, EventArgs e)
        {
            RemovePrefixFromLineRange(true, new string[] { "//", "//\t", "//" });
        }

        #endregion

        #region Form Functions

        private void FormMoved(object sender, EventArgs e)
        {
            my_shader.Location = this.Location;
        }

        private void FormResized(object sender, EventArgs e)
        {
            my_shader.Size = this.Size;
        }

        void frm_shader_Closing(object sender, CancelEventArgs e)
        {
            my_shader.Visible = false;
        }

        #endregion

        private void EditInputs(object sender, EventArgs e)
        {
            Classes.Variables.current_shader = my_shader;
            Classes.Helper.RuntimeCollectionEditor inputs_collection = new Classes.Helper.RuntimeCollectionEditor();
            inputs_collection.EditCollection(my_shader.Inputs,new Classes.Helper.ShaderParameterCollection(my_shader.Inputs.GetType(),"in",this.Tag.ToString()),this);
            UpdateParameters();
        }

        private void EditOutputs(object sender, EventArgs e)
        {
            Classes.Variables.current_shader = my_shader;
            Classes.Helper.RuntimeCollectionEditor outputs_collection = new Classes.Helper.RuntimeCollectionEditor();
            outputs_collection.EditCollection(my_shader.Outputs, new Classes.Helper.ShaderParameterCollection(my_shader.Outputs.GetType(), "out", this.Tag.ToString()), this);
            UpdateParameters();
        }

        private void SetShaderModel(object sender, EventArgs e)
        {
            my_shader.Model = this.cmb_models.SelectedItem.ToString();
        }


    }
}
